﻿using CimSyntaxGen.Utilities;
using EA;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using static System.Net.Mime.MediaTypeNames;

namespace CimSyntaxGen.AsciiDoc
{
    public class AsciiDocESMPDocumentation
    {
        private static Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);
        private bool iec = false;
        private EA.Repository repo;
        private XMLParser XMLP;
        private EA.Package selectedPackage;
        private List<string> subprofilesStereos; // list of non CIM stereotypes
        private Dictionary<long, string> dicPackim = new Dictionary<long, string>();
        private StringBuilder AbstractsBuffer = new StringBuilder();
        private StringWriter sw;
        private Dictionary<string, List<long>> TypeList = new Dictionary<string, List<long>>(); // string, elementID
        private AsciiDocGenerator asciiDocGenerator;
        private List<string> notPureClasses = new List<string>();  // to be used by find predicates
        private ConstantDefinition CD = new ConstantDefinition();
        private List<string> notClasseStereos; // sterotypes of objects different from classes (ex: enumeration or datattype)
        private string subProfileStereotype;  // to be used by finding predicates
        private List<long> alreadyFound = new List<long>(); // list of classes already processed (avoids duplicate processing of multiple stereotypes)
        public bool is351 = false; // in the special case of IEC62325-351, all primitives must be visible
        public List<EA.Element> listPrimitive = new List<Element>(); // List of primitives of 351

        public AsciiDocESMPDocumentation(EA.Repository repo, bool forIEC, EA.Package selectedPkg)
        {
            this.repo = repo;
            this.iec = forIEC;
            this.XMLP = new XMLParser(repo);
            this.selectedPackage = selectedPkg;

            this.subprofilesStereos = GetSubprofilesStereos();

            sw = new StringWriter(AbstractsBuffer);

            TypeList.Add("datatype", new List<long>());
            TypeList.Add("enumeration", new List<long>());
            TypeList.Add("compound", new List<long>());

            this.notPureClasses.Add(CD.GetEnumStereotype());
            this.notPureClasses.Add("Datatype");
            this.notPureClasses.Add(CD.GetDatatypeStereotype());
            this.notPureClasses.Add(CD.GetPrimitiveStereotype());
            this.notPureClasses.Add(CD.GetCompoundStereotype());

            this.notClasseStereos = new List<string>(this.notPureClasses);
            this.notPureClasses.AddRange(this.subprofilesStereos);
            asciiDocGenerator = new AsciiDocGenerator(this.repo);
        }

        public static string GenerateCoverIntro(EA.Repository repos, EA.Package pack, string outputDir)
        {
            AsciiDocGenerator asciiDocGenerator = new AsciiDocGenerator(repos);
            asciiDocGenerator.SaveCoverpageImage(outputDir);
            return asciiDocGenerator.GetCoverIntroAsciiDoc(pack.Name);
        }


        public List<EA.Element> GetListPrimitive()
        {
            return this.listPrimitive;
        }

        public void SetListPrimitive(List<EA.Element> listPrimitive)
        {
            this.listPrimitive = listPrimitive;
        }

        public void BuildListPrimitive(EA.Package pack)
        {
            List<EA.Element> ret = new List<Element>();
            RDFutilitaires.getAllElements(pack, ret, "Primitive");
            this.listPrimitive = ret;
        }


        public Dictionary<string, List<long>> GetTypelist()
        {
            return this.TypeList;
        }

        private List<string> GetSubprofilesStereos()
        {
            List<string> subprofilesStereos = new List<string>();
            if (XMLP.GetXmlValueConfig("StereotypeNameSpaces") == "Checked")
            {
                string listStereos = XMLP.GetXmlValueProfData("ListStereoNamespace");
                if (listStereos != null && listStereos.Length > 0)
                {
                    string[] stereos = listStereos.Split(new char[] { '|' });
                    subprofilesStereos = new List<string>(stereos);
                }
            }

            subprofilesStereos.Sort(RDFutilitaires.CompareStringsByLength);
            subprofilesStereos.Reverse();
            return subprofilesStereos;
        }

        public string GenerateDocu(string outputDir)
        {
            StringBuilder sb = new StringBuilder();
            asciiDocGenerator.SaveCoverpageImage(outputDir);

            List<EA.Element> profileClasses = new List<EA.Element>();
            RDFutilitaires.getTypedElements(this.selectedPackage, profileClasses, "Class");
            profileClasses.Sort(RDFutilitaires.CompareElementsbyName);

            /////// header
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();
            sb.Append(asciiDocStyle.GetHeader(this.selectedPackage.Name, 0));
            sb.Append(asciiDocStyle.GetHeader("Overview of the model", 1));
            sb.Append("The following diagram shows the model:\n");

            /////// image
            asciiDocGenerator = new AsciiDocGenerator(this.repo);
            string diagramFileName = asciiDocGenerator.SaveDiagram(this.selectedPackage, outputDir);
            AsciiDocImage asciiDocImage = new AsciiDocImage(this.selectedPackage.Name, diagramFileName, 1, -1, -1);
            sb.Append(asciiDocImage.GetImage());

            ////// table for isBasedOn
            sb.Append(asciiDocStyle.GetHeader("IsBasedOn relationships from the European style market profile:", 1));
            sb.Append("The following table shows the traceability dependency of the classes used in this package towards the upper level." + AsciiDocGenerator.LBR + AsciiDocGenerator.LBR);
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - IsBasedOn Dependency");
            asciiDocTable.SetHeaderRow(new List<string>() { "Name", "Complete IsBasedOn Path" });
            foreach (EA.Element elt in profileClasses)
            {
                EA.Element basedOnElem = RDFutilitaires.getElementOriginBasedonElement(elt);
                asciiDocTable.AddRow(new List<string>() { elt.Name, asciiDocGenerator.GetPackageContainmentPath(basedOnElem) + "::" + basedOnElem.Name });
            }
            sb.Append("\n");
            sb.Append(asciiDocTable.GetTable("1, 4", 100));

            ////// table for Classes
            List<EA.Element> classlists;
            for (int ii = 0; ii < subprofilesStereos.Count; ii++)
            {
                string stereo = subprofilesStereos[ii];
                classlists = new List<EA.Element>();
                subProfileStereotype = stereo;
                classlists = profileClasses.FindAll(HasClassStereotype);
                classlists.Sort(RDFutilitaires.CompareElementsbyName);
                if (classlists.Count > 0)
                {
                    sb.Append(asciiDocStyle.GetHeader("Concrete Classes" + " (" + stereo + ")" + AsciiDocGenerator.LBR, 1));
                    foreach (EA.Element elt in classlists)
                    {
                        //EA.Package pack = repo.GetPackageByID(elt.PackageID);
                        string asciiDocString = GetAsciiDocClass(this.selectedPackage, elt, false);
                        sb.Append(asciiDocString);
                    }
                }
            }

            ////// Concrete Classes
            classlists = new List<EA.Element>();
            classlists = profileClasses.FindAll(HasNotClassStereotype);
            List<EA.Element> rootlist = new List<EA.Element>();
            List<long> todelete = new List<long>();
            foreach (EA.Element elt in classlists)
            {
                if (elt.IsActive)
                {
                    rootlist.Add(elt);
                }
            }

            foreach (EA.Element el in rootlist)
            {
                classlists.Remove(el);
            }

            classlists.Sort(RDFutilitaires.CompareElementsbyName);
            sb.Append(asciiDocStyle.GetHeader("Detailed" + " " + this.selectedPackage.Name, 3));
            foreach (EA.Element elt in rootlist)
            {
                string asciiDocString = GetAsciiDocClass(this.selectedPackage, elt, true);
                sb.Append(asciiDocString);
            }

            foreach (EA.Element elt in classlists)
            {
                string asciiDocString = GetAsciiDocClass(this.selectedPackage, elt, false);
                sb.Append(asciiDocString);
            }

            /////// Datatypes
            if (!this.selectedPackage.Name.Contains("contextual") && !this.selectedPackage.Name.Contains("Contextual"))
            {
                foreach (EA.Element elt in profileClasses)
                {
                    string elttype = GetTypeElt(elt);
                    if (TypeList.ContainsKey(elttype))
                    {
                        if (!TypeList[elttype].Contains(elt.ElementID)) TypeList[elttype].Add(elt.ElementID);
                    }
                }

                if (is351)
                { 
                    // Primitive types types
                    if (!iec)
                    {
                        sb.Append(asciiDocStyle.GetHeader("Primitives", 3));
                    }
                    foreach (EA.Element prim in listPrimitive)
                    {
                        sb.Append(GetPrimitiveTypeAsciiDoc(prim));
                    }
                }

                /////// Compound types
                string COMPOUND = "compound";
                if (!iec)
                {
                    sb.Append(asciiDocStyle.GetHeader("Datatypes", 3));
                }
                if (TypeList[COMPOUND].Count > 0)
                {
                    TypeList[COMPOUND].Sort(RDFutilitaires.CompareElementIDsbyName); // alphabetical order
                }
                if (!iec)
                {
                    CreateCompoundList(); // explore the list and add compounds called by other compounds
                }
                this.TypeList[COMPOUND].Sort(RDFutilitaires.CompareElementIDsbyName);
                foreach (long comp in this.TypeList[COMPOUND])
                {
                    sb.Append(GetCompoundtypeAsciiDoc(Main.Repo.GetElementByID((int)comp)));
                }
                string DATATYPE = "datatype";
                if (this.TypeList[DATATYPE].Count > 0)
                {
                    List<string> l1 = new List<string>();
                    List<string> lsor = new List<string>();
                    foreach (long ii in this.TypeList[DATATYPE])
                    {
                        lsor.Add((repo.GetElementByID((int)ii)).Name);
                    }

                    this.TypeList[DATATYPE].Sort(RDFutilitaires.CompareElementIDsbyName);
                    List<string> lsora = new List<string>();
                    foreach (long id in this.TypeList[DATATYPE])
                    {
                        lsora.Add(repo.GetElementByID((int)id).Name);
                    }
                }
                if (!iec)
                {
                    foreach (long data in TypeList[DATATYPE])
                    {
                        sb.Append(GetDatatypeAsciiDoc(Main.Repo.GetElementByID((int)data)));
                    }
                }

                //////// Enumerations
                if (!iec)
                {
                    sb.Append(asciiDocStyle.GetHeader("Enumerations", 3));
                }
                if (TypeList["enumeration"].Count > 0)
                {
                    string rootclassname = "";
                    if ((rootlist != null) && (rootlist.Count > 0))
                    {
                        rootclassname = rootlist[0].Name; // nom de la rootclass
                    }
                    sb.Append("The list of enumerations used for the " + this.selectedPackage.Name + " is as follows:");
                    this.TypeList["enumeration"].Sort(RDFutilitaires.CompareElementIDsbyName);
                }
                List<string> typelists = new List<string>();

                foreach (long kind in this.TypeList["enumeration"])
                {
                    string nom = this.repo.GetElementByID((int)kind).Name;
                    if (!iec)
                    {
                        if ((!nom.Contains("TypeList"))
                            && (!(nom == "UnitMultiplier"))
                            && (!(nom == "UnitSymbol"))
                            )
                        {
                            sb.Append(GetEnumerationAsciiDoc(this.repo.GetElementByID((int)kind)));
                        }
                        else
                        {
                            if (!typelists.Contains(nom)) typelists.Add(nom);
                        }
                    }
                }
                if (typelists.Count > 0)
                {   // there are codelist enumerations to be displayed as name lists
                    foreach (string n in typelists)
                    {
                        sb.Append("* " + n + "\n");
                    }
                }
            }
            return sb.ToString();
        }

        private bool HasNotClassStereotype(EA.Element elt)
        {
            bool ret = true;
            if (elt.StereotypeEx == "") ret = true;  // obvious
            foreach (string stereo in this.notPureClasses)
            {
                if (elt.StereotypeEx.Contains(stereo))
                {
                    ret = false;
                    break;
                }
            }
            return ret;
        }


        private string GetEnumerationAsciiDoc(EA.Element enumelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(enumelt.Name + " enumeration", 4));
            sb.Append(enumelt.Notes.Replace("\n", AsciiDocGenerator.LBR) + AsciiDocGenerator.LBR);

            if (enumelt.Name.Contains("TypeList"))
            {
                sb.Append("The following table shows examples of literals for " + enumelt.Name + "." + AsciiDocGenerator.LBR);
            }
            else
            {
                sb.Append("The following table shows all literals of " + enumelt.Name + "." + AsciiDocGenerator.LBR);
            }
            string packName = this.repo.GetPackageByID(enumelt.PackageID).Name;
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Literals of " + packName + "::" + enumelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Literal", "Code", "Description" });

            foreach (EA.Attribute attr in enumelt.Attributes)
            {
                asciiDocTable.AddRow(new List<string>() { attr.Name,
                                                         GetEnumCode(attr),
                                                         attr.Notes.Replace("\n", AsciiDocGenerator.LBR)});
            }
            sb.Append(asciiDocTable.GetTable("2, 1, 4", 100));
            return sb.ToString();
        }


        private string GetEnumCode(EA.Attribute at)
        {
            string resul = "";
            foreach (EA.AttributeTag tag in at.TaggedValues)
            {
                if (tag.Name == CD.GetEnumCodeTag())
                {
                    resul = tag.Value;
                }
            }
            return resul;
        }


        private string GetDatatypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " datatype", 4));
            sb.Append(dataelt.Notes.Replace("\n", AsciiDocGenerator.LBR) + AsciiDocGenerator.LBR);
            string packName = this.repo.GetPackageByID(dataelt.PackageID).Name;

            if (dataelt.Connectors.Count > 0)
            {
                foreach (EA.Connector con in dataelt.Connectors)
                {
                    if (con.Type == "NoteLink")
                    {
                        sb.Append(asciiDocGenerator.GenerateNoteAsciiDoc(dataelt, con, "0"));
                    }
                }
            }

            sb.Append("The following table shows all attributes of " + dataelt.Name + "." + AsciiDocGenerator.LBR);
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });

            bool thereAreConstraints = false;
            foreach (EA.Attribute attr in dataelt.Attributes)
            {
                string type = GetAttrType(attr);
                asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", AsciiDocGenerator.LBR)});

                if (attr.Constraints.Count > 0) thereAreConstraints = true;
            }

            if (!thereAreConstraints) // there are no constraints
            {
                sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));
                return sb.ToString();
            }

            sb.Append(GetAsciiDocWithContraintsTable(packName, dataelt));
            return sb.ToString();
        }


        private string GetAsciiDocWithContraintsTable(string packName, EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("The following table shows all restrictions applied to the attributes  of " + dataelt.Name + "." + AsciiDocGenerator.LBR);
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Restrictions of attributes for " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Name", "Constraint", "Type", "Expression of constraint" });

            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if (at.IsConst)
                {
                    asciiDocTable.AddRow(new List<string>() { at.Name, "Constant", "", at.Default.ToString() });
                }

                if (at.IsStatic)
                {
                    asciiDocTable.AddRow(new List<string>() { at.Name, "Fixed", "", at.Default.ToString() });
                }

                if (at.Constraints.Count > 0)
                {
                    foreach (EA.AttributeConstraint ac in at.Constraints)
                    {
                        string constrNotes = ac.Notes;
                        string constraint = "";
                        if (ac.Type == "OCL")
                        {
                            int pos1 = constrNotes.IndexOf("->");
                            int pos2 = constrNotes.IndexOf('(');
                            int pos3 = constrNotes.IndexOf(')');
                            constraint = constrNotes.Substring(pos1 + 1, pos3 - pos1);
                        }
                        else
                        {
                            if (ac.Type.Equals("INV") || ac.Type.Equals("Invariant"))
                            {
                                constraint = ">" + constrNotes;
                            }

                        }
                        if (constraint.Length > 0)
                        {
                            asciiDocTable.AddRow(new List<string>() { at.Name, ac.Name, ac.Type, constraint });
                        }
                    }
                }
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 1, 3", 100));
            return sb.ToString() + AsciiDocGenerator.LBR + AsciiDocGenerator.LBR;
        }




        private string GetCompoundtypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " compound", 4));
            sb.Append(dataelt.Notes.Replace("\n", AsciiDocGenerator.LBR) + AsciiDocGenerator.LBR);
            sb.Append("The following table shows all attributes of " + dataelt.Name + "." + AsciiDocGenerator.LBR);
            string packName = this.repo.GetPackageByID(dataelt.PackageID).Name;
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });

            int ind = 0;
            Dictionary<int, int> dicOrder = new Dictionary<int, int>();
            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if (!dicOrder.ContainsKey(at.Pos)) dicOrder.Add(at.Pos, ind);
                ind++;
            }
            List<int> positions = new List<int>(); // The positions are copied from the CIM element 
                                                   // they are not necessarily in ascending order from 0

            for (short i = 0; i < dataelt.Attributes.Count; i++)
            {
                EA.Attribute atr = (EA.Attribute)dataelt.Attributes.GetAt(i);
                if (!positions.Contains((atr.Pos)))
                {
                    positions.Add(atr.Pos);
                }
            }
            foreach (int pos in positions)
            {
                if (dicOrder[pos] < dataelt.Attributes.Count)
                {
                    EA.Attribute attr = (EA.Attribute)dataelt.Attributes.GetAt((short)dicOrder[pos]);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", AsciiDocGenerator.LBR)});
                }
                else
                {
                    MessageBox.Show("Error the attribute position "
                        + dicOrder[pos].ToString() + " of " + dataelt.Name + " exceeds the maximumum index!");
                }
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));
            return sb.ToString();
        }


        /*
         * The GetAttributesTable method updates the lists of datatypes as it goes along
         * only compounds can have type attributes which are themselves compounds
         * the typelist[compound] cannot be explored recursively.
         * This subroutine is designed to explore all compounds (and only compounds)
         * recursively to update TypeList[compound].
         */
        private void CreateCompoundList()
        {
            string COMPOUND = "compound";
            int nbcompound = this.TypeList[COMPOUND].Count;
            List<long> locallist = new List<long>(this.TypeList[COMPOUND]);

            foreach (long elid in locallist)
            {
                EA.Element elt = this.repo.GetElementByID((int)elid);
                foreach (EA.Attribute attr in elt.Attributes)
                {
                    if (attr.ClassifierID != 0)
                    {
                        EA.Element classElt = Main.Repo.GetElementByID((int)attr.ClassifierID);
                        if (
                            (classElt.StereotypeEx.Contains(CD.GetCompoundStereotype()))
                            && (!this.TypeList[COMPOUND].Contains(classElt.ElementID))
                            )
                        {
                            this.TypeList[COMPOUND].Add(classElt.ElementID);
                        }
                    }
                }

            }
            if (TypeList[COMPOUND].Count != nbcompound)
            {
                CreateCompoundList(); // successive cascades must be taken into account, hence the recursion
            }
        }

        private string GetPrimitiveTypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            if (dataelt.Name != "")
            {
                sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " primitive", 4));
            }
            sb.Append(dataelt.Notes.Replace("\n", AsciiDocGenerator.LBR));
            sb.Append(AsciiDocGenerator.LBR);
            return sb.ToString();
        }



        private bool HasClassStereotype(EA.Element elt)
        {
            bool ret = false; // true if element 

            if (subProfileStereotype.Equals("Class") &&
                (elt.StereotypeEx.Equals("") || elt.StereotypeEx.Contains("Class")))
            {
                ret = true;
            }

            if ((!subProfileStereotype.Contains(",")) &&
                elt.StereotypeEx.Contains(subProfileStereotype))
            {
                ret = true;
                foreach (string stereo in this.notClasseStereos)
                {
                    if (elt.StereotypeEx.Contains(stereo)) return false;
                }
            }
            else
            {
                ArrayList subprofslist = new ArrayList(subProfileStereotype.Split(new char[] { ',' }));
                RDFutilitaires.setper(subprofslist);
                for (int j = 0; j < RDFutilitaires.resulpermutation.Count; j++)
                {
                    string res = "";
                    for (int i = 0; i < ((ArrayList)RDFutilitaires.resulpermutation[j]).Count; i++)
                    {
                        res = res + (string)((ArrayList)RDFutilitaires.resulpermutation[j])[i] + ",";
                    }
                    res = res.Trim(',');
                    if (elt.StereotypeEx.Contains(res))
                    {
                        ret = true;
                        foreach (string stereo in this.notClasseStereos)
                        {
                            if (elt.StereotypeEx.Contains(stereo))
                            {
                                ret = false;
                                break;
                            }

                        }
                        break;
                    }
                }
            }
            if (ret == true)
            {
                if (!alreadyFound.Contains(elt.ElementID)) alreadyFound.Add(elt.ElementID);
                else return false;
            }
            return ret;
        }

        private void StoreAbstractClasses(EA.Element elt)
        {
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();
            string ster = "";
            foreach (string stereo in this.subprofilesStereos)
            {
                if (elt.StereotypeEx.Contains(stereo))
                {
                    ster = stereo; break;
                }
            }
            if (ster.Length <= 0)
            {
                sw.WriteLine(asciiDocStyle.GetHeader(elt.Name, 4));
            }
            else
            {
                sw.WriteLine(asciiDocStyle.GetHeader(elt.Name + " (" + ster + ") ", 4));
            }
        }

        private string GetAsciiDocClass(EA.Package pack, EA.Element elt, bool isRoot)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            if (elt.Abstract.Equals("1"))
            {
                StoreAbstractClasses(elt);
            }
            else
            {
                if (isRoot) sb.Append(asciiDocStyle.GetHeader(elt.Name + " root class", 4));
                else sb.Append(asciiDocStyle.GetHeader(elt.Name, 4));
            }

            string notes = elt.Notes.Replace("\n", AsciiDocGenerator.LBR);
            if (elt.Abstract.Equals("1"))
            {
                sw.WriteLine(notes);
            }
            else sb.Append(notes);

            sb.Append(GenerateBasedOnAsciiDoc(elt, elt.Abstract, true));
            sb.Append(GenerateClassConstraintAsciiDoc(elt, elt.Abstract));

            if (elt.Connectors.Count > 0)
            {
                foreach (EA.Connector con in elt.Connectors)
                {
                    if (con.Type == "NoteLink")
                    {
                        string txt = asciiDocGenerator.GenerateNoteAsciiDoc(elt, con, elt.Abstract);
                        if(!elt.Abstract.Equals("1")) sb.Append(txt);
                    }
                }
            }

            List<long> attrIDs = new List<long>();
            bool native = false;
            Dictionary<string, List<EA.Attribute>> subprofiles = GetSubProfilesAttributes(elt);
            Dictionary<string, List<EA.Connector>> consubprofiles = GetSubProfilesConnectors(elt);

            foreach (string stereo in this.subprofilesStereos)
            {
                if ((subprofiles.ContainsKey(stereo) && (subprofiles[stereo].Count > 0))
                    || ThereIsLegitimateConnection(elt, consubprofiles, stereo))
                {
                    native = true;
                    string txt = asciiDocStyle.GetHeader("Native Members" + "(" + stereo + ")", 3);
                    if (elt.Abstract.Equals("1")) sw.WriteLine(txt);
                    else sb.Append(txt);

                    string tableString = GetNativeMembersTable(attrIDs, subprofiles, stereo);
                    if (elt.Abstract.Equals("1")) sw.WriteLine(tableString);
                    else sb.Append(tableString);


                    if (consubprofiles.ContainsKey(stereo) && (consubprofiles[stereo].Count > 0))
                    {
                        foreach (EA.Connector con in consubprofiles[stereo])
                        {
                            if ((con.Type == "Association") || (con.Type == "Aggregation"))
                            {
                                string assocTableString = asciiDocGenerator.GetAssocAsciiDoc(elt, pack, con);
                                if (elt.Abstract.Equals("1")) sw.WriteLine(assocTableString);
                                else sb.Append(assocTableString);
                            }
                        }
                    }
                }
            }

            native = false;
            if (elt.Attributes.Count > 0)
            {
                native = true;
                string asciiDocString = GetAttributesTable(pack, elt, attrIDs, subprofiles);
                if (elt.Abstract.Equals("1")) sw.WriteLine(asciiDocString);
                else sb.Append(asciiDocString);
            }

            if (consubprofiles.ContainsKey("profile") && consubprofiles["profile"].Count > 0)
            {
                StringBuilder profileSb = new StringBuilder();
                if (consubprofiles["profile"].Count > 0)
                {
                    if (!native)
                    {
                        native = true;
                        profileSb.Append(asciiDocStyle.GetHeader("Native Members", 3));
                    }

                    if (consubprofiles.ContainsKey("profile"))
                    {
                        List<EA.Connector> cs = new List<EA.Connector>();
                        foreach (EA.Connector cn in consubprofiles["profile"])
                        {
                            if (asciiDocGenerator.IsConEligible(elt, cn))
                            {
                                cs.Add(cn);
                            }
                        }

                        cs.Sort(RDFutilitaires.CompareEltConnectorsbyRoleName);

                        foreach (EA.Connector con in cs)
                        {
                            if ((con.Type == "Association") || (con.Type == "Aggregation"))
                            {
                                string assocTableString = asciiDocGenerator.GetAssocAsciiDoc(elt, pack, con);
                                profileSb.Append(assocTableString);
                            }
                        }
                    }
                }
                if (elt.Abstract.Equals("1")) sw.WriteLine(profileSb.ToString());
                else sb.Append(profileSb.ToString());
            }
            return sb.ToString();
        }



        private string[] GetAssocIsBasedOn(EA.Connector con)
        {
            string IBOGuid = RDFutilitaires.getBasedOnConnectoryGUID(repo, con.ConnectorGUID);
            EA.Connector IBOcon = repo.GetConnectorByGuid(IBOGuid);
            string[] text = { "", "" };
            if (IBOcon != null)
            {
                EA.Element source_element = repo.GetElementByID((int)IBOcon.ClientID);
                EA.Element target_element = repo.GetElementByID((int)IBOcon.SupplierID);
                EA.ConnectorEnd source = IBOcon.ClientEnd;
                EA.ConnectorEnd target = IBOcon.SupplierEnd;
                text[0] = repo.GetPackageByID((int)source_element.PackageID).Name + "::" + source_element.Name + "." + source.Role;
                text[0] = text[0] + "[" + source.Cardinality + "]";
                text[1] = repo.GetPackageByID((int)target_element.PackageID).Name + "::" + target_element.Name + "." + target.Role;
                text[1] = text[1] + "[" + target.Cardinality + "]";
            }
            else
            {
                MessageBox.Show("WARNING there is a problem in the model the association roles: " + "ClientRole=" + con.ClientEnd.Role + "," + "SupplierRole=" + con.SupplierEnd.Role + "\n\r model not built with CimSyntaxGen!");
            }
            return text;
        }

        private string GetAttributesTable(EA.Package pack, EA.Element elt, List<long> attrIDs, Dictionary<string, List<EA.Attribute>> subprofiles)
        {
            StringBuilder sb = new StringBuilder("The following table shows all attributes of " + elt.Name + ".");

            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + pack.Name + "::" + elt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });
            if (subprofiles.ContainsKey("profile"))
            {
                foreach (EA.Attribute attr in subprofiles["profile"])
                {
                    attrIDs.Add(attr.AttributeID);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", AsciiDocGenerator.LBR)});
                }
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));
            return sb.ToString();
        }

        private string GetNativeMembersTable(List<long> attrIDs, Dictionary<string, List<EA.Attribute>> subprofiles, string stereo)
        {
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - Native Members");
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });

            if ((subprofiles.ContainsKey(stereo) && (subprofiles[stereo].Count > 0)))
            {
                foreach (EA.Attribute attr in subprofiles[stereo])
                {
                    attrIDs.Add(attr.AttributeID);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", AsciiDocGenerator.LBR)});
                }
            }
            return asciiDocTable.GetTable("1, 2, 4", 100);
        }

        private string GetAttrType(EA.Attribute attr)
        {
            if ((attr.ClassifierID == 0))                                         
            {
                return attr.Type;
            }
            else
            {
                EA.Element claselt = Main.Repo.GetElementByID((int)attr.ClassifierID);
                string type = GetTypeElt(claselt);
                if (TypeList.ContainsKey(type))
                {
                    if (!TypeList[type].Contains(claselt.ElementID)) TypeList[type].Add(claselt.ElementID);
                }
                return type;
            }
        }

        private string GetTypeElt(EA.Element el)
        {
            string ret = "";

            if ((el.StereotypeEx.Contains(CD.GetEnumStereotype())) || (el.Type == "Enumeration")) //am nov 2015
            {
                ret = "enumeration";
            }
            if (el.StereotypeEx.Contains(CD.GetDatatypeStereotype())) ret = "datatype";
            if (el.StereotypeEx.Contains(CD.GetCompoundStereotype())) ret = "compound";
            if (el.StereotypeEx.Contains("datatype")) ret = "Datatype"; // legacy
            if ((is351) && (el.StereotypeEx.Contains(CD.GetPrimitiveStereotype()))) ret = "primitive";
            return ret;
        }



        private Dictionary<string, List<EA.Connector>> GetSubProfilesConnectors(EA.Element elt)
        {
            Dictionary<string, List<EA.Connector>> conSubProfiles = new Dictionary<string, List<EA.Connector>>();
            if (elt.Connectors.Count > 0)
            {
                foreach (string stereo in this.subprofilesStereos)
                {
                    conSubProfiles[stereo] = new List<EA.Connector>();
                }
                conSubProfiles["profile"] = new List<EA.Connector>();

                // classifying connections
                bool subprofile = false;
                EA.Element otherEndElt = null;
                foreach (EA.Connector con in elt.Connectors)
                {
                    subprofile = false;
                    foreach (string stereo in this.subprofilesStereos)
                    {
                        if ((con.Type == "Association") || (con.Type == "Aggregation"))
                        {
                            otherEndElt = RDFutilitaires.getTheotherend(this.repo, elt, con);
                            string otherendstereo = otherEndElt.StereotypeEx;
                            if (otherendstereo.Contains(stereo) || con.StereotypeEx.Contains(stereo))
                            {
                                if (!conSubProfiles[stereo].Contains(con)) conSubProfiles[stereo].Add(con);
                                subprofile = true;
                                break;
                            }
                        }
                    }
                    if (!subprofile)
                    {
                        if ((con.Type == "Association") || (con.Type == "Aggregation"))
                        {
                            conSubProfiles["profile"].Add(con);
                        }
                    }
                }
            }
            return conSubProfiles;
        }

        private Dictionary<string, List<EA.Attribute>> GetSubProfilesAttributes(EA.Element elt)
        {
            Dictionary<string, List<EA.Attribute>> subProfiles = new Dictionary<string, List<EA.Attribute>>();
            if (elt.Attributes.Count > 0)
            {
                foreach (string stereo in this.subprofilesStereos)
                {
                    subProfiles[stereo] = new List<EA.Attribute>();
                }
                subProfiles["profile"] = new List<EA.Attribute>();
                List<EA.Attribute> atrlist = this.SortAttributesByName(elt.Attributes);

                foreach (EA.Attribute atr in atrlist)
                {
                    string atrname = atr.Name;
                    bool subprofile = false; // the attribute is not in a subprofile
                    string stereosenordre = RDFLAutilitaires.RedressStereo(atr.StereotypeEx);

                    // we remember that the stereo Subprofiles are classified in descending order of length
                    foreach (string stereo in this.subprofilesStereos)
                    {
                        if (!stereo.Contains(","))
                        {
                            if (stereosenordre.Contains(stereo))
                            {
                                subProfiles[stereo].Add(atr);
                                subprofile = true;
                                break;
                            }
                        }
                        else
                        {
                            string sterenordre = RDFLAutilitaires.RedressStereo(stereo);
                            if (stereosenordre.Contains(sterenordre))
                            {
                                subProfiles[stereo].Add(atr);
                                subprofile = true;
                                break;
                            }
                        }
                    }
                    if (!subprofile) { subProfiles["profile"].Add(atr); }
                }
            }
            return subProfiles;
        }

        private bool ThereIsLegitimateConnection(EA.Element el, Dictionary<string, List<EA.Connector>> consubprofiles, string stereo)
        {
            bool ret = false;
            if (consubprofiles.ContainsKey(stereo) && consubprofiles[stereo].Count > 0)
            {
                foreach (EA.Connector con in consubprofiles[stereo])
                {
                    if ((con.Type == "Association") || (con.Type == "Aggregation"))
                    {
                        string cardinality = GetOtherEndCardinality(el, con);
                        EA.ConnectorEnd otherend = (ConnectorEnd)(RDFutilitaires.getOtherend(el, con)[0]);

                        if ((cardinality == "1") || (cardinality == "1..1") || (cardinality == "0..1") || (otherend.Navigable == "Navigable"))
                        {
                            ret = true;
                            break;
                        }
                    }
                }
            }
            return ret;
        }

        private string GetOtherEndCardinality(EA.Element el, EA.Connector con)
        {
            string cardinality = "";
            if ((con.Type == "Association") || (con.Type == "Aggregation"))
            {
                EA.ConnectorEnd end = null;
                if (con.ClientID == el.ElementID) end = con.SupplierEnd;
                if (con.SupplierID == el.ElementID) end = con.ClientEnd;
                if(end != null) cardinality = end.Cardinality;
            }
            return cardinality;
        }



        private string GenerateBasedOnAsciiDoc(EA.Element elt, string isAbstract, bool classHeader)
        {
            if (!classHeader) return ""; // we do nothing 
            StringBuilder sb = new StringBuilder();
            EA.Element basedOn = RDFutilitaires.getElementBasedonElement(elt);
            sb.Append(" IsBasedOn: " + asciiDocGenerator.GetPackageContainmentPath(basedOn) + "::" + basedOn.Name + AsciiDocGenerator.LBR);
            if (isAbstract.Equals("1"))
            {
                sw.WriteLine(sb.ToString());
                return "";
            }
            else return sb.ToString();
        }

        private string GenerateClassConstraintAsciiDoc(EA.Element elt, string isAbstract)
        {

            StringBuilder sb = new StringBuilder();
            foreach (EA.Constraint constr in elt.Constraints)
            {
                if (constr.Type.Equals("Invariant"))
                {
                    string content = "constraint:" + AsciiDocGenerator.LBR + constr.Name + "={" + constr.Notes + "}" + AsciiDocGenerator.LBR;
                    if (isAbstract.Equals("1"))
                    {
                        sw.WriteLine(content);
                    }
                    else sb.Append(content);
                }
            }
            return sb.ToString();
        }


        private List<EA.Attribute> SortAttributesByName(EA.Collection col)
        {
            List<EA.Attribute> retlist = new List<EA.Attribute>();
            Dictionary<string, EA.Attribute> dicAttrByName = new Dictionary<string, EA.Attribute>();
            foreach (EA.Attribute at in col)
            {
                dicAttrByName.Add(at.Name, at);
            }
            Dictionary<string, EA.Attribute>.KeyCollection keysCol = dicAttrByName.Keys;
            string[] arkeys = new string[keysCol.Count];
            keysCol.CopyTo(arkeys, 0);
            Array.Sort(arkeys);

            for (int i = 0; i < arkeys.Length; i++)
            {
                retlist.Add(dicAttrByName[arkeys[i]]);
            }
            return retlist;
        }
    }
}
