﻿using CimSyntaxGen.Utilities;
using EA;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection.Emit;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Net.Mime.MediaTypeNames;

namespace CimSyntaxGen.AsciiDoc
{
    public class AsciiDocESMPSpecificDocumentation
    {
        private bool iec = false;
        private EA.Repository repo;
        private XMLParser XMLP;
        private EA.Package selectedPackage;
        public int rgtable = 0;
        private StringBuilder AbstractsBuffer = new StringBuilder();
        private StringWriter sw;
        private Dictionary<string, List<long>> TypeList = new Dictionary<string, List<long>>(); // string, elementID
        private List<EA.Element> profileClasses = new List<EA.Element>();
        private List<EA.Element> listPrimitives = new List<EA.Element>(); // list of primitives 
        private Dictionary<long, string> dicPackim = new Dictionary<long, string>();
        private AsciiDocGenerator asciiDocGenerator;
        private List<string> notPureClasses = new List<string>();  // to be used by find predicates
        private ConstantDefinition CD = new ConstantDefinition();
        private static Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);
        private static ProgressWindow progressWindow = new ProgressWindow();
        private Thread progressThread;
        private void ShowProgress()
        {
            progressWindow.Show();
        }

        private void UpdateProgress(int value)
        {
            //progressWindow.progressBar.Value = value;
            //progressWindow.Refresh();
        }

        public void CloseProgressWindow()
        {
            progressWindow.Dispose();
            progressThread.Abort();
        }

        public void ResetProgress()
        {
            UpdateProgress(0);
        }

        public AsciiDocESMPSpecificDocumentation(EA.Repository repo, bool forIEC, EA.Package selectedPkg)
        {
            //progressThread = new Thread(ShowProgress);
            this.iec = forIEC;
            this.repo = repo;
            this.XMLP = new XMLParser(repo);
            rgtable = 0;
            this.selectedPackage = selectedPkg;
            EA.Package model = (EA.Package)repo.Models.GetAt(0);
            //RDFutilitaires.buildDicsElem(model);
            UpdateProgress(5);
            //RDFutilitaires.getAllElements(model, listPrimitives, "Primitive");
            sw = new StringWriter(AbstractsBuffer);
            UpdateProgress(10);

            this.TypeList.Add("datatype", new List<long>());
            this.TypeList.Add("enumeration", new List<long>());
            this.TypeList.Add("compound", new List<long>());
            this.TypeList.Add("primitive", new List<long>());

            this.notPureClasses.Add(CD.GetEnumStereotype());
            this.notPureClasses.Add("Datatype");
            this.notPureClasses.Add(CD.GetDatatypeStereotype());
            this.notPureClasses.Add(CD.GetPrimitiveStereotype());
            this.notPureClasses.Add(CD.GetCompoundStereotype());
            asciiDocGenerator = new AsciiDocGenerator(this.repo);
        }

        public static string GenerateCoverIntro(EA.Repository repos, EA.Package pack, string outputDir)
        {
            AsciiDocGenerator asciiDocGenerator = new AsciiDocGenerator(repos);
            asciiDocGenerator.SaveCoverpageImage(outputDir);
            return asciiDocGenerator.GetCoverIntroAsciiDoc(pack.Name);
        }


        public string GenerateDocu(string outputDir)
        {
            UpdateProgress(15);

            StringBuilder sb = new StringBuilder();
            List<EA.Element> profileClasses = new List<EA.Element>();
            RDFutilitaires.getTypedElements(this.selectedPackage, profileClasses, "Class");
            profileClasses.Sort(RDFutilitaires.CompareElementsbyName);

            /////// header
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();
            sb.Append(asciiDocStyle.GetHeader(this.selectedPackage.Name, 2));
            sb.Append(asciiDocStyle.GetHeader("Overview of the model", 3));
            sb.Append("The following diagram shows the model:\n");
            UpdateProgress(20);

            /////// image
            string diagramFileName = asciiDocGenerator.SaveDiagram(this.selectedPackage, outputDir);
            AsciiDocImage asciiDocImage = new AsciiDocImage(this.selectedPackage.Name, diagramFileName, 1, -1, -1);
            sb.Append(asciiDocImage.GetImage());
            UpdateProgress(25);

            ////// table for isBasedOn
            sb.Append(asciiDocStyle.GetHeader("IsBasedOn relationships from the European style market profile:", 3));
            sb.Append("\nThe following table shows the traceability dependency of the classes used in this package towards the upper level." + AsciiDocGenerator.LBR + "\n");
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - IsBasedOn Dependency");
            asciiDocTable.SetHeaderRow(new List<string>() { "Name", "Complete IsBasedOn Path" });
            foreach (EA.Element elt in profileClasses)
            {
                EA.Element basedOnElem = RDFutilitaires.getElementOriginBasedonElement(elt);
                asciiDocTable.AddRow(new List<string>() { elt.Name, asciiDocGenerator.GetPackageContainmentPath(basedOnElem) + "::" + basedOnElem.Name });
            }
            sb.Append("\n");
            sb.Append(asciiDocTable.GetTable("1, 4", 100));
            UpdateProgress(30);

            ////// Concrete Classes
            List<EA.Element> classlists = new List<EA.Element>();
            classlists = profileClasses.FindAll(HasNotClassStereotype);
            List<EA.Element> rootlist = new List<EA.Element>();
            foreach (EA.Element elt in classlists)
            {
                if (elt.IsActive)
                {
                    rootlist.Add(elt);
                }
            }
            UpdateProgress(35);

            foreach (EA.Element el in rootlist)
            {
                classlists.Remove(el);
            }
            UpdateProgress(40);

            classlists.Sort(RDFutilitaires.CompareElementsbyName);
            UpdateProgress(45);

            if (!iec)
            {
                if (this.selectedPackage.Name.Contains("contextual")) return sb.ToString(); // for the contextual, we refer to the diagram of the model
            }

            sb.Append(asciiDocStyle.GetHeader("Detailed" + " " + this.selectedPackage.Name, 3));
            foreach (EA.Element elt in rootlist)
            {
                string asciiDocString = GetAsciiDocClass(this.selectedPackage, elt, true);
                sb.Append(asciiDocString);
            }
            UpdateProgress(50);

            foreach (EA.Element elt in classlists)
            {
                string asciiDocString = GetAsciiDocClass(this.selectedPackage, elt, false);
                sb.Append(asciiDocString);
            }
            UpdateProgress(55);

            GetAllProfilePrimitivesFromDatatype();

            UpdateProgress(60);

            ///////// Datatypes
            if (!this.selectedPackage.Name.Contains("contextual") && !this.selectedPackage.Name.Contains("Contextual"))
            {
                UpdateProgress(65);

                /////// Primitives
                if (iec)
                { // special case of contextual and assembly
                    // Primitive types types
                    if ((TypeList["primitive"].Count > 0))
                    {
                        TypeList["primitive"].Sort(RDFutilitaires.CompareElementIDsbyName);
                        sb.Append(asciiDocStyle.GetHeader("Primitives", 3));
                    }
                    foreach (long primID in TypeList["primitive"])
                    {
                        EA.Element prim = repo.GetElementByID((int)primID);
                        if (prim.Name.Equals(""))
                        {
                            continue;
                        }
                        else
                        {
                            sb.Append(GetPrimitiveTypeAsciiDoc(prim));
                        }
                    }
                }
                UpdateProgress(70);


                // Any local datatypes and compounds are processed at this level
                // which are otherwise produced automatically from attribute classifiers
                foreach (EA.Element elt in profileClasses)
                {
                    string eltType = GetTypeElt(elt);
                    if (TypeList.ContainsKey(eltType))
                    {
                        if (!TypeList[eltType].Contains(elt.ElementID)) TypeList[eltType].Add(elt.ElementID);
                    }
                }
                UpdateProgress(75);

                /*************************************************
                * Compound types types
                * ***********************************************/
                string COMPOUND = "compound";
                string DATATYPE = "datatype";
                if ((TypeList[COMPOUND].Count > 0) || (TypeList[DATATYPE].Count > 0))
                {
                    FillDatatypeList(); //on explore la liste et on ajoute les compound appeles par d'aures compounds am avril 2016

                    if ((TypeList[COMPOUND].Count > 0))
                    {
                        TypeList[COMPOUND].Sort(RDFutilitaires.CompareElementIDsbyName);// am avril 2016 
                    }

                    if (TypeList[DATATYPE].Count > 0)
                    {
                        TypeList[DATATYPE].Sort(RDFutilitaires.CompareElementIDsbyName);//am avril 2016
                    }

                    sb.Append(asciiDocStyle.GetHeader("Datatypes", 3));
                    if (this.selectedPackage.Name.Contains("assembly"))
                    {
                        sb.Append("The list of datatypes used for the " + this.selectedPackage.Name + " is as follows:\n\n");
                    }
                    UpdateProgress(80);


                    foreach (long comp in TypeList[COMPOUND])
                    {
                        if (this.selectedPackage.Name.Contains("assembly") && !iec)
                        {// special treatment for assembly in the case of ENTSOE-Documentation
                            sb.Append("* " + this.repo.GetElementByID((int)comp).Name + " compound" + "\n");
                        }
                        else
                        {
                            sb.Append(GetCompoundtypeAsciiDoc(this.repo.GetElementByID((int)comp)));
                        }
                    }
                    UpdateProgress(85);

                    if (TypeList[DATATYPE].Count > 0)
                    {
                        sb.Append("\n");
                        foreach (long data in TypeList[DATATYPE])
                        {
                            if ((this.selectedPackage.Name.Contains("assembly")) && (!iec)) // we just need to edit the list of datatypes and compounds
                            {
                                // special treatment for assembly
                                string codelist = GetCodelist(this.repo.GetElementByID((int)data));
                                if (codelist.Equals(""))
                                {
                                    sb.Append("* " + this.repo.GetElementByID((int)data).Name + " datatype" + "\n");
                                }
                                else
                                {
                                    sb.Append("* " + this.repo.GetElementByID((int)data).Name + " datatype" + ", codelist " + codelist + "\n");
                                }
                                FillDatatypeForAssemblyList(this.repo.GetElementByID((int)data)); // feeds TypeList[enumeration] if necessary
                            }
                            else
                            {
                                sb.Append(GetDatatypeAsciiDoc(this.repo.GetElementByID((int)data)));
                            }
                        }
                    }
                    UpdateProgress(90);

                }

                ////////// Enumerations
                if (TypeList["enumeration"].Count > 0)
                {
                    TypeList["enumeration"].Sort(RDFutilitaires.CompareElementIDsbyName);
                }

                List<string> typelists = new List<string>();
                bool withEnumeration = false;
                foreach (long kind in TypeList["enumeration"])
                {
                    string nom = Main.Repo.GetElementByID((int)kind).Name;
                    if ((!nom.Contains("TypeList"))
                      && (!(nom == "UnitMultiplier"))
                      && (!(nom == "UnitSymbol"))
                      )
                    {
                        withEnumeration = true;
                        continue;
                    }
                    else
                    {
                        if (!typelists.Contains(nom)) typelists.Add(nom);
                    }
                }

                if (((withEnumeration) && (iec)) ||    // not true for ENTSOE doc
                    ((typelists.Count > 0) && (iec))) 
                {
                    sb.Append(asciiDocStyle.GetHeader("Enumerations", 3));
                    sb.Append("The list of enumerations used for the " + this.selectedPackage.Name + " is as follows:" + AsciiDocGenerator.LBR);
                }

                if (iec) // true if no doc entso-e
                {
                    foreach (long kind in TypeList["enumeration"])
                    {
                        string nom = this.repo.GetElementByID((int)kind).Name;
                        if ((!nom.Contains("TypeList"))
                          && (!(nom == "UnitMultiplier"))
                          && (!(nom == "UnitSymbol"))
                          )
                        {
                            sb.Append(GetEnumerationAsciiDoc(Main.Repo.GetElementByID((int)kind)));
                        }

                    }
                    UpdateProgress(95);

                    if (typelists.Count > 0)
                    {   // there are codelist enumerations to be displayed as name lists
                        sb.Append("\n");
                        foreach (string n in typelists)
                        {
                            sb.Append("* " + n + "\n");
                        }
                    }
                }
            }
            UpdateProgress(100);
            return sb.ToString();
        }

        private string GetEnumerationAsciiDoc(EA.Element enumelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(enumelt.Name + " enumeration", 4));
            sb.Append(enumelt.Notes.Replace("\n", "") + AsciiDocGenerator.LBR);

            if (enumelt.Name.Contains("TypeList"))
            {
                sb.Append("\nThe following table shows examples of literals for " + enumelt.Name + "." + AsciiDocGenerator.LBR + "\n");
            }
            else
            {
                sb.Append("\nThe following table shows all literals of " + enumelt.Name + "." + AsciiDocGenerator.LBR + "\n");
            }
            string packName = this.repo.GetPackageByID(enumelt.PackageID).Name;
            AsciiDocTable asciiDocTable = new AsciiDocTable("\nTable - " + " Literals of " + packName + "::" + enumelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Literal", "Code", "Description" });

            foreach (EA.Attribute attr in enumelt.Attributes)
            {
                asciiDocTable.AddRow(new List<string>() { attr.Name,
                                                         GetEnumCode(attr),
                                                         attr.Notes.Replace("\n", "")});
            }
            sb.Append(asciiDocTable.GetTable("2, 1, 4", 100));
            return sb.ToString();
        }

        private string GetEnumCode(EA.Attribute at)
        {
            string resul = "";
            foreach (EA.AttributeTag tag in at.TaggedValues)
            {
                if (tag.Name == CD.GetEnumCodeTag())
                {
                    resul = tag.Value;
                }
            }
            return resul;
        }



        private string GetDatatypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " datatype", 4));
            sb.Append(dataelt.Notes.Replace("\n", "") + AsciiDocGenerator.LBR);
            string packName = this.repo.GetPackageByID(dataelt.PackageID).Name;

            if (dataelt.Connectors.Count > 0)
            {
                foreach (EA.Connector con in dataelt.Connectors)
                {
                    if (con.Type == "NoteLink")
                    {
                        sb.Append(asciiDocGenerator.GenerateNoteAsciiDoc(dataelt, con, "0"));
                    }
                }
            }

            sb.Append("\nThe following table shows all attributes of " + dataelt.Name + "." + AsciiDocGenerator.LBR + "\n");
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });

            bool thereAreConstraints = false;
            foreach (EA.Attribute attr in dataelt.Attributes)
            {
                string type = GetAttrType(attr);
                asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " / " + type,
                                                         attr.Notes.Replace("\n", "")});

                if (attr.Constraints.Count > 0) thereAreConstraints = true;
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));

            if (!thereAreConstraints) // there are no constraints
            {
                return sb.ToString();
            }

            sb.Append(GetAsciiDocWithContraintsTable(packName, dataelt));
            return sb.ToString();
        }


        private string GetAsciiDocWithContraintsTable(string packName, EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("\nThe following table shows all restrictions applied to the attributes  of " + dataelt.Name + "." + AsciiDocGenerator.LBR + "\n");
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Restrictions of attributes for " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Name", "Constraint", "Type", "Expression of constraint" });

            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if (at.IsConst)
                {
                    asciiDocTable.AddRow(new List<string>() { at.Name, "Constant", "", at.Default.ToString() });
                }

                if (at.IsStatic)
                {
                    asciiDocTable.AddRow(new List<string>() { at.Name, "Fixed", "", at.Default.ToString() });
                }

                if (at.Constraints.Count > 0)
                {
                    foreach (EA.AttributeConstraint ac in at.Constraints)
                    {
                        string constrNotes = ac.Notes.Replace("|", "\\|");
                        string constraint = "";
                        if (ac.Type == "OCL")
                        {
                            int pos1 = constrNotes.IndexOf("->");
                            int pos2 = constrNotes.IndexOf('(');
                            int pos3 = constrNotes.IndexOf(')');
                            constraint = constrNotes.Substring(pos1 + 1, pos3 - pos1);
                        }
                        else
                        {
                            if (ac.Type.Equals("INV") || ac.Type.Equals("Invariant"))
                            {
                                constraint = ">" + constrNotes;
                            }

                        }
                        if (constrNotes.Length > 0)
                        {
                            asciiDocTable.AddRow(new List<string>() { at.Name, ac.Name, ac.Type, constrNotes });
                        }
                    }
                }
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 1, 3", 100));
            return sb.ToString();
        }


        private void FillDatatypeForAssemblyList(EA.Element dataelt)
        {
            // We simply feed TypeList[enumeration].
            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if ((at.ClassifierID != 0))
                {
                    EA.Element claselt = Main.Repo.GetElementByID((int)at.ClassifierID);
                    string type = GetTypeElt(claselt);
                    if (type == "enumeration") // we only look at the enumeration because in the event of a compound we could come across a datatype
                    {
                        if (TypeList.ContainsKey(type))
                        {
                            if (!TypeList[type].Contains(claselt.ElementID)) TypeList[type].Add(claselt.ElementID);

                        }
                    }
                }
            }
        }



        /// Returns the name of the codelist if it is the value or DomainQualification type
        public string GetCodelist(EA.Element datelt)
        {
            if ((!datelt.StereotypeEx.Contains(CD.GetDatatypeStereotype()))
                && (!datelt.StereotypeEx.Contains("datatype")))
            {
                return "";
            }

            foreach (EA.Attribute at in datelt.Attributes)
            {
                if (at.ClassifierID != 0)
                {
                    Element classifier = Main.Repo.GetElementByID((int)at.ClassifierID);
                    if ((!classifier.StereotypeEx.Contains(CD.GetEnumStereotype())) && !(classifier.Type == "Enumeration"))
                    {
                        continue;
                    }
                    else
                    {
                        if ((classifier.Name.Contains("TypeList"))
                       || ((classifier.Name == "UnitMultiplier"))
                       || ((classifier.Name == "UnitSymbol"))
                       )
                        {
                            return classifier.Name;
                        }
                    }
                }
            }
            return "";
        }


        private string GetCompoundtypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " compound", 4));
            sb.Append(dataelt.Notes.Replace("\n", "") + AsciiDocGenerator.LBR);
            sb.Append("\nThe following table shows all attributes of " + dataelt.Name + ". " + AsciiDocGenerator.LBR + "\n");
            string packName = this.repo.GetPackageByID(dataelt.PackageID).Name;
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + packName + "::" + dataelt.Name);
            asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });

            int ind = 0;
            Dictionary<int, int> dicOrder = new Dictionary<int, int>();
            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if (!dicOrder.ContainsKey(at.Pos)) dicOrder.Add(at.Pos, ind);
                ind++;
            }
            List<int> positions = new List<int>(); // The positions are copied from the CIM element 
                                                   // they are not necessarily in ascending order from 0

            for (short i = 0; i < dataelt.Attributes.Count; i++)
            {
                EA.Attribute atr = (EA.Attribute)dataelt.Attributes.GetAt(i);
                if (!positions.Contains((atr.Pos)))
                {
                    positions.Add(atr.Pos);
                }
            }
            foreach (int pos in positions)
            {
                if (dicOrder[pos] < dataelt.Attributes.Count)
                {
                    EA.Attribute attr = (EA.Attribute)dataelt.Attributes.GetAt((short)dicOrder[pos]);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " / " + type,
                                                         attr.Notes.Replace("\n", "")});
                }
                else
                {
                    MessageBox.Show("Error the attribute position "
                        + dicOrder[pos].ToString() + " of " + dataelt.Name + " exceeds the maximumum index!");
                }
            }
            sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));
            return sb.ToString();
        }


        private string GetPrimitiveTypeAsciiDoc(EA.Element dataelt)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            if (dataelt.Name != "")
            {
                sb.Append(asciiDocStyle.GetHeader(dataelt.Name + " primitive", 4));
            }
            sb.Append(dataelt.Notes.Replace("\n", "").Replace("<b><u>Note: </u></b>", "\n\n*Note:* ") + AsciiDocGenerator.LBR + "\n");
            return sb.ToString();
        }


        private void FillDatatypeList()
        {
            int nbcompound = TypeList["compound"].Count;
            int nbdatatype = TypeList["datatype"].Count;
            List<long> locallist = new List<long>(TypeList["compound"]);
            locallist.AddRange(TypeList["datatype"]); // merge datatypes and compounds
            foreach (long elid in locallist)
            {
                EA.Element elt = this.repo.GetElementByID((int)elid);
                foreach (EA.Attribute at in elt.Attributes)
                {
                    if (at.ClassifierID != 0)
                    {
                        EA.Element claselt = Main.Repo.GetElementByID((int)at.ClassifierID);
                        string texte = claselt.Name;
                        string typp = GetTypeElt(claselt);
                        if (typp != "")
                        {
                            if ((typp == "datatype") || (typp == "compound"))
                            {
                                if ((!TypeList[typp].Contains(claselt.ElementID)))
                                {
                                    TypeList[typp].Add(claselt.ElementID);
                                }
                                foreach (EA.Attribute att in claselt.Attributes)
                                {
                                    if (att.ClassifierID != 0)
                                    {
                                        string type = GetTypeElt(this.repo.GetElementByID((int)att.ClassifierID));
                                        if (type != "")
                                        {
                                            if ((!TypeList[type].Contains(att.ClassifierID)))
                                            {
                                                TypeList[type].Add(att.ClassifierID);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if ((TypeList["compound"].Count != nbcompound) || (TypeList["datatype"].Count != nbdatatype))
            {
                FillDatatypeList(); // successive cascades must be taken into account, hence the recursion
            }
        }



        // recovers all datatype primitives and compounds
        void GetAllProfilePrimitivesFromDatatype()
        {
            foreach (long compid in TypeList["compound"])
            {
                GetPrimitiveFromCompound(compid);
            }

            foreach (long dtid in TypeList["datatype"])
            {
                EA.Element dataelt = repo.GetElementByID((int)dtid);
                foreach (EA.Attribute at in dataelt.Attributes)
                {
                    if (at.Name == "value")
                    {
                        if (at.ClassifierID != 0)
                        {
                            EA.Element classifier = repo.GetElementByID((int)at.ClassifierID);
                            string type = GetTypeElt(classifier);
                            if (TypeList.ContainsKey(type))
                            {
                                if (!TypeList[type].Contains(classifier.ElementID)) TypeList[type].Add(classifier.ElementID);
                            }
                        }
                        break;
                    }
                }
            }
        }

        // recovers the primitive(s) of a compound, taking into account 
        // any cascading
        void GetPrimitiveFromCompound(long compoundid)
        {
            EA.Element dataelt = repo.GetElementByID((int)compoundid);
            string prov = dataelt.Name;
            foreach (EA.Attribute at in dataelt.Attributes)
            {
                if (at.ClassifierID != 0)
                {
                    EA.Element classifier = repo.GetElementByID((int)at.ClassifierID);
                    prov = classifier.Name;
                    string type = GetTypeElt(classifier);
                    if (type == "primitive")
                    {
                        if (TypeList.ContainsKey(type))
                        {
                            if (!TypeList[type].Contains(classifier.ElementID)) TypeList[type].Add(classifier.ElementID);
                        }
                    }
                    else
                    {
                        if (type == "compound")
                        {
                            GetPrimitiveFromCompound(classifier.ElementID);
                        }
                    }
                }
            }
        }



        public bool HasNotClassStereotype(EA.Element elt)
        {
            if (elt.StereotypeEx == "")
            {
                return true;  // obvious
            }

            if (elt.Type == "Enumeration")
            {
                return false;
            }
            foreach (string stereo in notPureClasses)
            {
                if (elt.StereotypeEx.Contains(stereo))
                {
                    return false;
                }
            }
            return true;
        }

        private string GetClassConstraintAsciiDoc(EA.Element elt, string isAbstract)
        {
            StringBuilder sb = new StringBuilder();
            foreach (EA.Constraint constr in elt.Constraints)
            {
                if (constr.Type == "Invariant")
                {
                    sb.Append("Constraint: " + constr.Name + "={" + constr.Notes + "}" + AsciiDocGenerator.LBR);
                    if (isAbstract.Equals("1")) this.sw.WriteLine(sb.ToString());
                }
            }
            return sb.ToString();
        }


        private string GetAsciiDocClass(EA.Package pack, EA.Element elt, bool isRoot)
        {
            StringBuilder sb = new StringBuilder();
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();

            if (isRoot)
            {
                sb.Append(asciiDocStyle.GetHeader(elt.Name + " root class", 4));
            }
            else
            {
                sb.Append(asciiDocStyle.GetHeader(elt.Name, 4));
            }
            sb.Append(elt.Notes.Replace("\n", " + \n") + AsciiDocGenerator.LBR);

            sb.Append(GetClassConstraintAsciiDoc(elt, elt.Abstract));

            //----------------
            if (elt.Connectors.Count > 0)
            {
                foreach (EA.Connector con in elt.Connectors)
                {
                    if (con.Type == "NoteLink")
                    {
                        string txt = asciiDocGenerator.GenerateNoteAsciiDoc(elt, con, elt.Abstract);
                        if (elt.Abstract.Equals("1")) sb.Append(txt);
                    }
                }

            }

            List<long> attrIDs = new List<long>();
            List<EA.Attribute> attrlist = new List<EA.Attribute>();
            if (elt.Attributes.Count > 0)
            {
                attrlist = SortAttributesByOrder(elt.Attributes);
            }

            List<EA.Connector> cs = new List<EA.Connector>();
            foreach (EA.Connector cn in elt.Connectors)
            {
                if (asciiDocGenerator.IsConEligible(elt, cn))
                {
                    try { cs.Add(cn); }
                    catch (Exception e) { MessageBox.Show(e.Message); }
                }
            }

            cs.Sort(RDFutilitaires.CompareEltConnectorsbyOrder);

            if (elt.Attributes.Count > 0)
            {
                string asciiDocAttrs = GetAttributesTable(pack, elt, attrIDs, attrlist, elt.Abstract);
                if (elt.Abstract.Equals("1")) sw.WriteLine(asciiDocAttrs);
                else sb.Append(asciiDocAttrs);
            }

            if (cs.Count > 0) // there are valid connectors
            {
                string assocTableString = GetAssocTableAsciiDoc(elt, pack, cs);
                if (elt.Abstract.Equals("1")) sw.WriteLine(assocTableString);
                else sb.Append(assocTableString);
            }
            return sb.ToString();
        }

        public string GetAssocTableAsciiDoc(EA.Element elt, EA.Package pack, List<EA.Connector> connListe)
        {
            StringBuilder sb = new StringBuilder();
            EA.ConnectorEnd otherend;
            EA.ConnectorEnd myend;
            EA.Element otherendelt;
            sb.Append("\nThe following table shows all association ends of " + elt.Name + " with other classes. " + AsciiDocGenerator.LBR + "\n");
            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - Association ends of " + pack.Name + "::" + elt.Name + " with other classes");
            asciiDocTable.SetHeaderRow(new List<string>() {"Order", "Mult.", "Class name / Role", "Description" });
            foreach (EA.Connector con in connListe)
            {
                if ((con.Type == "Association") || (con.Type == "Aggregation"))
                {
                    CIMSchemaGenerator.ConexesID = 0;
                    CIMSchemaGenerator.Connexion connex = new CIMSchemaGenerator.Connexion(con);
                    if (connex.Source_element.ElementID == elt.ElementID)
                    {// the element is the Source
                        otherend = connex.Target;
                        otherendelt = connex.Target_element;
                        myend = connex.Source;
                    }
                    else
                    {
                        otherend = connex.Source;
                        otherendelt = connex.Source_element;
                        myend = connex.Target;
                    }
                    string cardinality = otherend.Cardinality;
                    bool eligible = false;
                    if (cardinality == "1") cardinality = "1..1";
                    if (myend.Aggregation != 0)
                    {// the other end is eligible as member
                        eligible = true;
                    }
                    if (eligible && (otherend.Role != ""))
                    {
                        {// the association creates an effective memmber
                            string[] assocBasedOn = asciiDocGenerator.GetAssocIsBasedOn(con);
                            asciiDocTable.AddRow(new List<string>() { RDFutilitaires.getAtRang(con).ToString(),
                                                         "[" + cardinality + "]",
                                                         otherendelt.Name + " / " + otherend.Role,
                                                         otherend.RoleNote.Replace("\n", "") +
                                                         AsciiDocGenerator.LBR + "Association Based On : " + 
                                                         AsciiDocGenerator.LBR + assocBasedOn[0] +
                                                         AsciiDocGenerator.LBR + "--------" +
                                                         AsciiDocGenerator.LBR + assocBasedOn[1]});
                        }
                    }
                }

            }
            sb.Append(asciiDocTable.GetTable("1, 1, 2, 4", 100));
            return sb.ToString();
        }


        private string GetAttributesTable(EA.Package pack, EA.Element elt, List<long> attrIDs, List<EA.Attribute> attrlist, string withoutOrder)
        {
            StringBuilder sb = new StringBuilder("\nThe following table shows all attributes of " + elt.Name + "." + "\n");

            AsciiDocTable asciiDocTable = new AsciiDocTable("Table - " + " Attributes of " + pack.Name + "::" + elt.Name);
            if (!withoutOrder.Equals("1"))
            {
                asciiDocTable.SetHeaderRow(new List<string>() { "Order", "Mult.", "Attribute name / type", "Description" });
                foreach (EA.Attribute attr in attrlist)
                {
                    attrIDs.Add(attr.AttributeID);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { RDFutilitaires.getAtRang(attr).ToString(),
                                                         "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", "")});
                }
                sb.Append(asciiDocTable.GetTable("1, 1, 2, 4", 100));
            }
            else
            {
                asciiDocTable.SetHeaderRow(new List<string>() { "Mult.", "Attribute name / type", "Description" });
                foreach (EA.Attribute attr in attrlist)
                {
                    attrIDs.Add(attr.AttributeID);
                    string type = GetAttrType(attr);
                    asciiDocTable.AddRow(new List<string>() { "[" + attr.LowerBound + ".." + attr.UpperBound + "]",
                                                         attr.Name + " /" + AsciiDocGenerator.LBR + type,
                                                         attr.Notes.Replace("\n", "")});
                }
                sb.Append(asciiDocTable.GetTable("1, 2, 4", 100));
            }
            return sb.ToString();
        }

        private string GetAttrType(EA.Attribute attr)
        {
            if ((attr.ClassifierID == 0))
            {
                return attr.Type;
            }
            else
            {
                EA.Element claselt = Main.Repo.GetElementByID((int)attr.ClassifierID);
                string type = GetTypeElt(claselt);
                if (TypeList.ContainsKey(type))
                {
                    if (!TypeList[type].Contains(claselt.ElementID)) TypeList[type].Add(claselt.ElementID);
                }
                return claselt.Name;
            }
        }

        private string GetTypeElt(EA.Element el)
        {
            string ret = "";

            if ((el.StereotypeEx.Contains(CD.GetEnumStereotype())) || (el.Type == "Enumeration")) //am nov 2015
            {
                ret = "enumeration";
            }
            if (el.StereotypeEx.Contains(CD.GetDatatypeStereotype())) ret = "datatype";
            if (el.StereotypeEx.Contains(CD.GetCompoundStereotype())) ret = "compound";
            if (el.StereotypeEx.Contains("datatype")) ret = "Datatype"; // legacy
            if ((iec) && (el.StereotypeEx.Contains(CD.GetPrimitiveStereotype()))) ret = "primitive";
            return ret;
        }

        private List<EA.Attribute> SortAttributesByOrder(EA.Collection col)
        {
            List<EA.Attribute> retlist = new List<EA.Attribute>();
            Dictionary<int, EA.Attribute> dicAtrByOrder = new Dictionary<int, EA.Attribute>();
            foreach (EA.Attribute at in col)
            {
                dicAtrByOrder.Add(RDFutilitaires.getAtRang(at), at);
            }

            Dictionary<int, EA.Attribute>.KeyCollection keysCol = dicAtrByOrder.Keys;
            int[] arkeys = new int[keysCol.Count];
            keysCol.CopyTo(arkeys, 0);
            Array.Sort(arkeys);

            for (int i = 0; i < arkeys.Length; i++)
            {
                retlist.Add(dicAtrByOrder[arkeys[i]]);
            }
            return retlist;
        }
    }
}
