﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.AsciiDoc
{
    public partial class AsciiDocExportDialog : Form
    {
        private EA.Package pack;
        private XMLParser xmlp;
        private DialogResponse dialogResponse;

        public AsciiDocExportDialog(EA.Repository repo, EA.Package pack, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.buttonExport.Enabled = false;
            this.dialogResponse = dialogResponse;
            xmlp = new XMLParser(repo);
            this.pack = pack;
            this.textBoxExpDir.Text = xmlp.GetXmlValueAsciiDocData(XMLParser.LAST_ASCIIDOC_OUTPUT_DIRECTORY);
            List<string> fileExtensions = new List<string>();
            fileExtensions.Add("adoc");
            fileExtensions.Add("asciidoc");
            fileExtensions.Add("asc");
            fileExtensions.Add("ad");
            fileExtensions.ForEach(c =>
            {
                this.comboBoxFileExtension.Items.Add(c);
            });
            this.comboBoxFileExtension.SelectedIndex = 0;
        }

        private void buttonExpDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.textBoxExpDir.Text;
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxExpDir.Text = folderBrowserDialog.SelectedPath;
            }
            folderBrowserDialog.Dispose();
        }

        private void textBoxExpDir_TextChanged(object sender, EventArgs e)
        {
            if (this.textBoxFilename.Text.Trim().Length > 0 && this.textBoxExpDir.Text.Trim().Length > 0)
            {
                this.buttonExport.Enabled = true;
            }
            else this.buttonExport.Enabled = false;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            dialogResponse.Canceled = true;
            this.Dispose();
        }

        private void buttonExport_Click(object sender, EventArgs e)
        {
            string outputDir = this.textBoxExpDir.Text;
            xmlp.SetXmlValueAsciiDocData(XMLParser.LAST_ASCIIDOC_OUTPUT_DIRECTORY, outputDir);
            if (!outputDir.EndsWith("\\")) outputDir = outputDir + "\\";

            dialogResponse.Canceled = false;
            dialogResponse.FileName = this.textBoxFilename.Text + "." + this.comboBoxFileExtension.SelectedItem;
            dialogResponse.DirName = outputDir;
            this.Dispose();
        }

        private void textBoxFilename_TextChanged(object sender, EventArgs e)
        {
            if (this.textBoxFilename.Text.Trim().Length > 0 && this.textBoxExpDir.Text.Trim().Length > 0)
            {
                this.buttonExport.Enabled = true;
            }
            else this.buttonExport.Enabled = false;
        }
    }
}
