﻿using CimSyntaxGen.Utilities;
using EA;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TextBox;

namespace CimSyntaxGen.AsciiDoc
{
    public class AsciiDocIEC61970Documentation
    {
        private static Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);
        private EA.Repository repo;
        private EA.Element rootElem;
        private EA.Package selectedPackage;
        private AsciiDocGenerator asciiDocGenerator;

        public AsciiDocIEC61970Documentation(EA.Repository repo, EA.Package selectedPkg)
        {
            this.repo = repo;
            this.selectedPackage = selectedPkg;
            this.asciiDocGenerator = new AsciiDocGenerator(this.repo);
        }

        public static string GenerateCoverIntro_61970(EA.Repository repos, EA.Package pack, string outputDir)
        {
            AsciiDocGenerator asciiDocGenerator = new AsciiDocGenerator(repos);
            asciiDocGenerator.SaveCoverpageImage(outputDir);
            return asciiDocGenerator.GetCoverIntroAsciiDoc_61970(pack.Name);
        }


        public string GenerateDocu(string outputDir)
        {
            StringBuilder sb = new StringBuilder();
            string directives = asciiDocGenerator.GetAsciiDocDirectives();
            sb.Append(directives);

            ///// root element
            /*
            this.rootElem = asciiDocGenerator.GetRootElement(selectedPackage);
            if (this.rootElem == null)
            {
                ErrorCodes.ShowError(ErrorCodes.ERROR_051);
                return "";
            }
            */

            ///// start with diagrams
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();
            sb.Append(asciiDocStyle.GetHeader(selectedPackage.Name, 0));
            sb.Append(asciiDocStyle.GetHeader("Detailed Profile Specification", 1));
            sb.Append(asciiDocStyle.GetHeader("General", 2));
            sb.Append("This package contains the " + this.selectedPackage.Name.Replace("_", " ").Replace(".", " ") + ".\n");
            sb.Append(CreateAsciiDocAllDiagram(selectedPackage, outputDir));

            ///// classes and so on
            foreach (EA.Element elem in selectedPackage.Elements)
            {
                if (elem.Type.ToLower().Equals("class"))
                {
                    string title = GetElemTitle(elem, false);
                    sb.Append("[reftext=\"" + elem.Name + "\"]");
                    sb.Append(asciiDocStyle.GetHeader(title, 2));

                    List<EA.Element> parents = new List<EA.Element>();
                    GetParent(elem, parents);

                    string inheritancePath = GetInheritancePath(elem, parents);
                    if (inheritancePath.Length > 0)
                    {
                        sb.Append("Inheritance path = " + inheritancePath + " + \n");
                    }
                    sb.Append(elem.Notes.Replace("\n", "")).Append("\n");

                    //CheckInheritance(pack, elem);

                    // Attributes
                    if (elem.Attributes.Count > 0 || SumInheritedAttributes(parents) > 0)
                    {
                        sb.Append("\nThe following table shows all attributes of " + elem.Name + "." + "\n");
                        AsciiDocTable asciiDocTable = new AsciiDocTable("Attributes of " + selectedPackage.Name + "::" + elem.Name);
                        asciiDocTable.SetHeaderRow(new List<string>() { "Name", "Mult.", "Type", "Description" });
                        foreach (EA.Attribute attr in elem.Attributes)
                        {
                            asciiDocTable.AddRow(new List<string>() {attr.Name,
                                                                 attr.LowerBound + ".." + attr.UpperBound,
                                                                 attr.Type,
                                                                 attr.Notes.Replace("\n", "")});

                        }
                        foreach(EA.Element parent in parents)
                        {
                            foreach(EA.Attribute attr in parent.Attributes)
                            {
                                asciiDocTable.AddRow(new List<string>() {attr.Name,
                                                                 attr.LowerBound + ".." + attr.UpperBound,
                                                                 attr.Type,
                                                                 "inherited from: <<" + parent.Name + ">>"});
                            }
                        }
                        sb.Append(asciiDocTable.GetTable("2, 1, 2, 4", 100));
                    }

                    // Connectors / Associations /////////////////////
                    bool associationsExist = false;
                    List<string[]> conns = GetAssociations(elem);
                    if (conns != null && conns.Count > 0) associationsExist = true;

                    AsciiDocTable asciiDocConnTable = new AsciiDocTable("Association ends of " + selectedPackage.Name + "::" + elem.Name + " with other classes.");
                    asciiDocConnTable.SetHeaderRow(new List<string>() { "Mult.\nfrom", "Name", "Mult.\nto", "Type", "Description" });
                    foreach (string[] assocVals in conns)
                    {
                        //string[] assocVals = GetAssocValues(elem, conn);
                        asciiDocConnTable.AddRow(new List<string>() {assocVals[0], // mult.from
                                                                     assocVals[1], // name
                                                                     assocVals[2], // mult.to
                                                                     assocVals[3], // type
                                                                     assocVals[4].Replace("\n", "")});
                    }
                    foreach (EA.Element parent in parents)
                    {
                        List<string[]> inheritedConns = GetAssociations(parent);
                        if(inheritedConns != null && inheritedConns.Count > 0) associationsExist = true;

                        foreach (string[] assocVals in inheritedConns)
                        {
                            // assocVals := {multFrom, name, multTo, type, description}
                            //string[] assocVals = GetAssocValues(elem, conn);
                            //if (parent.Name.Equals(assocVals[3])) continue;
                            asciiDocConnTable.AddRow(new List<string>() {assocVals[0], // mult.from
                                                                     assocVals[1], // name
                                                                     assocVals[2], // mult.to
                                                                     assocVals[3], // type
                                                                     "inherited from: <<" + parent.Name + ">>"});
                        }
                    }
                    if(associationsExist)
                    {
                        sb.Append("\nThe following table shows assocation ends of " + elem.Name + " with other classes" + "\n");
                        sb.Append(asciiDocConnTable.GetTable("1, 3, 1, 3, 4", 100));
                    }
                }
            }

            return sb.ToString();
        }

        private string GetRefText(string title)
        {
            string[] parts = title.Split(' ');
            return parts[parts.Length - 1];
        }

        private int SumInheritedAttributes(List<EA.Element> parents)
        {
            int sum = 0;
            foreach(EA.Element parent in parents) sum += parent.Attributes.Count;
            return sum;
        }

        public string CreateAsciiDocAllDiagram(EA.Package pack, string outputDir)
        {
            AsciiDocStyle asciiDocStyle = new AsciiDocStyle();
            EA.Collection diagramCollection = repo.GetPackageByID(pack.PackageID).Diagrams;
            string diagFileName;
            int figureNr = 1;

            string imageDirName = outputDir + AsciiDocGenerator.IMAGE_DIR;
            if (!Directory.Exists(imageDirName))
            {
                Directory.CreateDirectory(imageDirName);
            }

            StringBuilder sb = new StringBuilder();
            foreach (EA.Diagram diag in diagramCollection)
            {
                diagFileName = diag.Name.Replace(" ", "_") + ".png";
                string diagramGUID = diag.DiagramGUID;
                EA.Project project = repo.GetProjectInterface();
                project.PutDiagramImageToFile(diagramGUID, imageDirName + "\\" + diagFileName, 1);
                AsciiDocImage asciiDocImage = new AsciiDocImage(diag.Name, diagFileName, figureNr, -1, -1);
                sb.Append(asciiDocImage.GetImage());
                sb.Append("Figure " + figureNr + ": " + diag.Notes.Replace("\n", ""));
                sb.Append("\n\n");
                figureNr++;
            }
            return sb.ToString();
        }

        /*
        private string[] GetAssocValues(EA.Element elem, EA.Connector conn)
        {
            string multFrom, multTo, name, type, description;
            EA.Element supplier = repo.GetElementByID(conn.SupplierID);
            EA.Element client = repo.GetElementByID(conn.ClientID);
            if (conn.ClientID == elem.ElementID)
            {
                multFrom = GetMultiplicity(conn.ClientEnd);
                multTo = GetMultiplicity(conn.SupplierEnd);
                name = conn.SupplierEnd.Role;
                type = supplier.Name;
                description = conn.SupplierEnd.RoleNote;
            }
            else
            {
                multFrom = GetMultiplicity(conn.SupplierEnd);
                multTo = GetMultiplicity(conn.ClientEnd);
                name = conn.ClientEnd.Role;
                type = client.Name;
                description = conn.ClientEnd.RoleNote;
            }
            return new string[] {multFrom, name, multTo, type, description};
        }
        */

        private string GetMultiplicity(EA.ConnectorEnd connEnd)
        {
            string card = connEnd.Cardinality;
            if (card == null) return "";
            else card = card.Trim();

            if (card.Equals("*") || card.Contains("..") || card.Equals("0")) return card;
            return card + ".." + card;
        }

        private List<string[]> GetAssociations(EA.Element elem)
        {
            List<string[]> conns = new List<string[]>();    
            foreach(EA.Connector conn in elem.Connectors)
            {
                //logger.Log("///// Element: " + elem.Name);
                //logger.Log("direction: " + conn.Direction + " type: " + conn.Type + " c: " + repo.GetElementByID(conn.ClientID).Name + " c: " + repo.GetElementByID(conn.SupplierID).Name);
                if(conn.Type.Equals("Association") && (!conn.Stereotype.Equals("IsBasedOn")))
                {
                    if(conn.Direction.Equals("Source -> Destination"))
                    {
                        if (elem.ElementID == conn.ClientID)
                        {
                            EA.Element supplier = repo.GetElementByID(conn.SupplierID);
                            string multFrom, multTo, name, type, description;
                            multFrom = GetMultiplicity(conn.ClientEnd);
                            multTo = GetMultiplicity(conn.SupplierEnd);
                            name = conn.SupplierEnd.Role;
                            type = supplier.Name;
                            description = conn.SupplierEnd.RoleNote;
                            conns.Add(new string[] { multFrom, name, multTo, type, description });
                        }
                    } else if(conn.Direction.Equals("Destination -> Source"))
                    {
                        if (elem.ElementID == conn.SupplierID)
                        {
                            EA.Element client = repo.GetElementByID(conn.ClientID);
                            string multFrom, multTo, name, type, description;
                            multFrom = GetMultiplicity(conn.SupplierEnd);
                            multTo = GetMultiplicity(conn.ClientEnd);
                            name = conn.ClientEnd.Role;
                            type = client.Name;
                            description = conn.ClientEnd.RoleNote;
                            conns.Add(new string[] { multFrom, name, multTo, type, description });
                        }
                    }
                }
            }
            return conns;
        }

        private string GetElemTitle(EA.Element elem, bool isRoot)
        {
            string prefix = "";
            if (elem.Abstract.Equals("1")) prefix += "Abstract";
            if (elem.StereotypeEx.Length > 0)
            {
                string[] stereos = elem.StereotypeEx.Split(',');
                foreach (string stereo in stereos)
                {
                    if (prefix.Length > 0) prefix += "," + stereo;
                    else prefix += stereo;
                }
            }
            string title = "";
            if(prefix.Length > 0) title = "(" + prefix + ") " + elem.Name;
            else title = elem.Name;

            if (isRoot) return title + " root class";
            return title;
        }

        private void CheckInheritance(EA.Package selPack, EA.Element elem)
        {
            foreach (EA.Connector conn in elem.Connectors)
            {
                if (conn.Type.Equals("Generalization"))
                {
                    EA.Element parent = null;
                    if (conn.ClientID == elem.ElementID)
                    {
                        parent = repo.GetElementByID(conn.SupplierID);
                    }
                    else
                    {
                        parent = repo.GetElementByID(conn.ClientID);
                    }
                    if (parent != null && IsInPackage(selPack, parent))
                    {
                        logger.Log(elem.Name + " -> parent.Name: " + parent.Name);
                    }
                }
            }
        }

        private bool IsInPackage(EA.Package pack, EA.Element elem)
        {
            foreach(EA.Element et in pack.Elements)
            {
                if(et.ElementID == elem.ElementID) return true;
            }
            return false;
        }

        private string GetInheritancePath(EA.Element elem, List<EA.Element> parents)
        {
            string inheritancePath = "";
            foreach (EA.Element parent in parents)
            {
                if (inheritancePath.Length > 0) inheritancePath += " : <<" + parent.Name + ">>";
                else inheritancePath += "<<" + parent.Name + ">>";
            }
            return inheritancePath;
        }

        private void GetParent(EA.Element elem, List<EA.Element> parents)
        {
            if (elem.Name.Equals("IdentifiedObject")) return;

            EA.Element parent = null;
            foreach (EA.Connector conn in elem.Connectors)
            {
                if (conn.Type.Equals("Generalization") && (conn.ClientID == elem.ElementID))
                {
                    if(parent != null)
                    {
                        ErrorCodes.ShowError(ErrorCodes.ERROR_055, elem.Name);
                        return;
                    }
                    parent = repo.GetElementByID(conn.SupplierID);
                }
            }
            if (parent != null)
            {
                if (parents.Contains(parent)) return;
                else parents.Add(parent);
                GetParent(parent, parents);
            }
            return;
        }


    }




}
