﻿using CimSyntaxGen.JsonGeneration;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CimSyntaxGen.AvroGeneration
{
    public class AvroEnum : JsonBase
    {
        private string name;
        private JsonObject<JsonBase> jObj = new JsonObject<JsonBase>("");
        private JsonArray<string> aliases = new JsonArray<string>();
        private JsonArray<string> symbols = new JsonArray<string>("symbols");

        public AvroEnum(string name)
        {
            this.name = name;
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            jObj.Add(new JsonKeyValue<string>("type", "enum"));
        }

        public AvroEnum(EA.Element elem, string nameSpace)
        {
            this.name = elem.Name;
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            jObj.Add(new JsonKeyValue<string>("type", "enum"));
            this.Namespace = nameSpace;
            this.SetDoc(elem.Notes);
            foreach(EA.Attribute attr in elem.Attributes)
            {
                this.AddSymbol(attr.Name);
            }
        }

        public string GetName()
        {
            return this.name;
        }

        public string Namespace
        {
            set { jObj.Add(new JsonKeyValue<string>("namespace", value)); }
        }

        public void SetDoc(string value)
        {
            value = (value != null) ? JsonGenerationUtilities.ReplaceLFCR(value) : "";
            jObj.Add(new JsonKeyValue<string>("doc", value));
        }

        public void AddAlias(string alias)
        {
            aliases.Add(alias);
        }

        public void AddSymbol(string symbol)
        {
            symbols.Add(symbol);
        }

        public string AsString()
        {
            if (jObj.GetList().Contains(aliases)) { jObj.GetList().Remove(aliases); }
            if (aliases.GetList().Count > 0) { jObj.Add(aliases); }

            if (jObj.GetList().Contains(symbols)) { jObj.GetList().Remove(symbols); }
            if (symbols.GetList().Count > 0) { jObj.Add(symbols); }

            return jObj.AsString();
        }

    }
}
