﻿using CimSyntaxGen.JsonGeneration;
using CimSyntaxGen.JsonMaster;
using CimSyntaxGen.Utilities;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace CimSyntaxGen.AvroGeneration
{
    public class AvroField: JsonBase
    {
        private string name;
        private JsonObject<JsonBase> jObj = new JsonObject<JsonBase>("");
        private Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);
        public static string LOGICAL_TYPE = "logicalType";


        public AvroField(string name, string type, string defaultValue, string doc) 
        {
            this.name = name;
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            if (doc != null && doc.Length > 0)
            {
                doc = JsonGenerationUtilities.ReplaceLFCR(doc);
                jObj.Add(new JsonKeyValue<string>("doc", doc));
            }
            if (defaultValue != null && defaultValue.Length > 0)
            {
                jObj.Add(new JsonKeyValue<string>("default", defaultValue));
            }
            jObj.Add(new JsonKeyValue<string>("type", type));
        }

        public AvroField(string name, string type, string logicalType, string defaultValue, string doc)
        {
            this.name = name;
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            if (doc != null && doc.Length > 0)
            {
                doc = JsonGenerationUtilities.ReplaceLFCR(doc);
                jObj.Add(new JsonKeyValue<string>("doc", doc));
            }
            if (defaultValue != null && defaultValue.Length > 0)
            {
                jObj.Add(new JsonKeyValue<string>("default", defaultValue));
            }
            if(logicalType != null && logicalType.Length > 0)
            {
                jObj.Add(new JsonKeyValue<JsonBase>("type", GetLogicalType(type, logicalType)));
                logger.Log("AvroField: " + jObj.AsString());
            }
            else
            {
                jObj.Add(new JsonKeyValue<string>("type", type));
            }
        }

        public AvroField(string name, JsonBase type, string defaultValue, string doc)
        {
            this.name = name;
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            if (doc != null && doc.Length > 0)
            {
                doc = JsonGenerationUtilities.ReplaceLFCR(doc);
                jObj.Add(new JsonKeyValue<string>("doc", doc));
            }
            jObj.Add(new JsonKeyValue<JsonBase>("type", type));
            if (defaultValue != null && defaultValue.Length > 0)
            {
                jObj.Add(new JsonKeyValue<string>("default", defaultValue));
            }
        }

        public AvroField(string name, JsonArray<JsonBase> arrayType, string defaultValue, string doc)
        {
            this.name = name;
            arrayType.SetName("type");
            jObj.Add(new JsonKeyValue<string>("name", AvroRecord.GetAvroCaseName(name)));
            if (doc != null && doc.Length > 0)
            {
                doc = JsonGenerationUtilities.ReplaceLFCR(doc);
                jObj.Add(new JsonKeyValue<string>("doc", doc));
            }
            if (defaultValue != null && defaultValue.Length > 0)
            {
                jObj.Add(new JsonKeyValue<string>("default", defaultValue));
            }
            jObj.Add(arrayType);
        }

        public static JsonBase GetLogicalType(string type, string logicalType)
        {
            JsonObject<string> logJObj = new JsonObject<string>(null);
            logJObj.Add(new JsonKeyValue<string>("type", type));
            logJObj.Add(new JsonKeyValue<string>(LOGICAL_TYPE, logicalType));
            return logJObj;
        }

        public string GetName()
        {
            return this.name;
        }

        public void AddDefault(JsonBase defaultValue)
        {
            jObj.Add(new JsonKeyValue<JsonBase>("default", defaultValue));
        }

        public string AsString()
        {
            return jObj.AsString();
        }


    }
}
