﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.AvroGeneration
{
    public partial class AvroGenDialog : Form
    {
        private EA.Package pack;
        private XMLParser xmlp;
        private DialogResponse dialogResponse;

        public AvroGenDialog(EA.Repository repo, EA.Package pack, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.dialogResponse = dialogResponse;
            xmlp = new XMLParser(repo);
            this.pack = pack;
            this.textBoxFilename.Text = pack.Name.Replace(" ", "-") + AvroMapper.AVRO_FILE_EXTENSION;
            this.textBoxNamespace.Text = xmlp.GetXmlValueAvroData(XMLParser.LAST_AVRO_NAMESPACE);
            this.textBoxOutputDirectory.Text = xmlp.GetXmlValueAvroData(XMLParser.LAST_AVRO_OUTPUT_DIRECTORY);
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            xmlp.SetXmlValueAvroData(XMLParser.LAST_AVRO_NAMESPACE, this.textBoxNamespace.Text);
            string outputDir = this.textBoxOutputDirectory.Text;
            xmlp.SetXmlValueAvroData(XMLParser.LAST_AVRO_OUTPUT_DIRECTORY, outputDir);
            if (!outputDir.EndsWith("\\")) outputDir = outputDir + "\\";

            dialogResponse.Canceled = false;
            dialogResponse.FileName = outputDir + this.textBoxFilename.Text;
            dialogResponse.NameSpace = this.textBoxNamespace.Text;

            this.Dispose();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            dialogResponse.Canceled = true;
            this.Dispose();
        }

        private void buttonOutputDirectory_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.textBoxOutputDirectory.Text;
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxOutputDirectory.Text = folderBrowserDialog.SelectedPath;
            }
            folderBrowserDialog.Dispose();
        }

        private void textBoxOutputDirectory_TextChanged(object sender, EventArgs e)
        {
            if(this.textBoxFilename.Text.Trim().Length > 0 && this.textBoxOutputDirectory.Text.Trim().Length > 0)
            {
                this.buttonCreate.Enabled = true;
            } else this.buttonCreate.Enabled = false;
        }

        private void textBoxFilename_TextChanged(object sender, EventArgs e)
        {
            if (this.textBoxFilename.Text.Trim().Length > 0 && this.textBoxOutputDirectory.Text.Trim().Length > 0)
            {
                this.buttonCreate.Enabled = true;
            }
            else this.buttonCreate.Enabled = false;
        }
    }
}
