﻿using CimSyntaxGen.JsonGeneration;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CimSyntaxGen.AvroGeneration
{
    public class AvroRecord: JsonBase
    {
        private JsonArray<string> aliases = new JsonArray<string>("aliases");
        private JsonArray<JsonBase> fields = new JsonArray<JsonBase>("fields");
        private string name;
        private JsonObject<JsonBase> jObj = new JsonObject<JsonBase>("");

        public AvroRecord(string name)
        {
            this.name = name;
            jObj.Add(new JsonKeyValue<string>("name", name));
            jObj.Add(new JsonKeyValue<string>("type", AvroTypes.RECORD));
        }

        public AvroRecord(EA.Attribute attr, string nameSpace)
        {
            this.name = GetAvroCaseName(attr.Name);
            jObj.Add(new JsonKeyValue<string>("name", name));
            jObj.Add(new JsonKeyValue<string>("type", AvroTypes.RECORD));
            this.Namespace = nameSpace;
            this.SetDoc(attr.Notes);
        }

        public AvroRecord(EA.Attribute attr, JsonBase type, string nameSpace)
        {
            this.name = GetAvroCaseName(attr.Name);
            jObj.Add(new JsonKeyValue<string>("name", name));
            // JsonObject<JsonBase> jObjType = new JsonObject<JsonBase>("type");
            // jObjType.Add(type);
            // jObj.Add(jObjType);
            JsonKeyValue<JsonBase> jKVType = new JsonKeyValue<JsonBase>("type", type);
            jObj.Add(jKVType);
            this.Namespace = nameSpace;
            this.SetDoc(attr.Notes);
        }

        public AvroRecord(EA.Element elem, string nameSpace)
        {
            this.name = GetAvroCaseName(elem.Name);
            jObj.Add(new JsonKeyValue<string>("name", name));
            jObj.Add(new JsonKeyValue<string>("type", AvroTypes.RECORD));
            this.Namespace = nameSpace;
            this.SetDoc(elem.Notes);
        }

        public AvroRecord(EA.Package pack, string nameSpace)
        {
            this.name = pack.Name;
            jObj.Add(new JsonKeyValue<string>("name", GetAvroCaseName(name)));
            jObj.Add(new JsonKeyValue<string>("type", AvroTypes.RECORD));
            this.Namespace = nameSpace;
            this.SetDoc(pack.Notes);
        }

        public string GetName()
        {
            return this.name;
        }

        public static string GetAvroCaseName(string name)
        {
            char BLANK = ' ';
            string pascal = "";
            string tmp = name.Replace('_', BLANK).Replace('-', BLANK);
            string[] parts = tmp.Split(BLANK);
            for(int n = 0; n < parts.Length; n++)
            {
                string trimmedPart = parts[n].Trim();
                if (trimmedPart.Length > 0)
                {
                    if(n > 0) trimmedPart = char.ToUpper(trimmedPart[0]) + trimmedPart.Substring(1);
                    pascal += trimmedPart;
                }
            }
            return pascal;
        }

        public string Namespace 
        { 
            set { jObj.Add(new JsonKeyValue<string>("namespace", value)); }
        }

        public void SetDoc(string value)
        {
            value = (value != null) ? JsonGenerationUtilities.ReplaceLFCR(value) : "";
            jObj.Add(new JsonKeyValue<string>("doc", value));
        }

        public void AddAlias(string alias)
        {
            aliases.Add(alias);
        }

        public void AddFields(JsonBase item)
        {
            fields.Add(item);
        }

        public JsonArray<JsonBase> GetFields()
        {
            return fields;
        }

        public string AsString()
        {
            if (jObj.GetList().Contains(aliases)) { jObj.GetList().Remove(aliases); }
            if(aliases.GetList().Count > 0) jObj.Add(aliases);
            
            if (jObj.GetList().Contains(fields)) { jObj.GetList().Remove(fields); }
            jObj.Add(fields);

            return jObj.AsString();
        }

    }
}
