﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TextBox;

namespace CimSyntaxGen.Configuration
{
    public partial class ConfigurationEditor : Form
    {
        private EA.Repository repo;
        private ConfigurationManager configManager;
        //private static string PARAM_CONFIGURATION = "configuration";
        //private string[] validXmlParameter = { PARAM_CONFIGURATION, "profdata", "shacldata", "profstereo", "jsondata", "avrodata", "asciidocdata" };
        private Dictionary<string, Dictionary<string, string>> valueDictionary = new Dictionary<string, Dictionary<string, string>>();
        private readonly string NAME = "name";
        private readonly string ATTRIBUTE_LABEL = "Attribute: ---";
        private string selectedNodeText = null;


        public ConfigurationEditor(EA.Repository repo)
        {
            InitializeComponent();
            this.repo = repo;
            this.configManager = ConfigurationManager.GetConfigurationManager(repo);
            this.dataGridViewAttributes.DefaultCellStyle.WrapMode = DataGridViewTriState.True;
            this.labelAttributes.Text = ATTRIBUTE_LABEL;
            LoadParameter();
        }

        private void LoadParameter()
        {
            string configXMLString = this.configManager.GetCustomConfiguration();
            if(configXMLString == null ) 
            {
                configXMLString = this.configManager.LoadDefaultConfiguration();
            }
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(configXMLString);
            TreeNode mainNode = new TreeNode();
            mainNode.Name = "mainNode" + xmlDoc.DocumentElement.Name;
            mainNode.Text = xmlDoc.DocumentElement.Name;

            foreach(XmlNode xmlNode in xmlDoc.DocumentElement.ChildNodes)
            {
                if (xmlNode.NodeType == XmlNodeType.Element)
                {
                    TreeNode node = new TreeNode();
                    node.Name = "node" + xmlNode.Name;
                    node.Text = xmlNode.Name;
                    foreach (XmlNode xmlNode2 in xmlNode.ChildNodes)
                    {
                        if (ConfigurationManager.validXmlParameter.Contains(xmlNode2.Name))
                        {
                            TreeNode node2 = new TreeNode();
                            string prefix = xmlNode2.Name;

                            string attrName = "";
                            Dictionary<string, string> attrValueDictionary = new Dictionary<string, string>();
                            foreach(XmlAttribute xmlAttr in xmlNode2.Attributes)
                            {
                                if(xmlAttr.Name.Equals(NAME))
                                {
                                    attrName = xmlAttr.Value;
                                }
                                attrValueDictionary.Add(xmlAttr.Name, xmlAttr.Value);
                            }
                            node2.Text = prefix + "." + attrName;
                            valueDictionary.Add(node2.Text, attrValueDictionary);
                            node2.Name = "node2" + node2.Text;
                            node.Nodes.Add(node2);
                        }
                    }
                    mainNode.Nodes.Add(node);
                }
            }

            this.treeViewParameters.Nodes.Add(mainNode);
        }


        private void buttonSave_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = MessageBox.Show("Do you really want to save the changes?", "Confirmation", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes)
            {
                string xmlString = CreateXML(this.valueDictionary);
                configManager.StoreImportedConfiguration(xmlString);
                //using (StreamWriter w = System.IO.File.AppendText("C:\\EA_CIM_Support\\test_config.xml"))
                //{
                //    w.Write(xmlString);
                //}
                this.Dispose();
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void treeViewParameters_AfterSelect(object sender, TreeViewEventArgs e)
        {
            this.dataGridViewAttributes.Rows.Clear();
            this.selectedNodeText = this.treeViewParameters.SelectedNode.Text;
            string prefix = GetPrefix(this.selectedNodeText);
            if (prefix != null)
            {
                this.labelAttributes.Text = "Attribute: " + this.selectedNodeText;
                Dictionary<string, string> attrDict = valueDictionary[this.selectedNodeText];
                foreach (KeyValuePair<string, string> keyVal in attrDict)
                {
                    string key = keyVal.Key;
                    string value = keyVal.Value;
                    if (!key.Equals(NAME))
                    {
                        this.dataGridViewAttributes.Rows.Add(key, value);
                    }
                }
            }
            else this.labelAttributes.Text = ATTRIBUTE_LABEL;
        }

        private string GetPrefix(string nodeText)
        {
            int pos = nodeText.IndexOf('.');
            if (pos == -1) return null;
            return nodeText.Substring(0, pos);
        }

        private void dataGridViewAttributes_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            //string msg = String.Format("Cell at row {0}, column {1} value changed", e.RowIndex, e.ColumnIndex);
            //MessageBox.Show(msg);
            if((this.selectedNodeText != null) && (e.RowIndex >= 0))
            {
                string attrName = (string)this.dataGridViewAttributes.Rows[e.RowIndex].Cells[0].Value;
                if (attrName != null)
                {
                    valueDictionary[this.selectedNodeText][attrName] = (string)this.dataGridViewAttributes.Rows[e.RowIndex].Cells[1].Value;
                }
            }
        }

        private string CreateXML(Dictionary<string, Dictionary<string, string>> valueDictionary)
        {
            XmlDocument doc = new XmlDocument();
            XmlDeclaration xmlDeclaration = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            XmlElement root = doc.DocumentElement;
            doc.InsertBefore(xmlDeclaration, root);

            XmlElement elemAddIn = doc.CreateElement("add-in");
            XmlElement elemAppSettings = doc.CreateElement("appSettings");
            XmlElement elemDataProfile = doc.CreateElement("dataProfile");
            foreach (KeyValuePair<string, Dictionary<string, string>> keyVal in valueDictionary)
            {
                if(keyVal.Key.StartsWith(ConfigurationManager.PARAM_CONFIGURATION))
                {
                    AppendElem(doc, elemAppSettings, ConfigurationManager.PARAM_CONFIGURATION, keyVal.Value);
                }
                else 
                {
                    AppendElem(doc, elemDataProfile, GetPrefix(keyVal.Key), keyVal.Value);
                }
            }
            elemAddIn.AppendChild(elemAppSettings);
            elemAddIn.AppendChild(elemDataProfile);
            doc.AppendChild(elemAddIn);
            StringWriterWithEncoding sw = new StringWriterWithEncoding(new StringBuilder(""), Encoding.UTF8);
            doc.Save(sw);
            return sw.ToString();
        }

        private void AppendElem(XmlDocument doc, XmlElement parent, string elemName, Dictionary<string, string> attrs)
        {
            try
            {
                XmlElement elem = doc.CreateElement(elemName);
                Dictionary<string, string> values = attrs;
                foreach (KeyValuePair<string, string> keyVal2 in values)
                {
                    XmlAttribute attribute = doc.CreateAttribute(keyVal2.Key);
                    attribute.Value = keyVal2.Value;
                    elem.SetAttributeNode(attribute);
                }
                parent.AppendChild(elem);
            } catch(Exception e)
            {
                MessageBox.Show("elemName = " + elemName + "\n" + e.StackTrace);
            }
        }
    }
}
