﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.JsonGeneration
{
    public partial class DialCodelistExportToJson : Form
    {
        private XMLParser xmlp;
        private DialogResponse dialogResponse;
        public const string LASTJSONCODELIST_EXPORT_DIRECTORY = "LastJsonCodelistExportDirectory";
        public const string LASTJSONCODELIST_EXPORT_FILENAME = "LastJsonCodelistExportFilename";
        public const string JSONCODELIST_SCHEMA_ID = "JsonCodelistSchemaID";
        public const string JSONCODELIST_SCHEMA_REF = "JsonSchemaVersion";

        public DialCodelistExportToJson(EA.Repository repo, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.dialogResponse = dialogResponse;
            xmlp = new XMLParser(repo);
            this.textBoxDir.Text = xmlp.GetXmlValueJsonData(LASTJSONCODELIST_EXPORT_DIRECTORY);
            this.textBoxFilename.Text = xmlp.GetXmlValueJsonData(LASTJSONCODELIST_EXPORT_FILENAME);
            this.textBoxSchemaId.Text = xmlp.GetXmlValueJsonData(JSONCODELIST_SCHEMA_ID);
            this.textBoxSchemaRef.Text = xmlp.GetXmlValueJsonData(JSONCODELIST_SCHEMA_REF);
            
            string localCodelistDir = xmlp.GetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_DIRECTORY);
            string localCodelistFilename = xmlp.GetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_FILENAME);
            if(!localCodelistDir.EndsWith("\\")) localCodelistDir += "\\";
            this.textBoxLocalCodelist.Text = localCodelistDir + localCodelistFilename;

            if(this.textBoxSchemaId.Text != null && this.textBoxSchemaId.Text.Trim().Length <= 0)
            {
                this.textBoxSchemaId.Text = CreateSchemaID();
            }

            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;

            if (EnableJsonIDTextBox()) this.textBoxSchemaId.Enabled = true;
            else this.textBoxSchemaId.Enabled = false;
        }

        private void buttonDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.textBoxDir.Text;
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxDir.Text = folderBrowserDialog.SelectedPath;
            }
            folderBrowserDialog.Dispose();
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;

            if (EnableJsonIDTextBox())
            {
                this.textBoxSchemaId.Enabled = true;
                this.textBoxSchemaId.Text = CreateSchemaID();
            }
            else this.textBoxSchemaId.Enabled = false;
        }

        private bool EnableExportButton()
        {
            return ((this.textBoxDir.Text != null && this.textBoxDir.Text.Trim().Length > 0) &&
                    (this.textBoxFilename.Text != null && this.textBoxFilename.Text.Trim().Length > 0) &&
                    (this.textBoxSchemaId.Text != null && this.textBoxSchemaId.Text.Trim().Length > 0) &&
                    (this.textBoxLocalCodelist.Text != null && this.textBoxLocalCodelist.Text.Trim().Length > 0));
        }

        private bool EnableJsonIDTextBox()
        {
            return ((this.textBoxDir.Text != null && this.textBoxDir.Text.Trim().Length > 0) &&
                    (this.textBoxFilename.Text != null && this.textBoxFilename.Text.Trim().Length > 0));
        }

        private string CreateSchemaID()
        {
            string outputDir = this.textBoxDir.Text;
            if (!outputDir.EndsWith("\\")) outputDir = outputDir + "\\";
            string expFileName = JsonGenerationUtilities.GetFileWithExtension(this.textBoxFilename.Text);
            expFileName = expFileName.Replace(' ', '_');
            string fullFileName = (outputDir + expFileName).Replace("\\", "/");
            return (@"file:///" + fullFileName);
        }


        private void textBoxFilename_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;

            if (EnableJsonIDTextBox())
            {
                this.textBoxSchemaId.Enabled = true;
                this.textBoxSchemaId.Text = CreateSchemaID();
            }
            else this.textBoxSchemaId.Enabled = false;
        }

        private void buttonExport_Click(object sender, EventArgs e)
        {
            string outputDir = this.textBoxDir.Text;
            if (!outputDir.EndsWith("\\")) outputDir = outputDir + "\\";

            string expFileName = JsonGenerationUtilities.GetFileWithExtension(this.textBoxFilename.Text);
            expFileName = expFileName.Replace(' ', '_');
            xmlp.SetXmlValueJsonData(LASTJSONCODELIST_EXPORT_DIRECTORY, outputDir);
            xmlp.SetXmlValueJsonData(LASTJSONCODELIST_EXPORT_FILENAME, expFileName);
            xmlp.SetXmlValueJsonData(JSONCODELIST_SCHEMA_ID, this.textBoxSchemaId.Text);
            xmlp.SetXmlValueJsonData(JSONCODELIST_SCHEMA_REF, this.textBoxSchemaRef.Text);
            xmlp.SetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_DIRECTORY, Path.GetDirectoryName(this.textBoxLocalCodelist.Text));
            xmlp.SetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_FILENAME, Path.GetFileName(this.textBoxLocalCodelist.Text));

            dialogResponse.Canceled = false;
            dialogResponse.FileName = expFileName;
            dialogResponse.DirName = outputDir;
            this.Dispose();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            dialogResponse.Canceled = true;
            this.Dispose();
        }

        private void textBoxSchemaId_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void textBoxSchemaRef_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void textBoxDir_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void textBoxLocalCodelist_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void buttonLocalCodelist_Click(object sender, EventArgs e)
        {
            this.openFileDialog.Filter = "All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxLocalCodelist.Text = openFileDialog.FileName;
            }
        }
    }
}
