﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.JsonGeneration
{
    public partial class DialExportCodelistToJsonTemplate : Form
    {
        private DialogResponse dialogResponse;
        private XMLParser xmlp;
        public const string LASTJSONLOCALCODELIST_EXPORT_DIRECTORY = "LastJsonLocalCodelistExportDirectory";
        public const string LASTJSONLOCALCODELIST_EXPORT_FILENAME = "LastJsonLocalCodelistExportFilename";


        public DialExportCodelistToJsonTemplate(EA.Repository repo, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.dialogResponse = dialogResponse;
            xmlp = new XMLParser(repo);
            this.textBoxDir.Text = xmlp.GetXmlValueJsonData(LASTJSONLOCALCODELIST_EXPORT_DIRECTORY);
            this.textBoxFilename.Text = xmlp.GetXmlValueJsonData(LASTJSONLOCALCODELIST_EXPORT_FILENAME);
            this.textBoxCodelistRef.Text = xmlp.GetXmlValueJsonData("JsonSchemaVersion");
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
            dialogResponse.Canceled = false;
        }

        private bool EnableExportButton()
        {
            return ((this.textBoxDir.Text != null && this.textBoxDir.Text.Trim().Length > 0) &&
                    (this.textBoxCodelistRef.Text != null && this.textBoxCodelistRef.Text.Trim().Length > 0) &&
                    (this.textBoxFilename.Text != null && this.textBoxFilename.Text.Trim().Length > 0));
        }


        private void buttonDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = this.textBoxDir.Text;
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxDir.Text = folderBrowserDialog.SelectedPath;
            }
            folderBrowserDialog.Dispose();
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void textBoxDir_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;

        }

        private void textBoxFilename_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }

        private void buttonExport_Click(object sender, EventArgs e)
        {
            string outputDir = this.textBoxDir.Text;
            if (!outputDir.EndsWith("\\")) outputDir = outputDir + "\\";

            string expFileName = JsonGenerationUtilities.GetFileWithExtension(this.textBoxFilename.Text);
            expFileName = expFileName.Replace(' ', '_');
            xmlp.SetXmlValueJsonData(LASTJSONLOCALCODELIST_EXPORT_DIRECTORY, outputDir);
            xmlp.SetXmlValueJsonData(LASTJSONLOCALCODELIST_EXPORT_FILENAME, expFileName);
            xmlp.SetXmlValueJsonData("JsonSchemaVersion", this.textBoxCodelistRef.Text);
            dialogResponse.Canceled = false;
            dialogResponse.FileName = expFileName;
            dialogResponse.DirName = outputDir;
            this.Dispose();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            dialogResponse.Canceled = true;
            this.Dispose();
        }

        private void textBoxCodelistRef_TextChanged(object sender, EventArgs e)
        {
            if (EnableExportButton()) this.buttonExport.Enabled = true;
            else this.buttonExport.Enabled = false;
        }
    }
}
