﻿using CimSyntaxGen.Utilities;
using CimSyntaxGen.JsonMaster;
using System;
using System.Collections.Generic;
using System.Runtime.CompilerServices;
using System.Text;
using System.Text.Json;
using System.Windows.Forms;
using System.Linq;
using System.IO;

namespace CimSyntaxGen.JsonGeneration
{
    public class JsonGenerationUtilities
    {
        private static Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);

        public string GetFacet(string name)
        {
            if (name.Equals("length")) return ("");
            if (name.Equals("minLength")) return ("minLength");
            if (name.Equals("maxLength")) return ("maxLength");
            if (name.Equals("pattern")) return ("pattern");
            if (name.Equals("minInclusive")) return ("minimum");
            if (name.Equals("maxInclusive")) return ("maximum");
            if (name.Equals("maxExclusive")) return ("exclusiveMaximum");
            if (name.Equals("minExclusive")) return ("exclusiveMinimum");
            if (name.Equals("enumeration")) return ("enum");
            if (name.Equals("multipleOf")) return ("multipleOf");
            return "";
        }

        public static bool IsBasicType(string name)
        {
            return (name.Equals("Boolean") ||
                    name.Equals("Decimal") ||
                    name.Equals("Float") ||
                    name.Equals("Integer") ||
                    name.Equals("Double") ||
                    name.Equals("String") ||
                    name.Equals("DateTime") ||
                    name.Equals("Date") ||
                    name.Equals("Duration") ||
                    name.Equals("Time") ||
                    name.Equals("MonthDay"));
        }

        public static List<JsonKeyValue<string>> GetBasicType(string name)
        {
            List<JsonKeyValue<string>> retList = new List<JsonKeyValue<string>>();
            if (name.Equals("Boolean"))
            {
                retList.Add(new JsonKeyValue<string>("type", "boolean"));
            }
            else if (name.Equals("Decimal"))
            {
                retList.Add(new JsonKeyValue<string>("type", "number"));
            }
            else if (name.Equals("Float"))
            {
                retList.Add(new JsonKeyValue<string>("type", "number"));
            }
            else if (name.Equals("Integer"))
            {
                retList.Add(new JsonKeyValue<string>("type", "integer"));
            }
            else if (name.Equals("Double"))
            {
                retList.Add(new JsonKeyValue<string>("type", "number"));
            }
            else if (name.Equals("String"))
            {
                retList.Add(new JsonKeyValue<string>("type", "string"));
            }
            else if (name.Equals("DateTime"))
            {
                JsonKeyValue<string> typeKV = new JsonKeyValue<string>("type", "string");
                string pattern = @"^(-?([1-9][0-9]{3,}|0[0-9]{3})-(0[1-9]|1[0-2])-(0[1-9]|[12][0-9]|3[01])T(([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]" + 
                    @"(\\.[0-9]+)?|(24:00:00(\\.0+)?))(Z|(\\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?)$";
                JsonKeyValue<string> patternKV = new JsonKeyValue<string>("pattern", RemoveCRLF(pattern));
                retList.Add(typeKV);
                retList.Add(patternKV);
            }
            else if (name.Equals("Date"))
            {
                JsonKeyValue<string> typeKV = new JsonKeyValue<string>("type", "string");
                JsonKeyValue<string> patternKV = new JsonKeyValue<string>("pattern", @"^(-?([1-9][0-9]{3,}|0[0-9]{3})-(0[1-9]|1[0-2])-(0[1-9]|" + 
                    @"[12][0-9]|3[01])(Z|(\\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?)$");
                retList.Add(typeKV);
                retList.Add(patternKV);
            }
            else if (name.Equals("Duration"))
            {
                JsonKeyValue<string> typeKV = new JsonKeyValue<string>("type", "string");
                JsonKeyValue<string> patternKV = new JsonKeyValue<string>("pattern", @"^(-?)P(?=\\d|T\\d)(?:(\\d+)Y)?(?:(\\d+)M)?(?:(\\d+)" + 
                    @"([DW]))?(?:T(?:(\\d+)H)?(?:(\\d+)M)?(?:(\\d+(?:\\.\\d+)?)S)?)?$");
                retList.Add(typeKV);
                retList.Add(patternKV);
            }
            else if (name.Equals("Time"))
            {
                JsonKeyValue<string> typeKV = new JsonKeyValue<string>("type", "string");
                string pattern = @"^((([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]" + 
                    @"(\\.[0-9]+)?|(24:00:00(\\.0+)?))(Z|(\\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?)$";
                JsonKeyValue<string> patternKV = new JsonKeyValue<string>("pattern", RemoveCRLF(pattern));
                retList.Add(typeKV);
                retList.Add(patternKV);
            }
            else if (name.Equals("MonthDay"))
            {
                JsonKeyValue<string> typeKV = new JsonKeyValue<string>("type", "string");
                string pattern = @"^(--(((02)-(0[1-9]|[1][0-9]|2[0-9]))|((0[4689]|(11))-" + 
                    @"(0[1-9]|[1][0-9]|2[0-9]|(30)))|((0[13578]|(1[02]))-(0[1-9]|[1][0-9]|2[0-9]|(3[0-1])))))$";
                JsonKeyValue<string> patternKV = new JsonKeyValue<string>("pattern", RemoveCRLF(pattern));
                retList.Add(typeKV);
                retList.Add(patternKV);
            }
            return retList;
        }

        private static string RemoveCRLF(string str)
        {
            str = str.Replace("\n", "");
            str = str.Replace("\r", "");
            return str;
        }

        public static string GetFileWithExtension(string filename)
        {
            string JSON_EXT = ".json";
            if (filename.ToLower().EndsWith(".json")) return filename;
            return filename + JSON_EXT;
        }
            
        public static string GetFileURIFromFullFilename(string fullFilename)
        {
            //string[] parts = fullFilename.Split(':');
            //if(parts.Length <= 1) return parts[0].Replace('\\', '/');
            //else return "file://" + parts[1].Replace('\\', '/');
            return "file:///" + fullFilename.Replace('\\', '/');
        }

        public void GetAllPackages(EA.Package superPackage, List<EA.Package> packList)
        {
            packList.Add(superPackage);
            foreach(EA.Package pack in superPackage.Packages)
            {
                GetAllPackages(pack, packList);
            }
        }

        // rootElements ... output, contains list of root elements
        public List<EA.Element> GetAllElements(EA.Repository repo, EA.Package superPackage, List<EA.Element> rootElements, List<string> classifierAttrNameList)
        {
            List<EA.Element> elemList = new List<EA.Element>();
            List<EA.Package> packList = new List<EA.Package>();
            GetAllPackages(superPackage, packList);
            foreach(EA.Package pack in packList)
            {
                foreach(EA.Element elem in pack.Elements)
                {
                    if ((!elem.Stereotype.Contains("Primitive")) && 
                       (!elemList.Contains(elem)))
                    {
                        if(elem.IsActive) // root element
                        {
                            rootElements.Add(elem);
                            foreach(EA.Attribute attr in elem.Attributes)
                            {
                                if (attr.ClassifierID > 0)
                                {
                                    EA.Element classifier = repo.GetElementByID(attr.ClassifierID);
                                    if ((classifier != null) &&
                                        (!classifierAttrNameList.Contains(classifier.Name)) &&
                                        (!IsBasicType(classifier.Name)))
                                    {
                                        elemList.Add(classifier);
                                        classifierAttrNameList.Add(classifier.Name);
                                    }
                                }
                            }
                        } else elemList.Add(elem);
                    }
                }
            }
            return elemList;
        }

        public List<EA.Element> GetAllLinkedAttributes(EA.Repository repo, EA.Package superPackage, List<string> classifierAttrNameList)
        {
            List<EA.Element> linkedAttrList = new List<EA.Element>();
            List<EA.Element> rootElements = new List<EA.Element>();
            List<EA.Element> elemList = GetAllElements(repo, superPackage, rootElements, classifierAttrNameList);
            foreach (EA.Element elem in elemList)
            {
                foreach (EA.Attribute attr in elem.Attributes)
                {
                    if(attr.ClassifierID > 0)
                    {
                        EA.Element linkedAttr = repo.GetElementByID(attr.ClassifierID);
                        if((linkedAttr != null) && 
                           (!classifierAttrNameList.Contains(linkedAttr.Name)) &&
                           (!IsBasicType(linkedAttr.Name)))
                        {
                            linkedAttrList.Add(linkedAttr);
                            classifierAttrNameList.Add(linkedAttr.Name);
                        }
                    }
                }
            }
            return linkedAttrList;
        }

        public static string RemoveQual(string name)
        {
            if(!name.Contains("#"))
            {
                return null; // error!
            }
            int lastInd = name.LastIndexOf("#");
            int underLineInd = name.IndexOf("_"); // if -1, we keep it
            if (underLineInd < 0) return name;
            else return name.Substring(0, lastInd + 1) + name.Substring(underLineInd + 1, name.Length - underLineInd - 1);
        }

        // Returns list of long arrays. Each long array contains all IDs of the participating 
        // connectors.
        public static List<long[]> GetXORConns(EA.Package package)
        {
            List<long[]> resultList = new List<long[]>();
            foreach(EA.Element elem in package.Elements)
            {
                if (elem.Type.Equals("Constraint") && elem.Notes.Equals("XOR"))
                {
                    List<long> refIds = new List<long>();
                    string miscData = elem.MiscData[3]; //e.g. "idref1=62278;idref2=62279;"
                    miscData = miscData.TrimEnd(';');
                    string[] parts = miscData.Split(';');
                    foreach(string part in parts)
                    {
                        string[] subParts = part.Trim().Split('='); // e.g. "idref2=456"
                        refIds.Add(Convert.ToInt64(subParts[1].Trim())); 
                    }
                    resultList.Add(refIds.ToArray());
                }
            }
            return resultList;
        }

        private static List<EA.Connector> GetAllConnectorsOfElements(EA.Package package)
        {
            List<EA.Connector> listOfConns = new List<EA.Connector>();
            foreach(EA.Element element in package.Elements)
            {
                foreach(EA.Connector connector in element.Connectors)
                {
                    if(!listOfConns.Contains(connector))
                    {
                        listOfConns.Add(connector);
                    }
                }
            }
            return listOfConns;
        }

        public static List<JsonMaster.XOROwner> GetXOROwners(EA.Repository repo, EA.Package package, List<long[]> xorConnsList)
        {
            List<JsonMaster.XOROwner> resList = new List<JsonMaster.XOROwner>();
            if(xorConnsList == null || xorConnsList.Count <= 0) return resList; // nothing to do

            // Get all connectors in hashtable to speed up
            Dictionary<long, EA.Connector> connDic = new Dictionary<long, EA.Connector>();
            List<EA.Connector> listOfConnectors = GetAllConnectorsOfElements(package);
            foreach (EA.Connector conn in listOfConnectors)
            {
                if(!connDic.ContainsKey(conn.ConnectorID))
                {
                    connDic.Add(conn.ConnectorID, conn);
                }
            }

            
            foreach (long[] xors in xorConnsList) // thru xor groups
            {
                int firstElemID = -1;
                int secondElemID = -1;
                int ownerID = -1;
                foreach(long xorConnID in xors)
                {
                    EA.Connector conn = connDic[xorConnID];
                    if (firstElemID >= 0)
                    {
                        if (conn.ClientID == firstElemID || conn.SupplierID == firstElemID)
                        {
                            ownerID = firstElemID;
                        } else ownerID = secondElemID;
                        break;
                    } else
                    {
                        firstElemID = conn.ClientID;
                        secondElemID = conn.SupplierID;
                    }
                }
                EA.Element owner = repo.GetElementByID(ownerID);
                resList.Add(new JsonMaster.XOROwner(owner, xors));
            }
            return resList;
        }

        // List<string>[0] ... output list of other (required) elements names of connectors with minLimit >= 1
        // List<string>[1] ... output list of other (optional) elements names of connectors with minLimit == 0
        public static List<string>[] GetRequiredLists(EA.Repository repo, XOROwner xorOwner)
        {
            List<string> requList = new List<string>();
            List<string> optList = new List< string >();
            List<string>[] result = {requList, optList};
            foreach (int connID in xorOwner.GetConnList())
            {
                EA.Connector conn = repo.GetConnectorByID(connID);
                EA.ConnectorEnd connEnd;
                EA.Element theOtherElem;
                if (conn.ClientID == xorOwner.GetOwner().ElementID)
                {
                    connEnd = conn.SupplierEnd;
                    theOtherElem = repo.GetElementByID(conn.SupplierID);
                } else
                {
                    connEnd = conn.ClientEnd;
                    theOtherElem = repo.GetElementByID(conn.ClientID);
                }
                string cardinality = connEnd.Cardinality;
                int min = GetMinCardinality(cardinality);
                if (min == 0) result[1].Add(theOtherElem.Name);
                else result[0].Add(theOtherElem.Name);
            }
            return result;
        }

        public static bool IsInOwnedXOR(List<JsonMaster.XOROwner> xorOwnerList, EA.Element elem, long connID)
        {
            foreach (XOROwner xorOwner in xorOwnerList)
            {
                if(xorOwner.GetOwner().ElementID == elem.ElementID)
                {
                    if (xorOwner.GetConnList().Contains(connID)) return true;
                }
            }
            return false;
        }

        public static int GetMinCardinality(string cardinality)
        {
            // No cardinality specified
            if (cardinality == null || cardinality.Equals("")) return 1;

            // there's a minimum specified, or there's just one character
            string ch = cardinality.Substring(0, 1);
            if ("0123456789".Contains(ch)) return Convert.ToInt16(ch);
            
            return 1; // Default
        }

        // Returns -1 for infinity or arbitrary many
        public static int GetMaxCardinality(string cardinality)
        {
            // no cardinality specified
            if (cardinality == null || cardinality.Equals("")) return 1;
            
            // There is one max value specified or there's is just one character
            int len = cardinality.Length;
            if(len >= 1)
            {
                string lastChar = cardinality.Substring(len - 1, 1);
                if ("123456789".Contains(lastChar)) return Convert.ToInt16(lastChar);
                if (lastChar.Equals("*")) return -1;
            }
            return 1; // Default value
        }

        public static void GetInheritedChain(EA.Repository repo, EA.Element startElem, List<EA.Element> chainList)
        {
            EA.Element otherEnd = null;
            foreach(EA.Connector conn in startElem.Connectors)
            {
                if(!conn.Type.Equals("Generalization"))
                {
                    continue;
                }

                if (conn.ClientID == startElem.ElementID)
                {
                    otherEnd = repo.GetElementByID(conn.SupplierID);
                    if (!chainList.Contains(otherEnd))
                    {
                        chainList.Add(otherEnd);
                    }
                }

                if (otherEnd != null)
                {
                    GetInheritedChain(repo, otherEnd, chainList);
                    break;
                }
            }
        }

        public static string Prettify(string jsonString)
        {
            string INDENT = "    ";
            var output = new StringBuilder();
            int depth = 0;
            bool inQuotes = false;
            bool inApos = false;

            for (int i = 0; i < jsonString.Length; ++i)
            {
                char ch = jsonString[i];

                switch (ch)
                {
                    case '{':
                    case '[':
                        output.Append(ch);
                        if (!inQuotes && !inApos)
                        {
                            output.AppendLine();
                            depth++;
                            output.Append(Repeat(INDENT, depth));
                        }
                        break;
                    case '}':
                    case ']':
                        if(!inQuotes && !inApos)
                        {
                            output.AppendLine();
                            depth--;
                            if (depth < 0) depth = 0;
                            output.Append(Repeat(INDENT, depth));
                        }
                        output.Append(ch);
                        break;
                    case '"':
                        inQuotes = !inQuotes;
                        output.Append(ch);
                        break;
                    case '\'':
                        inApos = !inApos;
                        output.Append(ch);
                        break;
                    case ',':
                        output.Append(ch);
                        if (!inQuotes && !inApos)
                        {
                            output.AppendLine();
                            output.Append(Repeat(INDENT, depth));
                        }
                        break;
                    default:
                        output.Append(ch);
                        break;
                }
            }

            return output.ToString();
        }

        private static string Repeat(string str, int count)
        {
            return new StringBuilder().Insert(0, str, count).ToString();
        }

        private static bool IsEscaped(string str, int index)
        {
            bool escaped = false;
            while (index > 0 && str[--index] == '\\') escaped = !escaped;
            return escaped;
        }

        private static bool IsEscapedSB(StringBuilder str, int index)
        {
            return IsEscaped(str.ToString(), index);
        }

        public static string ReplaceLFCR(string notes)
        {
            notes = notes.TrimEnd('\r', '\n');
            notes = notes.Replace("\"", @"'");
            notes = notes.Replace("\n", "\\n");
            notes = notes.Replace("\r", "\\r");
            return notes;
        }

        
        public static void MoveToTop(List<JsonBase> list, int index)
        {
            JsonBase item = list[index];
            for (int i = index; i > 0; i--)
                list[i] = list[i - 1];
            list[0] = item;
        }

        public static JsonArray<string> GetRequiredProperties(List<string> requiredAttributes)
        {
            if (requiredAttributes == null || requiredAttributes.Count <= 0) return null;

            JsonArray<string> jArray = new JsonArray<string>(JsonMapper.SCHEMA_ELEM_REQUIRED);
            requiredAttributes.Sort();
            foreach (string requAttr in requiredAttributes)
            {
                jArray.Add(requAttr);
            }
            return jArray;
        }

        public static void SaveJsonFile(string jsonString, string fileName)
        {
            if (fileName == null || fileName.Length == 0)
            {
                ErrorCodes.ShowError(ErrorCodes.ERROR_003);
                return;
            }
            try
            {
                //jsonString = JsonConvert.SerializeObject(jsonString, Formatting.Indented);
                jsonString = JsonGenerationUtilities.Prettify(jsonString);
                Directory.CreateDirectory(Path.GetDirectoryName(fileName));
                System.IO.File.WriteAllText(fileName, jsonString);
            }
            catch (Exception e)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_004, e);
            }
        }



        public static List<JsonKeyValue<string>> GetRootMinMax(EA.Element rootElem)
        {
            List < JsonKeyValue<string> > resList = new List < JsonKeyValue<string>> ();
            foreach (EA.Connector conn in rootElem.Connectors)
            {
                EA.ConnectorEnd connEnd;
                if (conn.ClientID == rootElem.ElementID)
                {
                    connEnd = conn.ClientEnd;
                } else
                {
                    connEnd = conn.SupplierEnd;
                }
                string cardinality = connEnd.Cardinality;
                if (cardinality != null && cardinality.Length > 0)
                {
                    if(!cardinality.StartsWith("1"))
                    {
                        JsonKeyValue<string> jMin = new JsonKeyValue<string>("minItems", cardinality.Substring(0,1));
                        resList.Add(jMin);
                    }
                    if((cardinality.Length >= 2) && !cardinality.EndsWith("1"))
                    {
                        string upperBound = cardinality.Substring (cardinality.Length-2, 1);
                        JsonKeyValue<string> jMax = new JsonKeyValue<string>("maxItems", upperBound);
                        resList.Add(jMax);
                    }
                }
            }
            return resList;
        }

    }
}
