﻿/////////////////////////////////////////////////////////////////////////////////////////
// Author: Alexander Balka
// File: JsonArray.cs
/////////////////////////////////////////////////////////////////////////////////////////


//using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Text.Json;
using CimSyntaxGen.JsonMaster;

namespace CimSyntaxGen
{
    public class JsonArray<TValue>: JsonBase
    {
        private readonly string QUOTES = "\"";
        private readonly string COLON = ": ";
        private readonly string START = "[";
        private readonly string END = "]";

        private string name;
        private List<TValue> elemList = new List<TValue>();

        public JsonArray(string name)
        {
            if(name == null)
            {
                this.name = "";
            } else
            {
                this.name = name;
            }
        }

        public JsonArray()
        {
            this.name = "";
        }

        public void Append(List<TValue> otherElemList)
        {
            elemList.AddRange(otherElemList);
        }

        public void Add(TValue value)
        {
            this.elemList.Add(value);
        }

        public string GetName()
        {
            return this.name;
        }

        public void SetName(string name)
        {
            if(name == null)
            {
                this.name = name;
            }
        }


        public List<TValue> GetList() { return this.elemList; }

        public string AsString()
        {
            string tmp = "";
            if (this.name.Length > 0)
            {
                tmp = QUOTES + this.name + QUOTES + COLON;
            }

            tmp += START;
            Type valueType = typeof(TValue);
            for (int i = 0; i < this.elemList.Count; ++i)
            {
                TValue value = this.elemList[i];
                if(value == null)
                {
                    continue;
                }

                if (valueType == typeof(string))
                {
                    tmp += QUOTES + value + QUOTES;
                } 
                else if(valueType == typeof(JsonBase)) 
                {
                    tmp += ((JsonBase)value).AsString();
                }
                else
                {
                    tmp += value;
                }

                if (i < (this.elemList.Count - 1))
                {
                    tmp += ",";
                }
            }
            tmp += END;
            return tmp;
        }

    }
}
