﻿/////////////////////////////////////////////////////////////////////////////////////////
// Author: Alexander Balka
// File: JsonObject.cs
/////////////////////////////////////////////////////////////////////////////////////////


using CimSyntaxGen.JsonGeneration;
using CimSyntaxGen.JsonMaster;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Xml;

namespace CimSyntaxGen
{
    public class JsonObject<TValue> : JsonBase
    {
        private readonly string QUOTES = "\"";
        private readonly string COLON = ": ";
        private readonly string START_CURLY = "{";
        private readonly string END_CURLY = "}";

        private string name;
        private List<JsonBase> elemList = new List<JsonBase>();

        public JsonObject(string name)
        {
            if (name == null)
            {
                this.name = "";
            }
            else
            {
                this.name = name;
            }
        }

        public string GetName()
        {
            return name;
        }

        public void Add(JsonBase value)
        {
            this.elemList.Add(value);
        }

        public List<JsonBase> GetList() { return this.elemList; }

        public void Sort()
        {
            JsonComparer jsonComparer = new JsonComparer();
            this.elemList.Sort(jsonComparer);
            Move2Top("mRID");
        }

        private void Move2Top(string name)
        {
            int index = -1;
            foreach(JsonBase elem in this.elemList)
            {
                index++;
                if (elem.GetName().Equals(name)) break;
            }
            if(index > 0)
            {
                JsonGenerationUtilities.MoveToTop(this.elemList, index);
            }
        }


        public string ListAsString()
        {
            string result = "";
            foreach(JsonBase elem in this.elemList)
            {
                result += elem.GetName().ToString() + ",";
            }
            return result;
        }

        public string AsString()
        {
            string tmp = "";
            if (this.name.Length > 0)
            {
                tmp = QUOTES + this.name + QUOTES + COLON;
            }

            tmp += START_CURLY;
            for (int i = 0; i < this.elemList.Count; ++i)
            {
                if(this.elemList[i] == null)
                {
                    tmp += "null";
                } else
                {
                    tmp += this.elemList[i].AsString();
                }
                if (i < (this.elemList.Count - 1))
                {
                    tmp += ",";
                }
            }
            tmp += END_CURLY;
            return tmp;
        }
    }
}
