﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.Option
{

    public partial class DialCodelistImport : Form
    {
        public static string LAST_CODELISTS_FILENAME = "LastCodeListsFile";
        private XMLParser xmlp;
        private DialogResponse dialogResponse;

        public DialCodelistImport(EA.Repository repo, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.dialogResponse = dialogResponse;
            xmlp = new XMLParser(repo);
            this.textBoxFilename.Text = xmlp.GetXmlValueProfData(LAST_CODELISTS_FILENAME);
            if(this.textBoxFilename.Text != null && this.textBoxFilename.Text.Length > 0)
            {
                this.buttonImport.Enabled = true;
            }
            else this.buttonImport.Enabled = false;
        }

        private void buttonFilename_Click(object sender, EventArgs e)
        {
            string initDir = Path.GetDirectoryName(xmlp.GetXmlValueProfData(LAST_CODELISTS_FILENAME));
            if (initDir == null || initDir.Trim().Length <= 0) initDir = @"C:\";

            OpenFileDialog openFileDialog = new OpenFileDialog
            {
                InitialDirectory = initDir,
                Title = @"Select Codelist File",

                CheckFileExists = true,
                CheckPathExists = true,

                DefaultExt = "xsd",
                Filter = @"xsd files (*.xsd)|*.xsd|All files (*.*)|*.*",
                FilterIndex = 1,
                RestoreDirectory = true,

                ReadOnlyChecked = true,
                ShowReadOnly = true
            };

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxFilename.Text = openFileDialog.FileName;
                this.buttonImport.Enabled = true;
            }
            openFileDialog.Dispose();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            dialogResponse.Canceled = true;
            this.Dispose();
        }

        private void buttonImport_Click(object sender, EventArgs e)
        {
            xmlp.SetXmlValueProfData(LAST_CODELISTS_FILENAME, this.textBoxFilename.Text);
            dialogResponse.Canceled = false;
            dialogResponse.FileName = Path.GetDirectoryName(this.textBoxFilename.Text);
            dialogResponse.DirName = Path.GetFileName(this.textBoxFilename.Text);
            this.Dispose();
        }

        private void textBoxFilename_TextChanged(object sender, EventArgs e)
        {
            if (this.textBoxFilename.Text != null && this.textBoxFilename.Text.Length > 0) this.buttonImport.Enabled = true;
            else this.buttonImport.Enabled = false;
        }
    }
}
