﻿using CimSyntaxGen.Utilities;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.DirectoryServices.ActiveDirectory;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.Option
{
    public partial class DialStereosNamespaces : Form
    {
        private XMLProfileParameters profParams;
        private XMLParser xmlp = new XMLParser(Main.Repo);
        private bool starting = true;
        private bool doForAll = false; // controls checkbox events
        private string uri = "";
        private DialogResponse dialogResponse = null;

        public DialStereosNamespaces(XMLProfileParameters profParameters, DialogResponse dialogResponse)
        {
            InitializeComponent();
            this.dialogResponse = dialogResponse;
            this.dataGridViewStereos.BackgroundColor = Color.White;
            this.profParams = profParameters;
            this.uri = profParameters.DefaultRootModelURI;

            foreach (string stereo in profParameters.ListStereoNamespace)
            {
                this.dataGridViewStereos.Rows.Add(false, "cim", stereo, uri); // default values
            }
            foreach(DataGridViewRow row in this.dataGridViewStereos.Rows)
            {
                row.Cells[1].ReadOnly = false;
                row.Cells[1].Style.BackColor = Color.White;

                row.Cells[2].ReadOnly = true;
                row.Cells[2].Style.BackColor = Color.LightGray;

                row.Cells[3].ReadOnly = false;
                row.Cells[3].Style.BackColor = Color.White;
                this.SetValuesByCheckbox(false, row);
            }
            this.dataGridViewStereos.Refresh();
            starting = false; // controls the checkbox events
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            RDFutilitaires.ongoing = false;
            dialogResponse.Canceled = true;
            this.Close();
            this.Dispose();
        }

        private void buttonAll_Click(object sender, EventArgs e)
        {
            this.doForAll = true;
            foreach (DataGridViewRow row in dataGridViewStereos.Rows)
            {
                row.Cells[0].Value = true;
                this.SetValuesByCheckbox(true, row);
            }
            this.dataGridViewStereos.Refresh();
            this.doForAll = false;
        }

        private void buttonNone_Click(object sender, EventArgs e)
        {
            this.doForAll = true;
            foreach (DataGridViewRow row in dataGridViewStereos.Rows)
            {
                row.Cells[0].Value = false;
                this.SetValuesByCheckbox(false, row);
            }
            this.dataGridViewStereos.Refresh();
            this.doForAll = false;
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            Main.XMLP.SetXmlValueConfig("VersionConfig", DateTime.Today.ToString("yyyy-MM-dd"));
            foreach(DataGridViewRow row in this.dataGridViewStereos.Rows)
            {
                string stereotype = row.Cells[2].Value.ToString();
                if (this.profParams.urisbystereo.ContainsKey(stereotype))
                {
                    string uri = row.Cells[3].Value.ToString();
                    if (uri.EndsWith("#"))
                    { // The URI ends with a #.
                        this.profParams.urisbystereo[stereotype][1] = row.Cells[3].Value.ToString();
                        this.profParams.urisbystereo[stereotype][0] = row.Cells[1].Value.ToString();
                        if (this.profParams.urisbystereo[stereotype][0] != "cim")
                        {
                            xmlp.AddXmlProfstereo(stereotype, this.profParams.urisbystereo[stereotype][0], 
                                                  this.profParams.urisbystereo[stereotype][1], null, null, null);
                        }
                    }
                    else
                    {
                        MessageBox.Show(" Warning: One of the URI values does not end with '#', please fix it!");
                        return;
                    }
                }
            }
            CIMSchemaGenerator.profparameters = this.profParams;
            dialogResponse.Canceled = false;
            this.Close();
        }

        private void dataGridViewStereos_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            if (dataGridViewStereos.IsCurrentCellDirty)
            {
                dataGridViewStereos.CommitEdit(DataGridViewDataErrorContexts.Commit);
            }
        }

        private void dataGridViewStereos_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            if(!starting && !doForAll)
            {
                int colIndex = this.GetColumnIndexByName("Default");
                if (e.ColumnIndex == colIndex)
                {
                    this.SetValuesByCheckbox((bool)dataGridViewStereos.CurrentCell.Value, this.dataGridViewStereos.CurrentRow);
                    dataGridViewStereos.CurrentCell = null;
                }
            }
        }

        private void SetValuesByCheckbox(bool checkBoxValue, DataGridViewRow row)
        {
            if (checkBoxValue)
            {
                row.Cells[1].Value = "cim";
                row.Cells[1].ReadOnly = true;
                row.Cells[1].Style.BackColor = Color.LightGray;

                row.Cells[3].Value = this.uri;
                row.Cells[3].ReadOnly = true;
                row.Cells[3].Style.BackColor = Color.LightGray;
            }
            else
            {
                ArrayList prefixuri = this.xmlp.GetXmlProfstereo(row.Cells[2].Value.ToString());
                if(prefixuri != null && prefixuri.Count > 1) 
                {
                    row.Cells[1].Value = (string)prefixuri[0];
                    row.Cells[3].Value = (string)prefixuri[1];
                }

                row.Cells[1].ReadOnly = false;
                row.Cells[1].Style.BackColor = Color.White;

                row.Cells[3].ReadOnly = false;
                row.Cells[3].Style.BackColor = Color.White;
            }
            this.dataGridViewStereos.Refresh();
        }

        private int GetColumnIndexByName(string headerName)
        {
            foreach (DataGridViewColumn col in dataGridViewStereos.Columns)
            {
                if (col.HeaderText.ToLower().Trim() == headerName.ToLower().Trim())
                {
                    return dataGridViewStereos.Columns.IndexOf(col);
                }
            }
            return -1;
        }


        private void DialStereosNamespaces_FormClosed(object sender, FormClosedEventArgs e)
        {
            //RDFutilitaires.ongoing = false; ABA20240829
            this.Close();
            this.Dispose();
        }
    }
}
