using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using CimSyntaxGen;
using CimSyntaxGen.JsonGeneration;
using CimSyntaxGen.Utilities;

/*************************************************************************\
***************************************************************************
* Product CimSyntaxGen      Company :Zamiren (Joinville-le-Pont France) *
*                                      Entsoe (Bruxelles Belgique)        *
***************************************************************************
***************************************************************************                     
* Version : 3.4.16                                     *    november 2020 * 
*                                                                         *
***************************************************************************
*                                                                         *
*       Credit to:                                                        *
*                   Andre Maizener   andre.maizener@zamiren.fr            *
*                   Jean-Luc Sanson  jean-luc.sanson@noos.fr              *
*                                                                         *
*                                                                         *
*      Contact: +33148854006                                              *
*                                                                         *
***************************************************************************

**************************************************************************/
namespace CimSyntaxGen.Option
{
    public partial class Dialjson : Form
    {
        public const string LAST_JSON_EXPORTED_FILE = "LastJsonExportedFile";
        string canonicalBaseURI = "";
        string envelopeName = "";
        DialogResponse dialogResponse = null;
        XMLParser xmlp;
        public Dialjson(string wg, DialogResponse dialogResponse)
        {
            this.WG = wg;
            this.dialogResponse = dialogResponse;
            xmlp=new XMLParser(Main.Repo);
            string eapfile = Main.Repo.ConnectionString;
            string profilename = Main.Repo.GetTreeSelectedPackage().Name;
            InitializeComponent();
            this.Text = this.Text + " of " + wg.ToUpper();
            string prov = this.label9.Text;
           
            //this.profparams = profparameters;            
            this.textBoxEnvelopeName.Text = Main.Repo.GetTreeSelectedPackage().Name.Replace(' ', '_');//Envelope Name

            if (wg == "wg16")
            {
                this.textBoxEnvelopeName.Enabled = false;
                envelopeName = this.textBoxEnvelopeName.Text.Trim(' ').Replace(' ', '-');
                this.textBoxEnvelopeName.Text = envelopeName;
            }
            else
            {
                envelopeName = this.textBoxEnvelopeName.Text;
            }
            string fic = xmlp.GetXmlValueJsonData(LAST_JSON_EXPORTED_FILE);
            string ficdir = "";
            if (fic != "")
            {
                ficdir = Path.GetDirectoryName(fic);
            }
            this.buttonSave.Enabled = false;

            this.textBox6.Text = ficdir; //ResultFileDirectory 
            this.textBox1.Text = this.textBoxEnvelopeName.Text + ".schema.json"; // ResultFileName
            if((this.textBox1.Text != null) && (this.textBox1.Text.Trim().Length > 0))
            {
                this.buttonSave.Enabled = true;
            }
            this.textBox2.Text = xmlp.GetXmlValueJsonData("JsonNameSpace");
            this.textBoxCanonBaseUri.Text = xmlp.GetXmlValueJsonData("CanonicalBaseURI");
            canonicalBaseURI = this.textBoxCanonBaseUri.Text;

            if (WG=="wg16")
            {
                string newCanonicalBaseURI = canonicalBaseURI;
                if (canonicalBaseURI.Contains("https"))
                {
                    newCanonicalBaseURI = canonicalBaseURI.Replace("https://","file:///");
                }
                if (canonicalBaseURI.Contains("http"))
                {
                    //AB2024011 newCanonicalBaseURI = canonicalBaseURI.Replace("http://", "file:///" + "C:/Users/Public/Documents/");
                    newCanonicalBaseURI = canonicalBaseURI.Replace("http://", "file:///");
                }

                //else
                //{
                //    if (canonicalBaseURI.Contains("http"))
                //    {
                //        newracine = canonicalBaseURI.Replace("http://", "file:///" + "C:/Users/Public/Documents/");
                //    }
                //    else
                //    {
                //        throw new Exception("The configuration parameter \"CanonicalBaseURI\" must contain http");
                //    }
                //}
                this.textBox3.Text = newCanonicalBaseURI + envelopeName.Trim(' ') + ".schema.json";//JsonSchemaID
                this.textBox3.Enabled = true;
                this.textBox2.Text = newCanonicalBaseURI + envelopeName + "#"; //JsonNameSpace
                this.textBox2.Enabled = true;

                // If outputdir and output filename are already loaded from configuration:
                SetJsonSchemaId(); // sets this.textBox3.Text
            }
            else
            {
                this.textBox3.Text = canonicalBaseURI + envelopeName.Trim(' ') + ".schema.json"; //
                this.textBox2.Text = canonicalBaseURI + this.textBoxEnvelopeName.Text + "#"; //JsonNameSpace
            }

            string versions = xmlp.GetXmlValueJsonData("JsonSchemaVersions");
            string[] split = versions.Split(new char[] { '|' });
            int i= 0;
            foreach(string version in split)
            {
                if (i == 0) comboBox1.Text = version;i++;
                comboBox1.Items.Add(version); 
            }

            string lastCodelistDir = xmlp.GetXmlValueJsonData(DialCodelistExportToJson.LASTJSONCODELIST_EXPORT_DIRECTORY);
            string lastCodelistFilename = xmlp.GetXmlValueJsonData(DialCodelistExportToJson.LASTJSONCODELIST_EXPORT_FILENAME);
            if(!lastCodelistDir.EndsWith("\\")) lastCodelistDir += "\\";
            this.textBox5.Text = lastCodelistDir + lastCodelistFilename;

            string lastLocalCodelistDir = xmlp.GetXmlValueJsonData(DialCodelistExportToJson.LASTJSONCODELIST_EXPORT_DIRECTORY);
            string lastLocalCodelistFilename = xmlp.GetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_FILENAME);
            if(lastLocalCodelistDir.EndsWith("\\")) lastLocalCodelistDir += "\\";
            this.textBox7.Text = lastLocalCodelistDir + lastLocalCodelistFilename;

            this.textBox10.Text = xmlp.GetXmlValueProfData("RootModelURI");
            //---------------------------------------------
            if (wg == "wg16") { }
            if (wg == "wg19")
            {
                this.label13.Hide(); // codelists non utiliss en wg19
                this.label7.Hide();
                this.textBox5.Hide();
                this.label4.Hide();
                this.label5.Hide();
                this.textBox7.Hide();
                //this.CBCodelist.Hide();
                this.BSelectFile1.Hide();
                this.BSelectfile2.Hide();
                this.groupBox2.Hide();
            }        
        }

        private XMLProfileParameters profparams;
        private string WG;

        private void Dialxsd_Load(object sender, EventArgs e)
        {
         
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
        
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {
            MessageBox.Show("le nom du fichier est valide");
        }

  

        private void button2_Click(object sender, EventArgs e)
        {
            string fic = xmlp.GetXmlValueJsonData(LAST_JSON_EXPORTED_FILE);
            string ficdir = this.textBox6.Text;
            if (ficdir == "")
            {
                MessageBox.Show(" You must choose an export directory");
                this.Dispose();
                return;
            }
            else dialogResponse.DirName = ficdir;

            string ficname = this.textBox1.Text;

            xmlp.SetXmlValueProfData("EnvelopName", this.textBoxEnvelopeName.Text);
            xmlp.SetXmlValueJsonData("ForWg", WG);
            //xmlp.SetXmlValueJsonData("LastJsonExportedFile", ficdir + "\\" + ficname);
            xmlp.SetXmlValueJsonData("JsonNameSpace", this.textBox2.Text);
            xmlp.SetXmlValueJsonData("JsonSchemaID", this.textBox3.Text);
            xmlp.SetXmlValueJsonData("JsonSchemaVersion", this.comboBox1.Text);

            /* ABA20240110
            string lastCodelistDir = Path.GetDirectoryName(this.textBox5.Text);
            string lastCodelistFilename = Path.GetFileName(this.textBox5.Text);
            xmlp.SetXmlValueJsonData(DialCodelistExportToJson.LASTJSONCODELIST_EXPORT_DIRECTORY, lastCodelistDir);
            xmlp.SetXmlValueJsonData(DialCodelistExportToJson.LASTJSONCODELIST_EXPORT_FILENAME, lastCodelistFilename);

            string lastLocalCodelistDir = Path.GetDirectoryName(this.textBox7.Text);
            string lastLocalCodelistFilename = Path.GetFileName(this.textBox7.Text);
            xmlp.SetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_DIRECTORY, lastLocalCodelistDir);
            xmlp.SetXmlValueJsonData(DialExportCodelistToJsonTemplate.LASTJSONLOCALCODELIST_EXPORT_FILENAME, lastLocalCodelistFilename);
            */
            string resultfile = this.textBox6.Text + "\\" + this.textBox1.Text.Replace(' ', '_');
            xmlp.SetXmlValueJsonData(LAST_JSON_EXPORTED_FILE, resultfile);
            xmlp.SetXmlValueProfData("RootModelURI", this.textBox10.Text);
            xmlp.SetXmlValueJsonData("CanonicalBaseURI", this.textBoxCanonBaseUri.Text);

            //if (this.CBCodelist.Checked)
            //{
            //    dialogResponse.CheckBoxTicked = true;
            //} else dialogResponse.CheckBoxTicked = false;

            CIMSchemaGenerator.profparameters = this.profparams;
            dialogResponse.Canceled = false;
            this.Dispose();
        }

   
        private void button3_Click(object sender, EventArgs e)
        {
            CIMSchemaGenerator.profparameters = this.profparams;
            RDFutilitaires.ongoing = false;
            dialogResponse.Canceled = true;
            this.Dispose();
        }

     

        private void BDirectory_Click(object sender, EventArgs e)
        {
            this.folderBrowserDialog1.SelectedPath = this.textBox6.Text;
            if (this.folderBrowserDialog1.ShowDialog()== DialogResult.OK)
            {
                this.textBox6.Text = this.folderBrowserDialog1.SelectedPath;
            }
        }

        private void BSelectFle1_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.textBox5.Text = openFileDialog1.FileName;
            }
        }

        private void BSelectfile2_Click(object sender, EventArgs e)
        {
            //this.openFileDialog2.Filter = " codelists (*.xsd)|*.xsd |All files (*.*)|*.*";
            this.openFileDialog2.Filter = "All files (*.*)|*.*";
            openFileDialog2.FilterIndex = 1;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog2.ShowDialog() == DialogResult.OK)
            {
                this.textBox7.Text = openFileDialog2.FileName;
            }
        }

        private void textBoxEnvelopeName_TextChanged(object sender, EventArgs e)
        {
            this.envelopeName = this.textBoxEnvelopeName.Text;
            this.textBox2.Text = canonicalBaseURI + this.envelopeName + "#"; //JsonNameSpace
            this.textBox3.Text = canonicalBaseURI + "/." + envelopeName + ".shema.json";
        }

        private void label14_Click(object sender, EventArgs e)
        {

        }

        private void textBoxCanonBaseUri_TextChanged(object sender, EventArgs e)
        {
            this.canonicalBaseURI = this.textBoxCanonBaseUri.Text;
            this.envelopeName = this.textBoxEnvelopeName.Text;
            this.textBox2.Text = canonicalBaseURI + this.envelopeName + "#"; //JsonNameSpace
            this.textBox3.Text = canonicalBaseURI + "/." + envelopeName + ".shema.json";
        }

        private void SetJsonSchemaId()
        {
            if(this.textBox1.Text != null && this.textBox1.Text.Trim().Length > 0)
            {
                this.textBox1.Text = this.textBox1.Text.Replace(' ', '_');
                if (this.textBox6.Text != null && this.textBox6.Text.Trim().Length > 0)
                {
                    string outputDir = this.textBox6.Text;
                    if (!outputDir.EndsWith("\\")) outputDir += "\\";
                    outputDir = outputDir.Replace("\\", "/");
                    this.textBox3.Text = @"file:///" + outputDir + this.textBox1.Text.Replace("\\", "/");
                } else
                {
                    this.textBox3.Text = @"file:///" + this.textBox1.Text.Replace("\\", "/");
                }
            }

        }

        private void textBox6_TextChanged(object sender, EventArgs e)
        {
            SetJsonSchemaId();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            SetJsonSchemaId();
            if((this.textBox1.Text != null) && (this.textBox1.Text.Trim().Length > 0))
            {
                this.buttonSave.Enabled = true;
            } else this.buttonSave.Enabled = false;
        }
    }
}
