using System;
using System.IO;
using System.Xml;
using System.Windows.Forms;
using System.Reflection;
using System.Collections;
using System.Security.Principal;
using Microsoft.Win32;
using System.Collections.Generic;
using EA;
using System.Runtime.InteropServices.ComTypes;
using CimSyntaxGen.Configuration;
using CimSyntaxGen.Utilities;
using Newtonsoft.Json.Linq;

/*************************************************************************\
***************************************************************************
* Product CimSyntaxGen      Company :Zamiren (Joinville-le-Pont France) *
*                                      Entsoe (Bruxelles Belgique)        *
***************************************************************************
***************************************************************************                     
* Version : 2.3.7                                         *    march 2019 * 
*                                                                         *
***************************************************************************
*                                                                         *
*       Credit to:                                                        *
*                   Andre Maizener   andre.maizener@zamiren.fr            *
*                   Jean-Luc Sanson  jean-luc.sanson@noos.fr              *
*                                                                         *
*                                                                         *
*      Contact: +33148854006                                              *
*                                                                         *
***************************************************************************

**************************************************************************/
namespace CimSyntaxGen
{
   public class XMLParser
    { 
        private static string CONFIG_FILE = @"\CimSyntaxgen-Config.xml";
        private static string LOG_FILE = @"\Log.xml";
        private string Path = "";
        
        //private XmlTextReader xmlReader = null;
        private XmlReader xmlReader = null;
        private FileStream StreamFile = null;
        private string ActivatedFilePath;
        private Repository repo;
        private Logger logger = new Logger(Main.LOG_FILE, Main.DO_LOG);

        // XML constants
        public static string LAST_AVRO_OUTPUT_DIRECTORY = "LastAvroOutputDirectory";
        public static string LAST_AVRO_NAMESPACE = "LastAvroNamespace";
        public static string LAST_ASCIIDOC_OUTPUT_DIRECTORY = "LastAsciiDocOutputDirectory";

        public string ActiveFilePath
        {
            get
            {
                return ActivatedFilePath;
            }
            
        }
        public string configPath
        {
            get
            {
                return Path;
            }
            set
            {
                Path = value;
            }
        }
        //---------- ajout am -------------------------------
        private XmlDocument confdoc;

        public XMLParser(Repository repo)
        {
            /* ABA20230903
            Path = FileManager.GetParentDirPath();
            string filename = Main.Repo.ConnectionString;
            string filepath;
            if (filename.Contains(".eap") || filename.Contains(".feap"))
            {
                filepath = System.IO.Path.GetDirectoryName(Main.Repo.ConnectionString);//am aout 2018


                if (File.Exists(filepath + CONFIG_FILE)) // if there is a config file in the database directory
                                                         // its path is take as Path or the directory is appdata
                {
                    Path = filepath;
                }
            }*/
            this.repo = repo;
            confdoc = new XmlDocument(); // am juil 2015
            Path = FileManager.GetParentDirPath(); // ABA20201020
            string logFilePath = Path + LOG_FILE;
            if (System.IO.File.Exists(logFilePath)) System.IO.File.Delete(logFilePath);

        }

        //public XMLParser() ABA20230903
        //{
        //    confdoc = new XmlDocument(); // am juil 2015
        //    Path = FileManager.GetParentDirPath();
        //}
        private void setFilePath(string FilePath){
            if (FilePath.Equals("DefaultLog"))
            {
                ActivatedFilePath = Path + LOG_FILE;
            }
            else if (FilePath.Equals("DefaultConfig"))
            {
                ActivatedFilePath = Path + CONFIG_FILE;
                //confdoc = new XmlDocument(); ABA20230910
                //confdoc.Load(ActivatedFilePath); // chargement du config
            }
            else { ActivatedFilePath = FilePath; }
            
            confdoc.Load(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfigurationAsStream());
        }

        private XmlReader ReadConfig()
        {
            try
            {
                ConfigurationManager.GetConfigurationManager(repo).LoadCustomConfiguration();
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            }
            catch (IOException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_007, ex);
            }
            return xmlReader;
        }

        public void Close()
        {
            try
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                /* ABA20230219
                if (StreamFile != null)
                {
                    StreamFile.Close();
                }
                */
            }
            catch (Exception)
            {
                // nothing to do ABA 20230215
            }
        }

        public string GetXmlValueConfig(string ElementName)
        {
            string Value = null;
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_038, ex);
            //}

            try
            {
                ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                if(configManager == null)
                {
                    throw new Exception("Creation of ConfigurationManager failed!");
                }
                string config = configManager.GetCustomConfiguration();
                if(config == null)
                {
                    throw new Exception("Loading configuration failed!");
                }
                XmlReader = XmlReader.Create(new StringReader(config));
                if(XmlReader == null) 
                {
                    throw new Exception("Creation of XmlReader failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("configuration") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_043, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }

        /*
        public string GetXmlValueConfig(string ElementName)
        {
            string ret = null;
            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            }
            catch (Exception ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_009, ex);
            }

            try
            {                 
                //this.setFilePath("DefaultConfig"); ABA20230904
                string filtre = "//appSettings//configuration[@name=\"" + ElementName + "\"]";
                XmlNode elnode = confdoc.SelectSingleNode(filtre);
                if(elnode == null) return ret;
                System.Xml.XPath.XPathNavigator elnav = elnode.CreateNavigator();
                ret = elnav.GetAttribute("value","");   
            }catch(Exception ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_008, ex);
            }
            return ret;
        }
        */
                
        public void SetXmlValueConfig(string ElementName, string Value)
        {
            if(this.GetXmlValueConfig("Log")==("Checked")){
                this.AddXmlLog("Config Saved", "Element:"+ElementName +" Value:"+Value);
            }

            //this.setFilePath("DefaultConfig"); ABA20230904
            //XmlTextReader XmlReader = null;
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;
                               
            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while ( xmlReader.Read() )
                {
                    if (xmlReader.Name.Equals("configuration") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();
                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());

                        XmlElement ElementRacine = NewXmlWriter.DocumentElement;

                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = ElementRacine;

                        sectionNode = groupNode.SelectSingleNode("appSettings");

                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("appSettings"));
                        }

                        XmlNode NodeCherche = sectionNode.SelectSingleNode("configuration[@name=\"" + ElementName + "\"]");

                        if (NodeCherche == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("configuration");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            NodeCherche = sectionNode.AppendChild(Element);
                            AttributeXml = NewXmlWriter.CreateAttribute("value");
                            AttributeXml.Value = Value.ToString();
                            NodeCherche.Attributes.Append(AttributeXml);
                        }
                        else
                        {
                            foreach (XmlAttribute xat in NodeCherche.Attributes)
                            {
                                if (xat.Name == "value")
                                {
                                    xat.Value = Value.ToString();
                                }
                            }
                        }
                        //NewXmlWriter.Save(ActivatedFilePath); ABA20230904
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                if (this.GetXmlValueConfig("Log") == ("Checked"))
                {
                    this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueConfig) XMLException:"+Ex.Message);
                }
                ErrorCodes.ShowException(ErrorCodes.ERROR_011, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)  //am juil 2015
                {
                    xmlReader.Close();
                }
                //  InitXMLConfig(); //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueConfig)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_012, Ex);
            }
            finally
            {
                if ( xmlReader != null )
                {
                    xmlReader.Close();
                }
            }
        }

        //Qualifier
        public ArrayList GetXmlQualifier(string Allowed)
        {
            this.setFilePath("DefaultConfig");
            ArrayList ValueList = new ArrayList();
            XmlReader xmlReader = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    XmlNodeType NodeType = xmlReader.NodeType;

                            
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.Equals("qualifier") )
                        {

                            if (xmlReader.GetAttribute("allowedTo").Equals(Allowed) || xmlReader.GetAttribute("allowedTo").Equals("any"))
                            {
                                ValueList.Add(xmlReader.GetAttribute("name"));
                            }
                        }
                    }
                }
                xmlReader.Close();
                return ValueList;
            }
            catch (XmlException Ex)
            {
                if (xmlReader != null)  //am juil 2015
                {
                    xmlReader.Close();
                }
                // InitXMLConfig();  //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (GetXMLQualifier) XMLException:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_013, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
            return ValueList;
        }


        public void AddXmlQualifier(string ElementName, string Allowed)
        {

            this.AddXmlLog("Adding Qualifier", "Added Qualifier:"+ElementName);
                    
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;
                    

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));

                while (xmlReader.Read())
                {
                    if (xmlReader.Name.Equals("appSettings"))
                    {
                        this.Close();
                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());

                        XmlElement RootElement = NewXmlWriter.DocumentElement;
                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = RootElement;
                        sectionNode = groupNode.SelectSingleNode("dataQualifier");
                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("data"));
                        }

                        XmlNode SearchedNode = sectionNode.SelectSingleNode("qualifier[@name=\"" + ElementName + "\"]");

                        if (SearchedNode == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("qualifier");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            AttributeXml = NewXmlWriter.CreateAttribute("allowedTo");
                            AttributeXml.Value = Allowed;
                            Element.Attributes.Append(AttributeXml);
                            SearchedNode = sectionNode.AppendChild(Element);
                        }
                        else{
                            sectionNode.RemoveChild(SearchedNode);
                            XmlElement Element = NewXmlWriter.CreateElement("qualifier");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            AttributeXml = NewXmlWriter.CreateAttribute("allowedTo");
                            AttributeXml.Value = Allowed;
                            Element.Attributes.Append(AttributeXml);
                                    
                            SearchedNode = sectionNode.AppendChild(Element);
                        }

                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                this.AddXmlLog("Warning!", "Error in XMLParser (AddXmlQualifier) XMLException:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_014, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null) //am juil 2015
                {
                    xmlReader.Close();
                }
                //  InitXMLConfig();  am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (AddXmlQualifier) Exception:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_015, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
        }

        public void DeleteXmlQualifier(string ElementName)
        {                
            this.AddXmlLog("Deleting Qualifier", "Deleted Qualifier:"+ElementName);
            //this.setFilePath("DefaultConfig"); ABA20230904
            if(!ElementName.Equals(null)) {
                XmlReader xmlReader = null;
                XmlDocument NewXmlWriter;

                try
                {
                    xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                    while (xmlReader.Read())
                    {
                        if (xmlReader.Name.Equals("qualifier"))
                        {
                            this.Close();

                            NewXmlWriter = new XmlDocument();
                            NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                            XmlElement ElementRacine = NewXmlWriter.DocumentElement;
                            XmlNode sectionNode = null;
                            XmlNode groupNode = ElementRacine;
                            sectionNode = groupNode.SelectSingleNode("dataQualifier");
                            XmlNode SearchedNode = sectionNode.SelectSingleNode("qualifier[@name=\"" + ElementName + "\"]");

                            if (!(SearchedNode==null))
                            {
                                sectionNode.RemoveChild(SearchedNode);
                            }
                            // save configuration                 
                            ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                            configManager.UpdateConfiguration(NewXmlWriter);
                        }
                    }
                }
                catch (XmlException Ex)
                {
                    this.AddXmlLog("Warning!", "Error in XMLParser (DeleteXmlQualifier) XMLException:" + Ex);
                    ErrorCodes.ShowException(ErrorCodes.ERROR_016, Ex);
                }
                catch (Exception Ex)
                {
                    if (xmlReader != null) //am juil 2015
                    {
                        xmlReader.Close();
                    }
                    // InitXMLConfig();   //am juil 2015                        this.AddXmlLog("Warning!", "Error in XMLParser (DeleteXmlQualifier) Exception:" + Ex);                        
                    ErrorCodes.ShowException(ErrorCodes.ERROR_017, Ex);
                }
                finally
                {
                    if (xmlReader != null)
                    {
                        xmlReader.Close();
                    }
                }
            }
        }

        //Log
        public void AddXmlLog(string EventType, string LogValue)
        {
            if (this.GetXmlValueConfig("Log") != ConfigurationManager.CHECKED)
            {
                return;
            }
            //eventtype must not have any whitespace
            EventType = EventType.Replace(" ","");
            //XmlReader xmlReader = null;
            XmlDocument NewXmlWriter;
            this.setFilePath("DefaultLog");

            try
            {//ErreurPar ICI
                /* ABA20230904
                xmlReader = ReadConfig();
                this.Close();
                */
                NewXmlWriter = new XmlDocument();                        
                XmlElement RootElement;
                try
                {
                        NewXmlWriter.Load(ActivatedFilePath);
                        RootElement = NewXmlWriter.DocumentElement;
                }
                catch
                {                            
                    XmlNode RootNode = NewXmlWriter.CreateNode(XmlNodeType.XmlDeclaration, "", "");
                    NewXmlWriter.AppendChild(RootNode);

                    RootElement = NewXmlWriter.CreateElement("", "add-inLogs", "");
                    NewXmlWriter.AppendChild(RootElement);
                }

                        
                XmlAttribute AttributeXml = null;
                XmlNode sectionNode = null;

                foreach( XmlNode ANode in RootElement.SelectNodes("logDate")){
                    if (ANode.Attributes[0].Value.Equals(DateTime.Now.ToString("MM-dd-yyyy")))
                    {
                        sectionNode = ANode;
                    }
                }
                if (sectionNode == null)
                {
                    XmlElement tmpelem = NewXmlWriter.CreateElement("","logDate","");
                    AttributeXml = NewXmlWriter.CreateAttribute("date");
                    AttributeXml.Value = (DateTime.Now.ToString("MM-dd-yyyy"));
                    tmpelem.Attributes.Append(AttributeXml);
                    sectionNode = RootElement.AppendChild(tmpelem);
                }
                                
                XmlElement Element = NewXmlWriter.CreateElement("","log","");

                AttributeXml = NewXmlWriter.CreateAttribute("time");
                AttributeXml.Value = DateTime.Now.ToString("HH:mm:ss");
                Element.Attributes.Append(AttributeXml);
                                    
                AttributeXml = NewXmlWriter.CreateAttribute("username");
                AttributeXml.Value = WindowsIdentity.GetCurrent().Name.ToString();
                Element.Attributes.Append(AttributeXml);

                AttributeXml = NewXmlWriter.CreateAttribute(EventType);
                AttributeXml.Value = LogValue;
                Element.Attributes.Append(AttributeXml);

                XmlNode NewNode = sectionNode.AppendChild(Element);
                NewNode.Attributes.Append(AttributeXml);
                NewXmlWriter.Save(ActivatedFilePath);
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_018, "EventType: " + EventType, Ex);
            }
            catch (Exception Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_019, Ex);
            }
            /* ABA202030904
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }*/
        }

        //Constraint
        public ArrayList GetXmlConstraint(string Allowed)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            ArrayList ValueList = new ArrayList();
            XmlReader xmlReader = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    XmlNodeType NodeType = xmlReader.NodeType;


                    if (NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.Equals("constraint"))
                        {
                            if (xmlReader.GetAttribute("allowedTo").Equals(Allowed) || xmlReader.GetAttribute("allowedTo").Equals("any"))
                            {
                            //  ValueList.Add( new XMLConstraint(XmlReader.GetAttribute("name"), XmlReader.GetAttribute("type"), XmlReader.GetAttribute("notes")) );
                            }
                        }
                    }
                }
                xmlReader.Close();
                return ValueList;
            }
            catch (XmlException Ex)
            {
                if (xmlReader != null) //am juil 2015
                {
                    xmlReader.Close();
                }
                // InitXMLConfig(); //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (GetXMLQualifier) XMLException:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_020, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }

            return ValueList;
        }

        public ArrayList GetXmlClassifierConstraint(string Allowed)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            ArrayList ValueList = new ArrayList();
            XmlReader xmlReader = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    XmlNodeType NodeType = xmlReader.NodeType;


                    if (NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.Equals("classifierconstraint"))
                        {
                            if (xmlReader.GetAttribute("allowedTo").ToLower().Equals(Allowed.ToLower()) || xmlReader.GetAttribute("allowedTo").ToLower().Equals("any".ToLower()))
                            {
                                try
                                {

                                //     ValueList.Add(new XMLClassifierConstraint(XmlReader.GetAttribute("name"), XmlReader.GetAttribute("type"), XmlReader.GetAttribute("variableType"), XmlReader.GetAttribute("variableList"), XmlReader.GetAttribute("comment"), XmlReader.ReadString()));
                                }
                                catch(Exception Ex)
                                {
                                    MessageBox.Show("Error while recovering the XML data of " + xmlReader.GetAttribute("name") + "\nThe constraint list will probably be corrupted, you should close this window and check the config file. \n" +Ex.Message , "Error while recovering XML data", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
                                }
                            }
                        }
                    }
                }
                xmlReader.Close();
                return ValueList;
            }
            catch (XmlException Ex)
            {
                if (xmlReader != null) //am juil 2015
                {
                    xmlReader.Close();
                }
                // InitXMLConfig();  //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (GetXMLQualifier) XMLException:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_020, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }

            return ValueList;
        }
        //-----------------------------------
        //profdata
        public List<string> GetXmlAllProfstereo()
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            List<string> ValueList = new List<string>();
            XmlReader xmlReader = null;
            string stereo = "";

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    XmlNodeType NodeType = xmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.Equals("profstereo"))
                        {

                            stereo = xmlReader.GetAttribute("name");
                            ValueList.Add(stereo);
                                    
                        }
                    }
                }
                xmlReader.Close();
                return ValueList;
            }
            catch (XmlException Ex)
            {
                if (xmlReader != null) //am juil 2015
                {
                    xmlReader.Close();
                }
                // InitXMLConfig();  //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (GetXMLQualifier) XMLException:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_021, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }

            return ValueList;
        }

        public string GetXmlValueProfData(string ElementName)
        {
            string Value = null;
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_039, ex);
            //}

            try
            {
                XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                if(XmlReader == null)
                {
                    throw new Exception("Creation of XmlReader failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("profdata") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_022, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }

        /*
        public string GetXmlValueProfData(string ElementName)
        {
            string ret = null;
            try
            {
                //this.setFilePath("DefaultConfig"); ABA20230904
                XmlNode elnode = confdoc.SelectSingleNode("//dataProfile/profdata[@name=\"" + ElementName + "\"]");
                if (elnode == null) return ret;
                System.Xml.XPath.XPathNavigator elnav = elnode.CreateNavigator();
                ret = elnav.GetAttribute("value", "");
            }catch(Exception e)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_022, e);
            }
            return ret;
        }
        */

        public string GetXmlValueJsonData(string ElementName)
        {
            string Value = "";
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_040, ex);
            //}

            try
            {
                XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                if(XmlReader == null)
                {
                    throw new Exception("Creation of XmlReader failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("jsondata") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_023, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }

        public string GetXmlValueAvroData(string ElementName)
        {
            string Value = "";
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_040, ex);
            //}

            try
            {
                XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                if(XmlReader == null)
                {
                    throw new Exception("Creation of XmlReader failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("avrodata") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_023, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }

        public string GetXmlValueAsciiDocData(string ElementName)
        {
            string Value = null;
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_046, ex);
            //}

            try
            {
                XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                if(XmlReader == null)
                {
                    throw new Exception("XmlReader creation failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("asciidocdata") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_047, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }


        /*
        public string GetXmlValueJsonData(string ElementName)
        {
            string ret = null;
            try
            {

                //this.setFilePath("DefaultConfig"); ABA20230904
                XmlNode elnode = confdoc.SelectSingleNode("//dataProfile/jsondata[@name=\"" + ElementName + "\"]");
                if (elnode == null) return ret;
                System.Xml.XPath.XPathNavigator elnav = elnode.CreateNavigator();
                ret = elnav.GetAttribute("value", "");
            }
            catch (Exception e)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_023, e);
            }
            return ret;
        }
        */


        //-------------------------------------------------------------------

        /*
        public void SetXmlValueProfData(string ElementName, string Value)
        {
            //Main.// if(Main.DO_REPORTLOG) reportlog.WriteLine("-----  come in  SetXmlValueProfData configpath=" + ActivatedFilePath);
            try
            {
                //this.setFilePath("DefaultConfig"); ABA20230904
                string filtre = "//dataProfile//profdata[@name=\"" + ElementName + "\"]";
                XmlNode elnode = confdoc.SelectSingleNode(filtre);
                if (elnode == null) return;
                System.Xml.XPath.XPathNavigator elnav = elnode.CreateNavigator();
                elnav.MoveToFirstAttribute();
                if(elnav.Name=="name")
                {
                    elnav.MoveToNextAttribute();
                    if(elnav.Name=="value")
                    {
                        elnav.SetValue(Value);
                    }

                }
                confdoc.Save(ActivatedFilePath);
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_024, Ex);
            }
            catch (Exception Ex)
            {

                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueProfData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_025, Ex);
            }
        }
        */

        public void SetXmlValueProfData(string ElementName, string Value)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    if (xmlReader.Name.Equals("profdata") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();

                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                        //NewXmlWriter.Load(ActivatedFilePath);

                        XmlElement ElementRacine = NewXmlWriter.DocumentElement;

                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = ElementRacine;

                        sectionNode = groupNode.SelectSingleNode("dataProfile");

                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                        }

                        XmlNode NodeCherche = sectionNode.SelectSingleNode("profdata[@name=\"" + ElementName + "\"]");

                        if (NodeCherche == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("profdata");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            NodeCherche = sectionNode.AppendChild(Element);
                        }

                        AttributeXml = NewXmlWriter.CreateAttribute("value");
                        AttributeXml.Value = Value.ToString();
                        NodeCherche.Attributes.Append(AttributeXml);
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_024, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueProfData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_025, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
        }


        public void SetXmlValueJsonData(string ElementName, string Value)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    if (xmlReader.Name.Equals("jsondata") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();

                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                        //NewXmlWriter.Load(ActivatedFilePath);

                        XmlElement ElementRacine = NewXmlWriter.DocumentElement;

                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = ElementRacine;

                        sectionNode = groupNode.SelectSingleNode("dataProfile");

                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                        }

                        XmlNode NodeCherche = sectionNode.SelectSingleNode("jsondata[@name=\"" + ElementName + "\"]");

                        if (NodeCherche == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("jsondata");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            NodeCherche = sectionNode.AppendChild(Element);
                        }

                        AttributeXml = NewXmlWriter.CreateAttribute("value");
                        AttributeXml.Value = Value.ToString();
                        NodeCherche.Attributes.Append(AttributeXml);
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_028, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueJsonData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_029, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
        }

        public void SetXmlValueAvroData(string ElementName, string Value)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    if (xmlReader.Name.Equals("avrodata") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();

                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                        //NewXmlWriter.Load(ActivatedFilePath);

                        XmlElement ElementRacine = NewXmlWriter.DocumentElement;

                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = ElementRacine;

                        sectionNode = groupNode.SelectSingleNode("dataProfile");

                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                        }

                        XmlNode NodeCherche = sectionNode.SelectSingleNode("avrodata[@name=\"" + ElementName + "\"]");

                        if (NodeCherche == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("avrodata");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            NodeCherche = sectionNode.AppendChild(Element);
                        }

                        AttributeXml = NewXmlWriter.CreateAttribute("value");
                        AttributeXml.Value = Value.ToString();
                        NodeCherche.Attributes.Append(AttributeXml);
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_028, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueJsonData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_029, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
        }

        public void SetXmlValueAsciiDocData(string ElementName, string Value)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    if (xmlReader.Name.Equals("asciidocdata") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();

                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                        //NewXmlWriter.Load(ActivatedFilePath);

                        XmlElement ElementRacine = NewXmlWriter.DocumentElement;

                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;

                        XmlNode groupNode = ElementRacine;

                        sectionNode = groupNode.SelectSingleNode("dataProfile");

                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                        }

                        XmlNode NodeCherche = sectionNode.SelectSingleNode("asciidocdata[@name=\"" + ElementName + "\"]");

                        if (NodeCherche == null)
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("asciidocdata");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                            AttributeXml.Value = ElementName;
                            Element.Attributes.Append(AttributeXml);

                            NodeCherche = sectionNode.AppendChild(Element);
                        }

                        AttributeXml = NewXmlWriter.CreateAttribute("value");
                        AttributeXml.Value = Value.ToString();
                        NodeCherche.Attributes.Append(AttributeXml);
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_048, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueJsonData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_049, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }
        }


        public string GetXmlValueShaclData(string ElementName)
        {
            string Value = null;
            XmlReader XmlReader = null;

            //try
            //{
            //    XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
            //}
            //catch (Exception ex)
            //{
            //    ErrorCodes.ShowException(ErrorCodes.ERROR_041, ex);
            //}

            try
            {
                XmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                if(XmlReader == null)
                {
                    throw new Exception("Creation of XmlReader failed!");
                }
                while (XmlReader.Read())
                {
                    XmlNodeType NodeType = XmlReader.NodeType;
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (XmlReader.Name.Equals("shacldata") && XmlReader.GetAttribute("name").Equals(ElementName))
                        {
                            Value = XmlReader.GetAttribute("value");
                            XmlReader.Close();
                            return Value;
                        }
                    }
                }
            }
            catch (XmlException ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_030, ex);
            }
            finally
            {
                if (XmlReader != null)
                {
                    XmlReader.Close();
                }
            }

            return Value;
        }

        /*
        public string GetXmlValueShaclData(string ElementName)
        {
            string ret = null;
            try
            {
                //this.setFilePath("DefaultConfig"); ABA20230904
                XmlNode elnode = confdoc.SelectSingleNode("//shacldata[@name=\"" + ElementName + "\"]");
                if (elnode == null) return ret;
                System.Xml.XPath.XPathNavigator elnav = elnode.CreateNavigator();
                ret = elnav.GetAttribute("value", "");
                }
                catch (Exception e)
                {
                    ErrorCodes.ShowException(ErrorCodes.ERROR_030, e);
                }
            return ret;
        }
        */


        public void SetXmlValueShaclData(string ElementName, string Value)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            XmlReader xmlReader = null;
            XmlDocument NewXmlWriter = null;
                               
            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while ( xmlReader.Read() )
                {                                       
                    if (xmlReader.Name.Equals("shacldata") && xmlReader.GetAttribute("name").Equals(ElementName))
                    {
                        this.Close();
                        NewXmlWriter = new XmlDocument();
                        NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                        XmlElement ElementRacine = NewXmlWriter.DocumentElement; 
                        XmlAttribute AttributeXml = null;
                        XmlNode sectionNode = null;
                        XmlNode groupNode = ElementRacine;
                        sectionNode = groupNode.SelectSingleNode("dataProfile");
 
                        if (sectionNode == null)
                        {
                            sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                        }                                               
 
                        XmlNode NodeCherche = sectionNode.SelectSingleNode("shacldata[@name=\"" + ElementName + "\"]");
 
                        if ( NodeCherche == null )
                        {
                            XmlElement Element = NewXmlWriter.CreateElement("shacldata");
                            AttributeXml = NewXmlWriter.CreateAttribute("name");
                                AttributeXml.Value = ElementName;
                                Element.Attributes.Append(AttributeXml);                       
 
                                NodeCherche = sectionNode.AppendChild(Element);                       
                        }

                        AttributeXml = NewXmlWriter.CreateAttribute("value");
                        AttributeXml.Value = Value.ToString();
                        NodeCherche.Attributes.Append(AttributeXml);
                        //am juil 2015
                        /*
                        // ---- ajout provisoire ----------------
                        try
                        {
                            System.IO.File.Delete(ActivatedFilePath);
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show("Error deletion fichier " + ActivatedFilePath + " " + e.Message);
                        }
                        */
                        // save configuration                 
                        ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                        configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
            }
            catch (XmlException Ex)
            {
                ErrorCodes.ShowException(ErrorCodes.ERROR_031, Ex);
            }
            catch (Exception Ex)
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
                //InitXMLConfig();
                this.AddXmlLog("Warning!", "Error in XMLParser (SetXmlValueProfData)Exception:" + Ex.Message);
                ErrorCodes.ShowException(ErrorCodes.ERROR_032, Ex);
            }
            finally
            {
                if ( xmlReader != null )
                {  
                        xmlReader.Close();
                }
            }
        }

        // profstereo
        public ArrayList GetXmlProfstereo(string name)
        {
            //this.setFilePath("DefaultConfig"); ABA20230904
            ArrayList ValueList = new ArrayList();
            XmlReader xmlReader = null;

            try
            {
                xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));
                while (xmlReader.Read())
                {
                    XmlNodeType NodeType = xmlReader.NodeType;

                            
                    if (NodeType == XmlNodeType.Element)
                    {
                        if (xmlReader.Name.Equals("profstereo") )
                        {
				   
                            if( xmlReader.GetAttribute("name").Equals(name))
                            { 
                                ValueList.Add(xmlReader.GetAttribute("prefix"));
 					            ValueList.Add(xmlReader.GetAttribute("uri"));
                                ValueList.Add(xmlReader.GetAttribute("html"));
                                ValueList.Add(xmlReader.GetAttribute("rdfs"));
                                ValueList.Add(xmlReader.GetAttribute("desc"));
                            }
				
                        }
                    }
                }
                xmlReader.Close();
                return ValueList;
            }
            catch (XmlException Ex)
            {
                if (xmlReader != null) //am juil 2015
                {
                    xmlReader.Close();
                }
                // InitXMLConfig();  //am juil 2015
                this.AddXmlLog("Warning!", "Error in XMLParser (GetXMLProfstereo) XMLException:" + Ex);
                ErrorCodes.ShowException(ErrorCodes.ERROR_033, Ex);
            }
            finally
            {
                if (xmlReader != null)
                {
                    xmlReader.Close();
                }
            }

            return ValueList;
        }


                public void AddXmlProfstereo(string Stereo, string Prefix,string Uri,string Html,string Rdfs,string Desc)
                {

                    this.AddXmlLog("Adding Profstereo", "Added Profstereo:"+ Stereo);
                    
                    //this.setFilePath("DefaultConfig"); ABA20230904
                    XmlReader xmlReader = null;
                    XmlDocument NewXmlWriter = null;
                    

                    try
                    {
                        xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));

                        while (xmlReader.Read())
                        {
                            if (xmlReader.Name.Equals("appSettings"))
                            {
                                this.Close();
                                NewXmlWriter = new XmlDocument();
                                NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                                XmlElement RootElement = NewXmlWriter.DocumentElement;
                                XmlAttribute AttributeXml = null;
                                XmlNode sectionNode = null;
                                XmlNode groupNode = RootElement;
                                sectionNode = groupNode.SelectSingleNode("dataProfile");
                                if (sectionNode == null)
                                {
                                    sectionNode = groupNode.AppendChild(NewXmlWriter.CreateElement("dataProfile"));
                                }

                                XmlNode SearchedNode = sectionNode.SelectSingleNode("profstereo[@name=\"" + Stereo + "\"]");

                                if (SearchedNode == null)
                                {
                                    XmlElement Element = NewXmlWriter.CreateElement("profstereo");
                                    AttributeXml = NewXmlWriter.CreateAttribute("name");
                                    AttributeXml.Value = Stereo;
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("prefix");
                                    AttributeXml.Value = Prefix;
                                    Element.Attributes.Append(AttributeXml);
				                    AttributeXml = NewXmlWriter.CreateAttribute("uri");
                                    AttributeXml.Value = Uri;
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("html");
                                    AttributeXml.Value = Html;
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("rdfs");
                                    AttributeXml.Value = Rdfs;
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("desc");
                                    AttributeXml.Value = Desc;
                                    Element.Attributes.Append(AttributeXml);

                                     SearchedNode = sectionNode.AppendChild(Element);
                                }
                                else
                                {
                                    ArrayList oldprofstereo = this.GetXmlProfstereo(Stereo);
                                    sectionNode.RemoveChild(SearchedNode);
                                    XmlElement Element = NewXmlWriter.CreateElement("profstereo");
                                    AttributeXml = NewXmlWriter.CreateAttribute("name");
                                    AttributeXml.Value = Stereo;
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("prefix");
                                    if (Prefix == null)
                                    {
                                        AttributeXml.Value = (string)oldprofstereo[0];
                                    }
                                    else
                                    {
                                        AttributeXml.Value = Prefix;
                                    }
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("uri");
                                    if (Uri == null)
                                    {
                                        AttributeXml.Value = (string)oldprofstereo[1];
                                    }
                                    else
                                    {
                                        AttributeXml.Value = Uri;
                                    }
                                    Element.Attributes.Append(AttributeXml);


                                    AttributeXml = NewXmlWriter.CreateAttribute("html");
                                    if (Html == null)
                                    {
                                        AttributeXml.Value = (string)oldprofstereo[2];
                                    }
                                    else
                                    {
                                        AttributeXml.Value = Html;
                                    }
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("rdfs");
                                    if (Rdfs == null)
                                    {
                                        AttributeXml.Value = (string)oldprofstereo[3];
                                    }
                                    else
                                    {
                                        AttributeXml.Value = Rdfs;
                                    }
                                    Element.Attributes.Append(AttributeXml);

                                    AttributeXml = NewXmlWriter.CreateAttribute("desc");
                                    if (Desc == null)
                                    {
                                        AttributeXml.Value = (string)oldprofstereo[4];
                                    }
                                    else
                                    {
                                        AttributeXml.Value = Desc;
                                    }
                                    Element.Attributes.Append(AttributeXml);

                                    
                                    SearchedNode = sectionNode.AppendChild(Element);
                                }

                                //  SearchedNode.Attributes.Append(AttributeXml);
                                // save configuration                 
                                ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                                configManager.UpdateConfiguration(NewXmlWriter);
                    }
                }
                    }
                    catch (XmlException Ex)
                    {
                        this.AddXmlLog("Warning!", "Error in XMLParser (AddXmlProfstereo) XMLException:" + Ex);
                        ErrorCodes.ShowException(ErrorCodes.ERROR_034, Ex);
                    }
            catch (Exception Ex)
                    {
                        if (xmlReader != null) //am juil 2015
                        {
                            xmlReader.Close();
                        }
                       // InitXMLConfig();  //am juil 2015
                        this.AddXmlLog("Warning!", "Error in XMLParser (AddXmlProfstereo) Exception:" + Ex);
                        ErrorCodes.ShowException(ErrorCodes.ERROR_035, Ex);
                    }
                    finally
                    {
                        if (xmlReader != null)
                        {
                            xmlReader.Close();
                        }
                    }
                }
       /// <summary>
      
                public void DeleteXmlProfstereo(string Stereo)
                {
                    
                    this.AddXmlLog("Deleting Profstereo", "Deleted Stereo:"+ Stereo);

                    //this.setFilePath("DefaultConfig"); ABA20230904
                    if(!Stereo.Equals(null)){
                    XmlReader xmlReader = null;
                    XmlDocument NewXmlWriter = null;

                    try
                    {
                        xmlReader = XmlReader.Create(new StringReader(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration()));

                        while (xmlReader.Read())
                        {
                            if (xmlReader.Name.Equals("profstereo"))
                            {
                                this.Close();

                                NewXmlWriter = new XmlDocument();
                                NewXmlWriter.LoadXml(ConfigurationManager.GetConfigurationManager(repo).GetCustomConfiguration());
                                XmlElement ElementRacine = NewXmlWriter.DocumentElement;
                                XmlNode sectionNode = null;
                                XmlNode groupNode = ElementRacine;
                                sectionNode = groupNode.SelectSingleNode("dataProfile");
                                XmlNode SearchedNode = sectionNode.SelectSingleNode("profstereo[@name=\"" + Stereo + "\"]");

                                if (!(SearchedNode==null))
                                {
                                    sectionNode.RemoveChild(SearchedNode);
                                }
                                // save configuration                 
                                ConfigurationManager configManager = ConfigurationManager.GetConfigurationManager(repo);
                                configManager.UpdateConfiguration(NewXmlWriter);
                        }
                    }
                    }
                    catch (XmlException Ex)
                    {
                        this.AddXmlLog("Warning!", "Error in XMLParser (DeleteXmlProfstereo) XMLException:" + Ex);
                        ErrorCodes.ShowException(ErrorCodes.ERROR_036, Ex);
                    }
                catch (Exception Ex)
                    {
                        if (xmlReader != null) //am juil 2015
                        {
                            xmlReader.Close();
                        }
                      //  InitXMLConfig(); //am juil 2015
                        this.AddXmlLog("Warning!", "Error in XMLParser (DeleteXmlProfstereo) Exception:" + Ex);
                        ErrorCodes.ShowException(ErrorCodes.ERROR_010, Ex);
                    }
                finally
                    {
                        if (xmlReader != null)
                        {
                            xmlReader.Close();
                        }
                    }
                    }
                }

//--------------------------------------------------------------------------
    }
}
