﻿using EA;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Text;
using System.Xml;
/*************************************************************************\
***************************************************************************
* Product CimSyntaxGen      Company :Zamiren (Joinville-le-Pont France) *
*                                      Entsoe (Bruxelles Belgique)        *
***************************************************************************
***************************************************************************                     
* Version : 3.7.10                                     *  november 2020   * 
*                                                                         *
***************************************************************************
*                                                                         *
*       Credit to:                                                        *
*                   Andre Maizener   andre.maizener@zamiren.fr            *
*                   Jean-Luc Sanson  jean-luc.sanson@noos.fr              *
*                                                                         *
*                                                                         *
*      Contact: +33148854006                                              *
*                                                                         *
***************************************************************************

**************************************************************************/

namespace CimSyntaxGen
{
    class Shaclgraph
    {
        // this class manage shacl constraints for a profile
        //Random rand = new Random(1);
        int compteur = 1;
        string profdirectory = "";
        string profname = "";
        XMLParser XMLP;
        string proffile = ""; // path to profile file
        string shaclfile = ""; // file for constraints
        TextReader rw;
        XmlDocument shacldoc;
        XmlNamespaceManager nsmgr;
        RDFLAutilitaires rdflautil;
        XMLProfileParameters profparams;
        ConstantDefinition CD = new ConstantDefinition();
        Dictionary<string, int> ordersdic = new Dictionary<string, int>(); // group/order
        Dictionary<string, Tuple<EA.Connector,string>> connectdic = new Dictionary<string, Tuple<EA.Connector, string>>(); // class/(con,sens)
        List<EA.Element> concreteClasses = new List<EA.Element>();
        Dictionary<string, XmlNode> shapenodedic = new Dictionary<string, XmlNode>();// nomelt/shapenode
        Dictionary<string, List<string>> preddic = new Dictionary<string, List<string>>(); //compound-shape.Name/list of predecessors
        Dictionary<EA.Element,List<string>> compoundsdonedic = new Dictionary<EA.Element,List<string>>(); // originElement/compounds  already treated 
        Dictionary<string, XmlNode> compounddic = new Dictionary<string, XmlNode>();
        Dictionary<string, XmlNode> datshapenoddic = new Dictionary<string, XmlNode>();
        Dictionary<string, XmlNode> cardshapenoddic = new Dictionary<string, XmlNode>();
        List<long> dejatraitee;
        List<string> datmessagelist = new List<string>(); // messages already treated
        EA.Element originElement = null;// element whiche is at the origin of a compound cascade
        List<EA.Element> originElementList = new List<EA.Element>(); // list of origin elements of compounds
        XmlNode shapenode = null; // node for current element
        /// <summary>
        /// 
        /// </summary>
        /// <param name="profparmeters"></param>
        public Shaclgraph(XMLProfileParameters profparmeters)
        {
            XMLP = new XMLParser(Main.Repo);
            try
            {
                connectdic = new Dictionary<string, Tuple<EA.Connector, string>>(); // class/(con,sens)
                profparams = profparmeters;
                rdflautil = new RDFLAutilitaires(profparams, Main.Repo);
                dejatraitee = new List<long>();
                proffile = XMLP.GetXmlValueProfData("LastExportedProfile");
                if (proffile == "") proffile = Main.Repo.ConnectionString;
                profdirectory = Path.GetDirectoryName(proffile);
                profname = Path.GetFileNameWithoutExtension(proffile);
                shaclfile = profdirectory + "\\" + profname + "-constraints" + ".rdf";
                if (System.IO.File.Exists(shaclfile)) System.IO.File.Delete(shaclfile);
                //Stream str = File.Open(   shaclfile, FileMode.OpenOrCreate);
                //XmlWriter rw = new StreamWriter(str);
                shacldoc = new XmlDocument();
                XmlNamespaceManager nsmgr = new XmlNamespaceManager(shacldoc.NameTable);
                nsmgr.AddNamespace("rdf", RDFLAutilitaires.namespaces["rdf"].TrimEnd('#'));
                //string gg = "\u0022"; //guillement exprime en unicode
                string docstart = "";
                EA.Package thepack = Main.Repo.GetTreeSelectedPackage();
                docstart = docstart + rdflautil.GenerEntete(Main.Repo, thepack, RDFutilitaires.GetModelPackage(thepack, Main.Repo), "shacl");
                docstart = docstart + rdflautil.GenerHeader(thepack, RDFutilitaires.GetModelPackage(thepack, Main.Repo), "shacl");
                docstart = docstart + "\n</rdf:RDF>";
                shacldoc.LoadXml(docstart);
                XmlElement RootElement = shacldoc.DocumentElement;
                //XmlElement Element = null;
                //XmlAttribute AttributeXml = null;
                // ----------------   group declarations --------------------------------

                creGroupsh("Cardinality", 0);
                creGroupsh("Datatypes", 1);
                creGroupsh("Associations", 2);

                // ------------------- exploring all the element of the profile ------------------//

                List<EA.Element> profelements = new List<EA.Element>();
                RDFLAutilitaires.getAllElements(thepack, profelements, "all");
                foreach (EA.Element elt in profelements)
                {
                    traitElement(elt);
                }
                // foreach (EA.Element elt in concreteClasses)
                foreach (EA.Element elt in profelements)
                {
                    traitInheritedAssociations(elt);
                }
                // adding the missing sh:properties
                addPropertiesToDescendants();
                //-------------   writing the rdf file   ------------------------------------//
                /*
                using (StreamWriter writer = new StreamWriter(shaclfile, true, Encoding.UTF8))
                {
                    shacldoc.Save(writer);
                }
                */
                XmlWriterSettings settings = new XmlWriterSettings();
                settings.Indent = true;
                UTF8Encoding utf8 = new UTF8Encoding();
                settings.Encoding = utf8;
                // Save the document to a file and auto-indent the output.
                XmlWriter writer = XmlWriter.Create(shaclfile, settings);
                shacldoc.Save(writer);
                writer.Flush();
                writer.Close();

            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("Shaclgraph", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" Shaclgraph {0} ", ee.Message));
            }
        }
             /// <summary>
             /// group shape creation
             ///   <rdf:Description rdf:about="#CardinalityGroup">
             ///    < sh:order rdf:datatype = "http://www.w3.org/2001/XMLSchema#integer" > 0 </ sh:order >
             ///     < rdfs:label > Cardinality </ rdfs:label >
             ///     < rdf:type rdf:resource = "http://www.w3.org/ns/shacl#PropertyGroup" />
             ///  </ rdf:Description >
             /// </summary>
             /// <param name="groupname"></param>
                          
            XmlNode creGroupsh(string groupname,int ordre)
            {
                XmlNode res = null;
                try
                {
                    XmlElement rootelement = shacldoc.DocumentElement;
                    XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                    XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#" + groupname + "Group";
                    Element.Attributes.Append(AttributeXml);
                    XmlNode groupnode = rootelement.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = ordre.ToString();
                    XmlNode eltnode = groupnode.AppendChild(Element);
                    if (!ordersdic.ContainsKey(groupname)) ordersdic[groupname] = 0;

                    Element = shacldoc.CreateElement("rdfs:label", RDFLAutilitaires.namespaces["rdfs"]);
                    Element.InnerText = groupname;
                    eltnode = groupnode.AppendChild(Element);

                    Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/ns/shacl#PropertyGroup";
                    Element.Attributes.Append(AttributeXml);
                    eltnode = groupnode.AppendChild(Element);

                }
                catch (Exception ee)
                {
                    XMLP.AddXmlLog("creGroupsh", String.Format(" {0}", ee.Message));
                    throw new Exception(String.Format("  creGroupsh  {0} ", ee.Message));
                }
                return res;
            }

            /// <summary>
            /// dispatch element treatment
            /// </summary>
            /// <param name="elem"></param>
            void traitElement(EA.Element elem)
            {

                switch(elem.Type)
                {
                    case "Class":
                        if (
                                (elem.StereotypeEx.Contains("enumeration"))
                                ||
                                (elem.StereotypeEx.Contains("Enumeration"))
                            )
                        { 
                            break;
                        }
                        if (elem.StereotypeEx.Contains("Compound"))
                        {
                            traitCompound(elem);
                            break;
                        }
                        if (elem.StereotypeEx.Contains("CIMDatatype"))
                        {
                            traitDatatype(elem);
                            break;
                        }

                        if (elem.StereotypeEx.Contains("Primitive"))
                        {
                            traitPrimitive(elem);
                            break;
                        }
                        if(elem.Name=="") //note
                        {
                            break;
                        }
                        traitClass(elem);
                        break;
                    
                    default:
                        break;
                }

            }
            /// <summary>
            /// Compound treatment
            /// </summary>
            /// <param name="elem"></param>
            void traitCompound(EA.Element elem)
            {
                try
                {
                }
                catch (Exception ee)
                {
                    XMLP.AddXmlLog("Shacl traitCompound", String.Format(" {0}", ee.Message));
                    throw new Exception(String.Format(" Shacl traitCompound  {0} ", ee.Message));
                }
            }
            /// <summary>
            /// Datatype treatment
            /// </summary>
            /// <param name="elem"></param>
            void traitDatatype(EA.Element elem)
            {
                try
                {
                }
                catch (Exception ee)
                {
                    XMLP.AddXmlLog("Shaclgraph traitDatatype", String.Format(" {0}", ee.Message));
                    throw new Exception(String.Format(" Shaclgraph traitDatatype  {0} ", ee.Message));
                }

            }
            /// <summary>
            /// Primitive treatment
            /// </summary>
            /// <param name="elem"></param>
            void traitPrimitive(EA.Element elem)
            {
                try
                {
                }
                catch (Exception ee)
                {
                    XMLP.AddXmlLog("trait Primitive ", String.Format(" {0}", ee.Message));
                    throw new Exception(String.Format(" Shaclgraph traitPrimitive  {0} ", ee.Message));
                }

            }

        /// <summary>
        /// Class treatment
        /// </summary>
        /// <param name="elem"></param>
        void traitClass(EA.Element elem)
        {
            RDFLAutilitaires.wtest("TEST", String.Format(" traitclass elem={0}",elem.Name));
            /* ABA20230905
            if (elem.Name == "Equipment")
            {
                string prov = "";
            }*/
            try
            {
                if (elem.Abstract == "1")
                {
                    // if the class is abstract and has descendants then we have to generate the corresponding propoerty shapes
                    List<EA.Element> elemlist = RDFLAutilitaires.getDescendantsInProfile(elem);
                    if (elemlist.Count > 0) // must gener shapes
                    {
                        foreach (EA.Attribute at in elem.Attributes)
                        {
                            genCardinalityPropertyShape(elem, at,new List<string>());
                            genDatatypePropertyShape(elem, at, new List<string>());
                        }
                    }
                }
                else  // Class is concrete
                {
                    if (!concreteClasses.Contains(elem))
                    {
                        concreteClasses.Add(elem);
                    }
                    XmlNode shapenode = genShapeNode(elem);
                    if (!shapenodedic.ContainsKey(elem.Name))
                    {
                        shapenodedic.Add(elem.Name, shapenode);
                    }
                    shapenodedic[elem.Name] = shapenode;
                    XmlElement ps;
                    // ------------  the direct  attributes  ------------------ //
                    List<string> atlist = new List<string>();
                    foreach (EA.Attribute atr in elem.Attributes)
                    {
                        RDFLAutilitaires.wtest("TEST", String.Format(" traitclass direc attribut  elem={0} atr={1}", elem.Name,atr.Name ));
                        atlist.Add(atr.AttributeGUID);
                        XmlElement Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + elem.Name + "." + atr.Name + "-cardinality";
                        Element.Attributes.Append(AttributeXml);
                        shapenode.AppendChild(Element);
                        List<string> cardpredecessors = new List<string>();
                        ps = genCardinalityPropertyShape(elem, atr,cardpredecessors);
                        Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + elem.Name + "." + atr.Name + "-datatype";
                        Element.Attributes.Append(AttributeXml);
                        shapenode.AppendChild(Element);
                        List<string> predecessors = new List<string>();
                        ps = genDatatypePropertyShape(elem, atr, predecessors);
                    }
                    // ---   les attributs hérités -----
                    foreach (EA.Attribute atr in elem.AttributesEx)
                    {
                        if (atlist.Contains(atr.AttributeGUID)) continue; //native
                        EA.Element parentel = Main.Repo.GetElementByID((int)atr.ParentID);
                        XmlElement Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + parentel.Name + "." + atr.Name + "-cardinality";
                        Element.Attributes.Append(AttributeXml);
                        shapenode.AppendChild(Element);
                        //ps = genCardinalityPropertyShape(elem, atr);
                        Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + parentel.Name + "." + atr.Name + "-datatype";
                        Element.Attributes.Append(AttributeXml);
                        shapenode.AppendChild(Element);
                        //ps = genDatatypePropertyShape(elem, atr, predecessors);
                    }
                }
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("Shaclgraph traitClass", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" Shaclgraph traitClass  {0} ", ee.Message));
            }
        }
        // XmlNode genShapeNode(string classname)
        XmlNode genShapeNode(EA.Element elem)
        {
            string classname = elem.Name;
            XmlNode res=null;
            try
            { 
               XmlElement rootelement = shacldoc.DocumentElement;
               XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
               XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
               AttributeXml.Value = "#" + classname;
               Element.Attributes.Append(AttributeXml);
               XmlNode shapenode = rootelement.AppendChild(Element);

               Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
               AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
               AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "NodeShape";
               Element.Attributes.Append(AttributeXml);
               XmlNode anode = shapenode.AppendChild(Element);

               Element = shacldoc.CreateElement("sh:targetClass", RDFLAutilitaires.namespaces["sh"]);
               AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                // AttributeXml.Value = RDFLAutilitaires.namespaces["cim"]  + classname;
                AttributeXml.Value = getPathURI(elem) + classname;
                 Element.Attributes.Append(AttributeXml);
               anode = shapenode.AppendChild(Element);
               
               res = shapenode;
            }
            catch (Exception ee)
            {
               XMLP.AddXmlLog("Shaclgraph genShapeNode", String.Format(" {0}", ee.Message));
               throw new Exception(String.Format(" Shaclgraph genShapeNode  {0} ", ee.Message));
            }
               return res;
        }
            // }


        //}

        /// <summary>
        ///  generate shape for inherited association
        /// </summary>
        /// <param name="elt"></param>
        private void traitInheritedAssociations(Element elem)
        {
            /* ABA20230905
            if(elem.Name=="Equipment")
            {
                string tt = "";
            }*/
            try
            { 
                List<EA.Element> descendants = new List<EA.Element>();
                RDFLAutilitaires.getAllDescendantsInProfile(elem, descendants);

                foreach (EA.Connector con in elem.Connectors)
                {
                    if (XMLP.GetXmlValueShaclData("ShapesFromCim") == "true")
                    {
                        // nothing sofar
                    }
                    else// this is the normal approach for profile we select only the orientated association
                    {
                        if (con.Type != "Association") continue;
                        string sens = Eligible(elem, con);

                        if (sens == "") continue; // the navigable side is not proper
                        if (dejatraitee.Contains(con.ConnectorID))
                        {
                            continue;
                        }
                        else
                        {
                            dejatraitee.Add(con.ConnectorID);
                        }
                        if (!connectdic.ContainsKey(elem.Name))
                        {
                            connectdic.Add(elem.Name,new Tuple<Connector, string>(con, sens));
                        }
                        //there are two cases either there are several inherited classes at the otherend or only one
                        string endrole;
                        EA.Element conend_element;
                        if (sens == "source")
                        {
                            endrole = con.ClientEnd.Role;
                            conend_element = Main.Repo.GetElementByID((int)con.ClientID);
                        }
                        else
                        {
                            endrole = con.SupplierEnd.Role;
                            conend_element=Main.Repo.GetElementByID((int)con.SupplierID);
                        }
                        List<EA.Element> endfilles = new List<EA.Element>();
                        RDFLAutilitaires.getAllDescendantsInProfile(conend_element, endfilles); ;
                        /* ABA20230905
                        if (endrole=="DiagramObject")
                        {
                            string pr = "";
                        }*/
                        XmlNode ps;
                        XmlElement Element;
                        XmlAttribute AttributeXml;
                        if (elem.Abstract!="1")
                                {
                            Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "#" + elem.Name + "." + endrole + "-cardinality";
                            Element.Attributes.Append(AttributeXml);
                            shapenodedic[elem.Name].AppendChild(Element);
                        }
                        ps = genAssociationCardinalityPropertyShape(elem, con,sens);
                        if (elem.Abstract != "1")
                        {
                            if (endfilles.Count == 0)// a shapeNode can't be declared as a sh:property
                            {
                                Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                                AttributeXml.Value = "#" + elem.Name + "." + endrole + "-valueType";
                                Element.Attributes.Append(AttributeXml);
                                shapenodedic[elem.Name].AppendChild(Element);
                            }
                        }
                         ps = genAssociatonValuetypePropertyShape(elem, con, sens, descendants);// elem is added at descendants
                        
                        descendants= new List<EA.Element>();
                        RDFLAutilitaires.getAllDescendantsInProfile(elem,descendants);
                        foreach (EA.Element descendant in descendants)
                        {
                            // if (descendant.Abstract == "1") continue;
                            if (descendant.Abstract != "1")
                            {
                               // if (endfilles.Count == 0)// a shapeNode can't be declared as a sh:property// a shapeNode can't be declared as a sh:property
                               // {
                                    Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                                    AttributeXml.Value = "#" + elem.Name + "." + endrole + "-valueType";
                                    Element.Attributes.Append(AttributeXml);
                                    shapenodedic[descendant.Name].AppendChild(Element);

                                    Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                                    AttributeXml.Value = "#" + elem.Name + "." + endrole + "-cardinality";
                                    Element.Attributes.Append(AttributeXml);
                                    shapenodedic[descendant.Name].AppendChild(Element);

                               // }
                            }
                            
                            
                        }
                        
                       
                    }
                }
                
            }
            catch(Exception ee)
            {
                throw new Exception("traitInheritedAssociations " + ee.Message);
            }

        }

        private XmlElement genAssociatonValuetypePropertyShape(EA.Element elem, EA.Connector con, string sens,List<EA.Element> descendants)
        {
            XmlElement res = null;
            List<EA.Element> elemfilles = new List<EA.Element>();
            /* ABA20230905
            if(elem.Name=="PositionPoint")
            {
                string hh = "";
            }*/
            /* ABA20230905
            if ((elem.Name == "PositionPoint")  && ((con.ClientEnd.Role == "Location") || (con.SupplierEnd.Role == "PositionPoints")))
            {
                string prov = "";
            }*/

            try
            {
                    EA.ConnectorEnd conend;
                    EA.Element theotherend;
                    if(sens=="source")
                    {
                        conend = con.ClientEnd;
                        theotherend = Main.Repo.GetElementByID(con.ClientID);
                    }else
                    {
                        conend = con.SupplierEnd;
                        theotherend = Main.Repo.GetElementByID(con.SupplierID);
                    }
                    RDFLAutilitaires.getAllDescendantsInProfile(theotherend,elemfilles);
                    if (elemfilles.Count == 0)
                    {
                        creAssociationValueTypeShape(elem, con, sens, null);
                    }
                    else
                    {
                        if(elem.Abstract != "1")//the class is concrete and must b included
                         {
                           if (!elemfilles.Contains(theotherend)) elemfilles.Add(theotherend);
                         }

                        XmlElement rootelement = shacldoc.DocumentElement;
                        XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + elem.Name + "." + conend.Role + "-valueType";
                        Element.Attributes.Append(AttributeXml);
                        XmlNode shapenode = rootelement.AppendChild(Element);
                    

                    Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "NodeShape";
                        Element.Attributes.Append(AttributeXml);
                        XmlNode anode = shapenode.AppendChild(Element);

                        Element = shacldoc.CreateElement("sh:or", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "Collection";
                        Element.Attributes.Append(AttributeXml);
                        XmlNode shinnode = shapenode.AppendChild(Element);

                        descendants.Add(elem);
                        foreach (EA.Element elt in descendants)
                        {
                         
                        /* // apparently the only necessary sh:target are in the NoseShape
                             if (elt.Abstract != "1")
                             {
                                 Element = shacldoc.CreateElement("sh:targetClass", RDFLAutilitaires.namespaces["sh"]);
                                 AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            // AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elt.Name;
                                 AttributeXml.Value = getPathURI(elt) + elt.Name;
                                 Element.Attributes.Append(AttributeXml);
                                 anode = shapenode.AppendChild(Element);
                             }
                        */
                        }
                        foreach (EA.Element elt in elemfilles)
                        {

                        if (elt.Abstract != "1")
                        {
                            Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "#" + elem.Name + "." + conend.Role + elt.Name + "-valueType";
                            Element.Attributes.Append(AttributeXml);
                            anode = shinnode.AppendChild(Element);
                            creAssociationValueTypeShape(elem, con, sens, elt);
                        }
                        }

                }
               
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("genAssociatonValuetypePropertyShape", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" genAssociatonValuetypePropertyShape  {0} ", ee.Message));
            }
            
            return res;
        }

        XmlElement creAssociationValueTypeShape(EA.Element elem, EA.Connector con, string sens,EA.Element elemfille)
        {
            XmlElement res = null;
            try {

                /* ABA20230905
                if ((elem.Name == "Diagram") && ((con.ClientEnd.Role == "description") || (con.SupplierEnd.Role == "description")))
                {
                    string prov = "";
                }*/
                EA.ConnectorEnd conend = null;
                EA.Element theotherend = null;
                EA.Element myend = null;
                if(sens=="source")
                {
                    conend = con.ClientEnd;
                    theotherend = Main.Repo.GetElementByID((int)con.ClientID);
                    myend= Main.Repo.GetElementByID((int)con.SupplierID); 
                }
                else
                {
                    conend = con.SupplierEnd;
                    theotherend = Main.Repo.GetElementByID((int)con.SupplierID);
                    myend= Main.Repo.GetElementByID((int)con.ClientID);
                }
                try
                {

                    XmlElement rootelement = shacldoc.DocumentElement;
                    XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                    XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    if (elemfille == null)
                    {
                        AttributeXml.Value = "#" + elem.Name + "." + conend.Role +  "-valueType";
                    }
                    else
                    {
                        AttributeXml.Value = "#" + elem.Name + "." + conend.Role + elemfille.Name + "-valueType";
                    }
                    Element.Attributes.Append(AttributeXml);
                    XmlNode shapenode = rootelement.AppendChild(Element);

                    Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "PropertyShape";
                    Element.Attributes.Append(AttributeXml);
                    XmlNode anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:group", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#AssociationGroup";
                    Element.Attributes.Append(AttributeXml);
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);
                    int order = ordersdic["Associations"];
                    ordersdic["Associations"] = order + 1;
                    //order = 0;
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = order.ToString();
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + conend.Role;
                    AttributeXml.Value = getPathURI(elem) + elem.Name + "." + conend.Role;
                    Element.Attributes.Append(AttributeXml);
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = XMLP.GetXmlValueShaclData("descriptionValueType");
                    anode = shapenode.AppendChild(Element);

                    
                    Element = shacldoc.CreateElement("sh:severity", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Violation";
                    Element.Attributes.Append(AttributeXml);
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "IRI";
                    Element.Attributes.Append(AttributeXml);
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:name", RDFLAutilitaires.namespaces["sh"]);
                    if(elemfille==null)
                    {
                        Element.InnerText = elem.Name + "." + conend.Role +  "-valueType";
                    }
                    else
                    {
                        Element.InnerText = elem.Name + "." + conend.Role + elemfille.Name + "-valueType";
                    }
                    anode = shapenode.AppendChild(Element);


                    Element = shacldoc.CreateElement("sh:class", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    if (elemfille != null)
                    {
                        //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elemfille.Name;
                        AttributeXml.Value = getPathURI(elemfille) + elemfille.Name;
                    }
                    else
                    {
                        //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + theotherend.Name;
                        AttributeXml.Value = getPathURI(theotherend) + theotherend.Name;
                    }
                    Element.Attributes.Append(AttributeXml);
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                    string message= XMLP.GetXmlValueShaclData("messageValueType");
                    if (elemfille != null)
                    {
                        message = message.Replace("{the IRI of the class}", "cim:" + elemfille.Name);
                    }
                    else
                    {
                        message = message.Replace("{the IRI of the class}", "cim:" + theotherend.Name);
                    }
                    Element.InnerText = message;
                    anode = shapenode.AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = XMLP.GetXmlValueShaclData("descriptionValueType");
                    anode = shapenode.AppendChild(Element);

                    if(elemfille==null)
                    {
                        List<EA.Element> descendants = new List<EA.Element>();
                        RDFLAutilitaires.getAllDescendantsInProfile(myend, descendants);             
                        descendants.Add(elem);
                        foreach (EA.Element elt in descendants)
                        {
                            /*// apparently the only necessary sh:target are in the NoseShape
                            if (elt.Abstract != "1")
                            {
                                Element = shacldoc.CreateElement("sh:targetClass", RDFLAutilitaires.namespaces["sh"]);
                                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                                //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elt.Name;
                                AttributeXml.Value = getPathURI(elt) + elt.Name;
                                Element.Attributes.Append(AttributeXml);
                                anode = shapenode.AppendChild(Element);
                            }
                            */
                        }
                    }
                }
                catch (Exception ee)
                {
                    XMLP.AddXmlLog("creAssociationValueTypeShape", String.Format(" {0}", ee.Message));
                    throw new Exception(String.Format(" creAssociationValueTypeShape {0} ", ee.Message));
                }
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("creAssociationValueTypeShape", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" creAssociationValueTypeShape  {0} ", ee.Message));
            }
            return res;
}
        private XmlElement genAssociationCardinalityPropertyShape(EA.Element elem, EA.Connector con, string sens)
        {
            XmlElement res = null;
            /* ABA20230905
            if ((elem.Name == "Diagram") && ((con.ClientEnd.Role == "description") || (con.SupplierEnd.Role == "description")))
            {
                string prov = "";
            }*/
            
            try
            {
                int intlowerbound=0;
                int intupperbound=0;
                string lowerbound="";
                string upperbound="";
                EA.ConnectorEnd conend;
                if (sens == "source")
                {
                    conend = con.ClientEnd;
                }
                else
                {
                    conend = con.SupplierEnd;
                }
                if (conend.Cardinality == "1")
                {
                    intlowerbound = 1;
                    lowerbound = "1";
                    intupperbound = 1;
                    upperbound = "1";
                }
                if (conend.Cardinality.Contains(".."))
                {
                    string[] sep = { ".." };
                    string[] card = conend.Cardinality.Split(sep, StringSplitOptions.None);
                    lowerbound = card[0];
                    upperbound = card[1];
                    if ((upperbound == "*") || (upperbound == "n"))
                    {
                        intupperbound = 0;
                    }
                    else
                    {
                        intupperbound = Int32.Parse(upperbound);
                    }
                    intlowerbound = Int32.Parse(lowerbound);
                }
                XmlElement rootelement = shacldoc.DocumentElement;
                XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "#" + elem.Name + "." + conend.Role + "-cardinality";
                Element.Attributes.Append(AttributeXml);
                XmlNode shapenode = rootelement.AppendChild(Element);
                
                Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "PropertyShape";
                Element.Attributes.Append(AttributeXml);
                XmlNode anode = shapenode.AppendChild(Element);
                
                Element = shacldoc.CreateElement("sh:group", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "#CardinalityGroup";
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);
                int order = ordersdic["Cardinality"];
                ordersdic["Cardinality"] = order + 1;
                AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                Element.Attributes.Append(AttributeXml);
                Element.InnerText = order.ToString();
                anode = shapenode.AppendChild(Element);

                if (intlowerbound > 0)
                {
                    Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = intlowerbound.ToString();
                    anode = shapenode.AppendChild(Element);
                }
                if (intupperbound > 0)
                {
                    Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = intupperbound.ToString();
                    anode = shapenode.AppendChild(Element);
                }
                Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + conend.Role;
                AttributeXml.Value = getPathURI(elem) + elem.Name + "." + conend.Role;
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = getCardinalityShapeMessage("association", intlowerbound, intupperbound, "");
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = XMLP.GetXmlValueShaclData("descriptionAssociationCardinality");
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:severity", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Violation";
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:name", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = elem.Name + "." + conend.Role + "-cardinality";
                anode = shapenode.AppendChild(Element);
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("genAssociationCardinalityPropertyShape", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" genAssociationCardinalityPropertyShape  {0} ", ee.Message));
            }
            
                return res;
        }

        string Eligible(EA.Element elem,EA.Connector con)
        {
            string res = "";
            if (con.ClientID == elem.ElementID)
            {
               res = "target";
               if (con.SupplierEnd.Navigable != "Navigable") res = "";
            }
            else
            {
             res = "source";
             if (con.ClientEnd.Navigable != "Navigable") res = "";
            }     
            return res;
        }
        /// <summary>
        ///  creates the sh:property entries in the original element
        ///  and generete the schema for the cascaded classifier
        /// </summary>
        /// <param name="elem"></param>
        /// <param name="atr"></param>
        /// <param name="classifier"></param>
        /// <param name="predecessors"></param>
        public void oldcascade(EA.Element elem, EA.Attribute atr, EA.Element classifier,List<string> predecessors)
        {

            string tt = "";
            foreach (string p in predecessors)
            {
                int ind = p.IndexOf("#");
                tt = tt + "|" + p.Substring(ind);
            }

            string texte = String.Format("cascade el={0} atr={1} classifier={2} pred={3}", elem.Name, atr.Name, classifier.Name, tt);
            RDFLAutilitaires.wtest("TEST", texte);
            /* ABA20230905
            if ((elem.Name == "Location") && (atr.Name == "mainAdress"))
            {
                string dd = "";
            }*/
            //XmlElement res = null;
            try
            {
               
                shapenode = shapenodedic[originElement.Name];
                string nomcompound = elem.Name + "." + atr.Name;

                
                foreach (EA.Attribute at in classifier.Attributes)
                {
                    string compoundname= classifier.Name + "." + at.Name + "-datatype";
                   
                    if (!compoundsdonedic.ContainsKey(originElement)) compoundsdonedic[originElement] = new List<string>();
                    if (!compoundsdonedic[originElement].Contains(compoundname))
                    {
                        compoundsdonedic[originElement].Add(compoundname);
                        XmlElement Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "#" + classifier.Name + "." + at.Name + "-datatype";
                        Element.Attributes.Append(AttributeXml);
                        XmlNode node = shapenode.AppendChild(Element);
                    }
                    string predecessor = elem.Name + "." + atr.Name;

                     genDatatypePropertyShape(classifier, at, predecessors);
                }
                if (predecessors.Count != 0) predecessors.RemoveAt(predecessors.Count - 1);
                
            }
            catch(Exception ee)
            {
                //throw new Exception(String.Format("cascade {0}.{1} classifier={2} level={3} err={4}", elem.Name, atr.Name, classifier.Name, predecessors.Count.ToString(),ee.Message));
                throw new Exception(String.Format("cascade {0}.{1} classifier={2} err={3}", elem.Name, atr.Name, classifier.Name, ee.Message));
            }
        }

        /// <summary>
        ///  creates the sh:property entries in the original element
        ///  and generete the schema for the cascaded classifier
        ///  the propertyshape is specified by kind
        /// </summary>
        /// <param name="elem"></param>
        /// <param name="atr"></param>
        /// <param name="classifier"></param>
        /// <param name="kind"></param> // -datatype | -cardinality
        /// <param name="predecessors"></param>
        public void cascade(EA.Element elem, EA.Attribute atr, EA.Element classifier,string kind, List<string> predecessors)
        {
            
            string tt = "";
            foreach (string p in predecessors)
            {
                int ind = p.IndexOf("#");
                tt = tt + "|" + p.Substring(ind);
            }

            string texte = String.Format("cascade el={0} atr={1} classifier={2} kind={3} pred={4}", elem.Name, atr.Name, classifier.Name,kind, tt);
            RDFLAutilitaires.wtest("TEST", texte);
            /* ABA20230905
            if ((elem.Name == "Location") && (atr.Name == "mainAdress"))
            {
                string dd = "";
            }*/
            //XmlElement res = null;
            try
            {
               
                shapenode = shapenodedic[originElement.Name];
                string nomcompound = elem.Name + "." + atr.Name;


                foreach (EA.Attribute at in classifier.Attributes)
                {
                    //string compoundname = classifier.Name + "." + at.Name + "-datatype";
                     string compoundname = classifier.Name + "." + at.Name + kind;

                    if (!compoundsdonedic.ContainsKey(originElement)) compoundsdonedic[originElement] = new List<string>();
                    if (!compoundsdonedic[originElement].Contains(compoundname))
                    {
                        compoundsdonedic[originElement].Add(compoundname);
                        XmlElement Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        //AttributeXml.Value = "#" + classifier.Name + "." + at.Name + "-datatype";
                        AttributeXml.Value = "#" + classifier.Name + "." + at.Name + kind;
                        Element.Attributes.Append(AttributeXml);
                        XmlNode node = shapenode.AppendChild(Element);
                    }
                    string predecessor = elem.Name + "." + atr.Name;

                    if (kind == "-datatype")
                    {
                        genDatatypePropertyShape(classifier, at, predecessors);
                    }
                    else 
                    {
                        genCardinalityPropertyShape(classifier, at, predecessors);
                    }
                }
                if (predecessors.Count != 0) predecessors.RemoveAt(predecessors.Count - 1);

            }
            catch (Exception ee)
            {
                throw new Exception(String.Format("cascade {0}.{1} classifier={2} err={3}", elem.Name, atr.Name, classifier.Name, ee.Message));
            }
        }

        /// <summary>
        /// to create the dtatype shape according to the atrribute of a class
        /// </summary>
        /// <param name="elem"></param>
        /// <param name="atr"></param>
        /// <param name="predecessors">In case of compounding the predecessors</param>
        /// <returns></returns>
        XmlElement genDatatypePropertyShape(Element elem, EA.Attribute atr, List<string> predecessors)
        {
            /************* test **********************/
            string tt = "";
            string atrname = elem.Name + "." + atr.Name;
            /* ABA20230905
            if(atrname=="StreetDetail.number")
            {
                string hh = "";
            }*/
            foreach(string p in predecessors )
            {
                int ind = p.IndexOf("#");
                tt = tt + "|" + p.Substring(ind);
            }
            string texte = String.Format("genDatatypePropertyShape el={0} atr={1} pred={2}", elem.Name, atr.Name, tt);
            RDFLAutilitaires.wtest("TEST", texte );
            /*********************************************/
            /* ABA20230905
            if((elem.Name=="StreetAddress") && (atr.Name=="streetDetail"))
            {
                string ff = "";
            }*/
            XmlElement res = null;
            try
            {
                XmlAttribute AttributeXml;
                XmlElement Element;
                XmlNode  anode;
                string[] messages;
                if (!datshapenoddic.ContainsKey(elem.Name + "." + atr.Name+"datatype"))
                        {
                    XmlElement rootelement = shacldoc.DocumentElement;
                     Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                     AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#" + elem.Name + "." + atr.Name + "-datatype";
                    Element.Attributes.Append(AttributeXml);
                    XmlNode datshapenode = rootelement.AppendChild(Element);
                    datshapenoddic[elem.Name + "." + atr.Name+"datatype"] = datshapenode;


                    Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "PropertyShape";
                    Element.Attributes.Append(AttributeXml);
                     anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:group", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#DatatypesGroup";
                    Element.Attributes.Append(AttributeXml);
                    anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);



                    Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);
                    int order = ordersdic["Datatypes"];
                    ordersdic["Datatypes"] = order + 1;
                    //order = 0;
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = order.ToString();
                    anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                    string ss = XMLP.GetXmlValueShaclData("descriptionDatatype");
                    Element.InnerText = ss.Replace("attribute", atr.Name);
                    anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:severity", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Violation";
                    Element.Attributes.Append(AttributeXml);
                    anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:name", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = elem.Name + "." + atr.Name + "-datatype";
                    anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                }
                    //there are  4 cases
                    // 1- type is primitive
                    // 2- type is CimDatatype
                    // 3- type is enumeration
                    // 4- type is Compound
                    string type = "Primitive";
                    EA.Element classifier = Main.Repo.GetElementByID((int)atr.ClassifierID);
                    if (classifier.StereotypeEx.Contains("Primitive"))
                    {
                        type = "Primitive";
                    }
                    else
                    {
                        if (classifier.StereotypeEx.Contains("Compound"))
                        {
                            type = "Compound";
                        }
                        else
                        {
                            if (classifier.StereotypeEx.Contains("CIMDatatype"))
                            {
                                type = "CIMDatatype";
                            }
                            else
                            {
                                if (
                                    (classifier.StereotypeEx.Contains("Enumeration"))
                                    ||
                                    (classifier.StereotypeEx.Contains("enumeration"))
                                    ||
                                    (classifier.Type == "Enumeration")
                                    )
                                {
                                    type = "Enumeration";
                                }
                                else
                                {
                                    XMLP.AddXmlLog("Shacl genDatatypePropertyShappe", " type of classifier unknown for " + elem.Name + "." + atr.Name);
                                    throw new Exception("Shacl genDatatypePropertyShappe type of classifier unknown for " + elem.Name + "." + atr.Name);
                                }
                            }
                        }
                    }
                
                   
                switch (type)
                {
                    case "Primitive":
                        if (predecessors.Count > 0)
                        {
                            //generCompoundPath(datshapenode,elem, atr, predecessors);
                            string nodekind = "http://www.w3.org/ns/shacl#Literal";
                            string datatype = RDFLAutilitaires.namespaces["xsd"] + classifier.Name.ToLower();
                            //generPath(datshapenode, atrname,  nodekind, datatype, predecessors,getPathtURI(elem,atr));
                            generPath(datshapenoddic[atrname+"datatype"], atrname, nodekind, datatype, predecessors, getPathtURI(elem, atr));
                        }
                        else
                        {
                            Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                            AttributeXml.Value =getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                            Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Literal";
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            Element = shacldoc.CreateElement("sh:datatype", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["xsd"] + classifier.Name.ToLower();
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                        }
                        if (!datmessagelist.Contains(atrname + "datatype"))
                        {
                            datmessagelist.Add(atrname + "datatype");
                            Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                            messages = XMLP.GetXmlValueShaclData("messageDatatype").Split(new char[] { '|' });
                            Element.InnerText = messages[0];
                            anode = datshapenoddic[atrname + "datatype"].AppendChild(Element);
                        }
                        break;
                    case "CIMDatatype":
                        string typ = "";
                        foreach (EA.Attribute at in classifier.Attributes)
                        {
                            if (at.Name == "value")
                            {
                                typ = Main.Repo.GetElementByID((int)at.ClassifierID).Name;
                                break;
                            }
                        }
                        if (predecessors.Count > 0)
                        {
                            string nodekind = "http://www.w3.org/ns/shacl#Literal";
                            string datatype = RDFLAutilitaires.namespaces["xsd"] + typ.ToLower();
                            // generPath(datshapenoddic[atrname+"datatype"], atrname, nodekind, datatype, predecessors);
                            generPath(datshapenoddic[atrname+"datatype"], atrname, nodekind, datatype, predecessors,getPathtURI(elem,atr));
                        }
                        else
                        {
                            Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            // AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                            AttributeXml.Value = getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            Element = shacldoc.CreateElement("sh:datatype", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["xsd"] + typ.ToLower();
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Literal";
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                        }
                        if (!datmessagelist.Contains(atrname + "datatype"))
                        {
                            datmessagelist.Add(atrname + "datatype");
                            Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                            messages = XMLP.GetXmlValueShaclData("messageDatatype").Split(new char[] { '|' });
                            Element.InnerText = messages[0];
                            anode = datshapenoddic[atrname + "datatype"].AppendChild(Element);
                        }
                        break;
                    case "Enumeration":
                        /*
                        if (predecessors.Count > 0)
                        {
                        }
                        else 
                        { 
                        */
                            Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                           //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                           AttributeXml.Value = getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                          Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                        //}

                            if (!Main.dicEnums.ContainsKey(classifier.Name))
                                Main.dicEnums[classifier.Name] = new Dictionary<string, string>();
                            foreach (EA.Attribute member in classifier.Attributes)
                            {
                                Main.dicEnums[classifier.Name][member.Name] = "";
                                if (member.Default != null) Main.dicEnums[classifier.Name][member.Name] = member.Default;
                            }

                            Element = shacldoc.CreateElement("sh:in", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "Collection";
                            Element.Attributes.Append(AttributeXml);
                            XmlNode shinnode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            foreach (string member in Main.dicEnums[classifier.Name].Keys)
                            {
                                Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                                AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                                AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + classifier.Name + "." + member;
                                Element.Attributes.Append(AttributeXml);
                                anode = shinnode.AppendChild(Element);
                            }

                            Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "IRI";
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                        if (!datmessagelist.Contains(atrname + "datatype"))
                        {
                            datmessagelist.Add(atrname + "datatype");
                            Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                            messages = XMLP.GetXmlValueShaclData("messageDatatype").Split(new char[] { '|' });
                            Element.InnerText = messages[1];
                            anode = datshapenoddic[atrname + "datatype"].AppendChild(Element);
                        }
                            break;
                        case "Compound":
                       
                        if (!datmessagelist.Contains(atrname + "datatype"))
                        {
                            datmessagelist.Add(atrname + "datatype");
                            Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                            messages = XMLP.GetXmlValueShaclData("messageDatatype").Split(new char[] { '|' });
                            //Element.InnerText = messages[0];
                            // COMPOUNDTYPE
                            string ss = messages[2].Replace("datatype", classifier.Name);
                            Element.InnerText = ss;
                            anode = datshapenoddic[atrname + "datatype"].AppendChild(Element);
                        }

                        // it is where we create the pointers to the predecessors 
                        if (predecessors.Count > 0)
                        {
                            string nodekind= RDFLAutilitaires.namespaces["sh"] + "BlankNode";

                            //generPath(datshapenoddic[atrname+"datatype"], atrname, nodekind, classifier.Name, predecessors);
                            generPath(datshapenoddic[atrname+"datatype"], atrname, nodekind, classifier.Name, predecessors,getPathtURI(elem,atr));
                        }
                        else
                        {

                            Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "BlankNode";
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                            AttributeXml.Value = getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);

                            Element = shacldoc.CreateElement("sh:class", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                            // AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + classifier.Name;
                            AttributeXml.Value = getPathURI(classifier) + classifier.Name;
                            Element.Attributes.Append(AttributeXml);
                            anode = datshapenoddic[atrname+"datatype"].AppendChild(Element);
                        }
                        // add cascades
                        //<rdf:Description rdf:about="#Location.mainAddress-datatype">
                        if (predecessors.Count == 0)
                        {
                            originElement = elem;
                            if(!originElementList.Contains(elem))
                            {
                                originElementList.Add(elem);
                            }
                        }

                        //predecessors.Add(RDFLAutilitaires.namespaces["cim"] + elem.Name +"."+ atr.Name);
                        string predecessor = getPathtURI(elem, atr) + elem.Name + "." + atr.Name;
                        if(!predecessors.Contains(predecessor)) predecessors.Add(predecessor);
                        // a ce niveau on se branche sur une nouvelle branche de compound
                        // il faut creer le point d'entrée    
                        cascade(elem, atr, classifier,"-datatype",predecessors);
        
                                    break;

                        default:
                            break;
                    }
                
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("Shaclgraph genDatatypePropertyShape", String.Format("elem={0} atr={1} err= {2}",elem,atr, ee.Message));
                throw new Exception(String.Format(" Shaclgraph genDatatypePropertyShape  {0} ", ee.Message));
            }
            return res;
        }

        
        /// <summary>
        /// to create the Cardilality shape according to the atrribute of a class
        /// </summary>
        /// <param name="elem"></param>
        /// <param name="atr"></param>
        /// <returns></returns>
        XmlElement oldgenCardinalityPropertyShape(Element elem, EA.Attribute atr)
        {
            /* ABA20230905
            if((elem.Name =="Diagram") && (atr.Name=="description"))
            {
                string prov = "";
            }*/
            XmlElement res = null;
            try
            {
                string lowerbound = atr.LowerBound;
                string upperbound = atr.UpperBound;
                int intlowerbound = Int32.Parse(lowerbound);
                int intupperbound;
                if((upperbound=="*") || (upperbound == "*"))
                {
                    intupperbound = 0;
                }
                else
                {
                    intupperbound = Int32.Parse(upperbound);
                }
               
                XmlElement rootelement = shacldoc.DocumentElement;
                XmlElement Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "#" + elem.Name +"." + atr.Name+"-cardinality";
                Element.Attributes.Append(AttributeXml);
                XmlNode shapenode = rootelement.AppendChild(Element);

                Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "PropertyShape";
                Element.Attributes.Append(AttributeXml);
                XmlNode anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:group", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "#CardinalityGroup";
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);             
                int order = ordersdic["Cardinality"];
                ordersdic["Cardinality"] =order+1;
                AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                Element.Attributes.Append(AttributeXml);
                Element.InnerText=order.ToString();
                anode = shapenode.AppendChild(Element);

                if (intlowerbound > 0)
                {
                    Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = intlowerbound.ToString();
                    anode = shapenode.AppendChild(Element);
                }
                if(intupperbound > 0)
                {
                    Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = intupperbound.ToString();
                    anode = shapenode.AppendChild(Element);
                }
                Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);
                
                
                Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = getCardinalityShapeMessage("attribute", intlowerbound, intupperbound, atr.Name); 
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = XMLP.GetXmlValueShaclData("descriptionAttributeCardinality");
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:severity", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Violation";
                Element.Attributes.Append(AttributeXml);
                anode = shapenode.AppendChild(Element);

                Element = shacldoc.CreateElement("sh:name", RDFLAutilitaires.namespaces["sh"]);
                Element.InnerText = elem.Name + "." + atr.Name + "-cardinality";
                anode = shapenode.AppendChild(Element);

            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("genCardinalityPropertyShape", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" genCardinalityPropertyShape  {0} ", ee.Message));
            }
            return res;
        }
        /// <summary>
        /// to create the Cardilality shape according to the atrribute of a class
        /// </summary>
        /// <param name="elem"></param>
        /// <param name="atr"></param>
        /// <returns></returns>
        XmlElement genCardinalityPropertyShape(Element elem, EA.Attribute atr, List<string>predecessors)
        {
            string clefcard = "";
            /* ABA20230905
            if ((elem.Name == "StreetDetail") && (atr.Name == "number"))
            {
                string prov = "";
            }*/
            XmlElement res = null;
            /************* test **********************/
            string tt = "";
            string atrname = elem.Name + "." + atr.Name;
            foreach (string p in predecessors)
            {
                int ind = p.IndexOf("#");
                tt = tt + "|" + p.Substring(ind);
            }
            string texte = String.Format("genCardinalityPropertyShape el={0} atr={1} pred={2}", elem.Name, atr.Name, tt);
            RDFLAutilitaires.wtest("TEST", texte);
            /*********************************************/
            try
            {
                XmlNode Element;
                XmlAttribute AttributeXml;
                XmlNode anode;
                //--------------------------- classifier
                bool compound = false;
                EA.Element classifier = Main.Repo.GetElementByID((int)atr.ClassifierID);
                if(classifier.StereotypeEx.Contains("Compound"))
                {
                    compound = true;
                }
                //-------------------------
                string lowerbound = atr.LowerBound;
                string upperbound = atr.UpperBound;
                int intlowerbound = Int32.Parse(lowerbound);
                int intupperbound;
                if ((upperbound == "*") || (upperbound == "*"))
                {
                    intupperbound = 0;
                }
                else
                {
                    intupperbound = Int32.Parse(upperbound);
                }
                clefcard = elem.Name + "." + atr.Name + "-cardinality";
                if (!cardshapenoddic.ContainsKey(clefcard))
                {
                    XmlElement rootelement = shacldoc.DocumentElement;
                     Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                     AttributeXml = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#" + elem.Name + "." + atr.Name + "-cardinality";
                    Element.Attributes.Append(AttributeXml);
                    XmlNode shapenode = rootelement.AppendChild(Element);
                    cardshapenoddic[clefcard] = shapenode;

                    Element = shacldoc.CreateElement("rdf:type", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "PropertyShape";
                    Element.Attributes.Append(AttributeXml);
                    anode = cardshapenoddic[clefcard].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:group", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "#CardinalityGroup";
                    Element.Attributes.Append(AttributeXml);
                    anode = cardshapenoddic[clefcard].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:order", RDFLAutilitaires.namespaces["sh"]);
                    int order = ordersdic["Cardinality"];
                    ordersdic["Cardinality"] = order + 1;
                    AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                    Element.Attributes.Append(AttributeXml);
                    Element.InnerText = order.ToString();
                    anode = cardshapenoddic[clefcard].AppendChild(Element);
                    /*
                    if (intlowerbound > 0)
                    {
                        Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = intlowerbound.ToString();
                        anode = cardshapenoddic[clefcard].AppendChild(Element);
                    }
                    if (intupperbound > 0)
                    {
                        Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = intupperbound.ToString();
                        anode = cardshapenoddic[clefcard].AppendChild(Element);
                    }
                    */

                    Element = shacldoc.CreateElement("sh:message", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = getCardinalityShapeMessage("attribute", intlowerbound, intupperbound, atr.Name);
                    anode = cardshapenoddic[clefcard].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:description", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = XMLP.GetXmlValueShaclData("descriptionAttributeCardinality");
                    anode = cardshapenoddic[clefcard].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:severity", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = RDFLAutilitaires.namespaces["sh"] + "Violation";
                    Element.Attributes.Append(AttributeXml);
                    anode = cardshapenoddic[clefcard].AppendChild(Element);

                    Element = shacldoc.CreateElement("sh:name", RDFLAutilitaires.namespaces["sh"]);
                    Element.InnerText = elem.Name + "." + atr.Name + "-cardinality";
                    anode = cardshapenoddic[clefcard].AppendChild(Element);
                }
                if (!compound)
                {

                    if (predecessors.Count > 0)
                    {
                        // string nodekind = "http://www.w3.org/ns/shacl#Literal";
                        // string datatype = RDFLAutilitaires.namespaces["xsd"] + classifier.Name.ToLower();
                        //string classname = RDFLAutilitaires.namespaces["cim"] + elem.Name;
                        string classuri = getPathURI(elem);
                        // string classname = classuri + elem.Name;
                         Tuple<int,int> bounds = new Tuple<int,int>(intlowerbound, intupperbound);
                        generCardinalityPath(cardshapenoddic[clefcard], elem.Name +"."+atr.Name, "", bounds, predecessors,classuri);
                    }
                    else
                    {
                        Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        //AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + elem.Name + "." + atr.Name;
                        AttributeXml.Value = getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                        Element.Attributes.Append(AttributeXml);
                        anode = cardshapenoddic[clefcard].AppendChild(Element);

                        if (intlowerbound > 0)
                        {
                            Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                            Element.Attributes.Append(AttributeXml);
                            Element.InnerText = intlowerbound.ToString();
                            anode = cardshapenoddic[clefcard].AppendChild(Element);
                        }
                        if (intupperbound > 0)
                        {
                            Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                            Element.Attributes.Append(AttributeXml);
                            Element.InnerText = intupperbound.ToString();
                            anode = cardshapenoddic[clefcard].AppendChild(Element);
                        }
                    }
                }
                else //this is a compound
                {

                    if (predecessors.Count > 0)
                    {
                        string nodekind = RDFLAutilitaires.namespaces["sh"] + "BlankNode";
                        Tuple<int, int> bounds = new Tuple<int, int>(intlowerbound, intupperbound);
                        generCardinalityPath(cardshapenoddic[clefcard], elem.Name + "." + atr.Name, nodekind,bounds , predecessors, getPathtURI(elem, atr));
                    }
                    else
                    {
                        

                        Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = getPathtURI(elem,atr) + elem.Name + "." + atr.Name;
                        Element.Attributes.Append(AttributeXml);
                        anode = cardshapenoddic[clefcard].AppendChild(Element);

                        if (intlowerbound > 0)
                        {
                            Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                            Element.Attributes.Append(AttributeXml);
                            Element.InnerText = intlowerbound.ToString();
                            anode = cardshapenoddic[clefcard].AppendChild(Element);
                        }
                        if (intupperbound > 0)
                        {
                            Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                            AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                            AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                            Element.Attributes.Append(AttributeXml);
                            Element.InnerText = intupperbound.ToString();
                            anode = cardshapenoddic[clefcard].AppendChild(Element);
                        }
                    }
                    // add cascades
                   
                    if (predecessors.Count == 0) originElement = elem;

                    string predecessor = getPathtURI(elem, atr) + elem.Name + "." + atr.Name;
                    if(!predecessors.Contains(predecessor)) predecessors.Add(predecessor);
                    // a ce niveau on se branche sur une nouvelle branche de compound
                    // il faut creer le point d'entrée    
                    cascade(elem, atr, classifier, "-cardinality", predecessors);
                }
            }
            catch (Exception ee)
            {
                XMLP.AddXmlLog("genCardinalityPropertyShape", String.Format(" {0}", ee.Message));
                throw new Exception(String.Format(" genCardinalityPropertyShape  {0} ", ee.Message));
            }
            return res;
        }


        string getCardinalityShapeMessage(string type,int intlowerbound,int intupperbound, string atrname)
        {
           
            string confmessage = "";
            string[] messages;
            string upperbound = intupperbound.ToString();
            string lowerbound = intlowerbound.ToString();
            bool isAttribute = false; // isAttribute is true if mmessage for attrinutes, false for associations
            if (type == "attribute") isAttribute = true;
            if(isAttribute)
            {
                messages = XMLP.GetXmlValueShaclData("messageAttributeCardinality").Split(new char[] { '|' });
            }
            else
            {
                messages = XMLP.GetXmlValueShaclData("messageAssociationCardinality").Split(new char[] { '|' });
            }
            if (intupperbound > 0)
            {
                confmessage = messages[0];
                if (intlowerbound > 0)
                {
                    confmessage = messages[2];
                    if(isAttribute) confmessage = confmessage.Replace("attribute", atrname);
                }
                confmessage = confmessage.Replace("1", upperbound);
            }
            else
            {
                if (intlowerbound > 0)
                {
                    confmessage = messages[1];
                    confmessage = confmessage.Replace("attribute", atrname);
                }
                else
                {
                    confmessage = "no constraints";
                }

            }
            return confmessage;
        }
        
       
        /// <summary>
        /// < sh:and rdf:parseType = "Collection" >
   ///< rdf:Description rdf:nodeID = "abc" >

  ///  < sh:nodeKind rdf:resource = "http://www.w3.org/ns/shacl#BlankNode" />
 
    /// < sh:class rdf : resource="http://iec.ch/TC57/CIM100#StreetDetail"/>
	///			<rdf:Description rdf:about="http://iec.ch/TC57/CIM100#Location.mainAddress" />
	///		  <rdf:Description rdf:about="http://iec.ch/TC57/CIM100#StreetAddress.streetDetail" />
	///	  </sh:path>
    ///  </rdf:Description>
     /// <rdf:Description rdf:nodeID="xyz">
     ///  <sh:nodeKind rdf:resource="http://www.w3.org/ns/shacl#BlankNode"/>
      ///  <sh:class rdf : resource="http://iec.ch/TC57/CIM100#StreetDetail"/>
		///	<sh:path rdf:parseType="Collection">
		///		<rdf:Description rdf:about="http://iec.ch/TC57/CIM100#Location.secondaryAddress" />
		///	  <rdf:Description rdf:about="http://iec.ch/TC57/CIM100#StreetAddress.streetDetail" />
     /// </sh:path>
     /// </rdf:Description>
   /// </sh:and>
        /// </summary>
        /// <param name="atrname"></param>
        /// <param name="nodekind"></param>
        /// <param name="classname"></param>
        /// <param name="predecessors"></param>
        void generPath(XmlNode shapenode,string atrname,string nodekind,string classname,List<string> predecessors,string uri)
        {
            RDFLAutilitaires.wtest("TEST", String.Format("generPath {0}", shapenode.ToString()));
            try
            {
               
                List<string> locallist = new List<string>();
                locallist.AddRange(predecessors);

                //locallist.Add(RDFLAutilitaires.namespaces["cim"] + atrname);
                if ((uri == "#") || (uri==RDFLAutilitaires.namespaces["cim"]))
                {
                    locallist.Add(RDFLAutilitaires.namespaces["cim"] + atrname);
                }
                else
                {
                    locallist.Add(uri + atrname);
                }

                XmlAttribute AttributeXml;
                XmlElement Element;
                XmlNode anode;
                if (!compounddic.ContainsKey(atrname+"-datatype"))
                {
                    Element = shacldoc.CreateElement("sh:and", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "Collection";
                    Element.Attributes.Append(AttributeXml);
                    compounddic.Add(atrname+"-datatype", shapenode.AppendChild(Element));
                }
                

                Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:nodeID", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "N" + compteur.ToString();
                compteur = compteur+1;
                Element.Attributes.Append(AttributeXml);
                anode = compounddic[atrname+ "-datatype"].AppendChild(Element);
                    Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = nodekind;
                    Element.Attributes.Append(AttributeXml);
                    XmlNode nn = anode.AppendChild(Element);
                    if((classname !="")&&(nodekind.Contains("BlankNode")))
                    {
                         Element = shacldoc.CreateElement("sh:class", RDFLAutilitaires.namespaces["sh"]);
                         AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    //AttributeXml.Value =RDFLAutilitaires.namespaces["cim"]+ classname;
                        if ((uri == "#") || (uri == RDFLAutilitaires.namespaces["cim"]))
                        {
                             AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + classname;
                        }
                        else
                        {
                                 AttributeXml.Value = uri + classname;
                        }
                        Element.Attributes.Append(AttributeXml);
                        XmlNode  nnn = anode.AppendChild(Element);
                                        }
                    else
                    {
                        Element = shacldoc.CreateElement("sh:datatype", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value =  classname;
                        Element.Attributes.Append(AttributeXml);
                        nn = anode.AppendChild(Element);
                    }
                     Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                     AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                     AttributeXml.Value = "Collection";
                     Element.Attributes.Append(AttributeXml);
                     XmlNode seqnode = anode.AppendChild(Element);

                     foreach (string link in locallist)
                     {
                        XmlElement ilel = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                        XmlAttribute liat = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    //liat.Value = RDFLAutilitaires.namespaces["cim"] + link;
                        liat.Value = link;
                        ilel.Attributes.Append(liat);
                        XmlNode node = seqnode.AppendChild(ilel);
                     }
            }
            catch(Exception ee)
            {
                throw new Exception("generpath  " + ee.Message);
            }

        }
        void generCardinalityPath(XmlNode shapenode, string atrname, string nodekind, Tuple<int,int> bounds, List<string> predecessors,string uri)
        {
            /* ABA20230905
            if (atrname== "IdentifiedObject" + "." +"mRID")
            {
                string ff = "";
            }*/
            try
            {
                RDFLAutilitaires.wtest("TEST", String.Format("generCardinalityPath {0}", shapenode.ToString()));

               List<string> locallist = new List<string>();
                locallist.AddRange(predecessors);
                string ns;
                if ((uri == "#") || (uri == RDFLAutilitaires.namespaces["cim"]))
                {
                    ns = RDFLAutilitaires.namespaces["cim"];
                }
                else
                {
                    ns = uri;
                }
                //locallist.Add(RDFLAutilitaires.namespaces["cim"] + atrname);
                locallist.Add(ns + atrname);
                XmlAttribute AttributeXml;
                XmlElement Element;
                XmlNode anode;
                if (!compounddic.ContainsKey(atrname+ "-cardinality"))
                {
                    Element = shacldoc.CreateElement("sh:and", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = "Collection";
                    Element.Attributes.Append(AttributeXml);
                    compounddic.Add(atrname+"-cardinality", shapenode.AppendChild(Element));
                }


                Element = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:nodeID", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "N" + compteur.ToString();
                compteur = compteur + 1;
                Element.Attributes.Append(AttributeXml);
                anode = compounddic[atrname+"-cardinality"].AppendChild(Element);
                XmlNode nn;
                if (nodekind != "")
                {
                    Element = shacldoc.CreateElement("sh:nodeKind", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = nodekind;
                    Element.Attributes.Append(AttributeXml);
                    nn = anode.AppendChild(Element);
                }
                else
                {
                    int lower = bounds.Item1;
                    int upper = bounds.Item2;
                    if (lower > 0)
                    {
                        Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = lower.ToString();
                        nn = anode.AppendChild(Element);
                    }
                    if (upper > 0)
                    {
                        Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = upper.ToString();
                        nn = anode.AppendChild(Element);
                    }
                }
                
                if (nodekind.Contains("BlankNode"))
                {
                    int lower = bounds.Item1;
                    int upper = bounds.Item2;
                    if (lower > 0)
                    {
                        Element = shacldoc.CreateElement("sh:minCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = lower.ToString();
                        nn = anode.AppendChild(Element);
                    }
                    if (upper > 0)
                    {
                        Element = shacldoc.CreateElement("sh:maxCount", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:datatype", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = "http://www.w3.org/2001/XMLSchema#integer";
                        Element.Attributes.Append(AttributeXml);
                        Element.InnerText = upper.ToString();
                        nn = anode.AppendChild(Element);
                    }
                    /*
                        Element = shacldoc.CreateElement("sh:class", RDFLAutilitaires.namespaces["sh"]);
                        AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        AttributeXml.Value = RDFLAutilitaires.namespaces["cim"] + classname;
                        Element.Attributes.Append(AttributeXml);
                        nn = anode.AppendChild(Element);
                    */

                }
                /*
                else
                {
                    Element = shacldoc.CreateElement("sh:datatype", RDFLAutilitaires.namespaces["sh"]);
                    AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                    AttributeXml.Value = classname;
                    Element.Attributes.Append(AttributeXml);
                    nn = anode.AppendChild(Element);
                }
                */
                Element = shacldoc.CreateElement("sh:path", RDFLAutilitaires.namespaces["sh"]);
                AttributeXml = shacldoc.CreateAttribute("rdf:parseType", RDFLAutilitaires.namespaces["rdf"]);
                AttributeXml.Value = "Collection";
                Element.Attributes.Append(AttributeXml);
                XmlNode seqnode = anode.AppendChild(Element);
                foreach (string link in locallist)
                {
                    XmlElement ilel = shacldoc.CreateElement("rdf:Description", RDFLAutilitaires.namespaces["rdf"]);
                    XmlAttribute liat = shacldoc.CreateAttribute("rdf:about", RDFLAutilitaires.namespaces["rdf"]);
                    // liat.Value = RDFLAutilitaires.namespaces["cim"] + link;
                    liat.Value = link;
                    ilel.Attributes.Append(liat);
                    XmlNode node = seqnode.AppendChild(ilel);
                }
            }
            catch (Exception ee)
            {
                throw new Exception("generpath  " + ee.Message);
            }

        }
        //--------------------------------------------------------------
        public string getPathURI(EA.Element elt)
        {
            string prov = elt.Name;
            string ret = "";
            try
            { 
                // XMLParser xmlp = new XMLParser();
                XMLParser xmlp = Main.XMLP;
           
                string stereoex = elt.StereotypeEx;
                string liststereos = xmlp.GetXmlValueProfData("ListStereoNamespace");
                string uridebase = profparams.ProfileNamespace.Trim('#') + "/Constraints#";
                string  cimuri = RDFLAutilitaires.namespaces["cim"];
                string eligible = "";

                string[] splitns = liststereos.Split('|');
                for (int i = 0; i < splitns.Length; i++)
                {

                    if (splitns[i].Contains(","))
                    {
                        eligible = splitns[i];
                        string[] split = splitns[i].Split(',');
                        for (int j = 0; j < split.Length; j++)
                        {
                            if (!RDFLAutilitaires.StereoExContainsStereo(stereoex, split[j]))
                            {
                                eligible = "";
                                break;
                            }

                        }
                    }
                    else
                    {
                        if (RDFLAutilitaires.StereoExContainsStereo(stereoex, splitns[i]))
                        {
                            eligible = splitns[i];
                            break;
                        }

                    }

                }
                if (eligible != "")
                {// on a trouve un stereotype qui convient
                    string[] prefuri = profparams.urisbystereo[eligible];
                     ret = prefuri[1]; // +"#";           
                }
                else
                {
                    ret = cimuri;
                }
 
            }
            catch( Exception ee)
            {
                throw new Exception(String.Format(" getPathUri elt={0} err={1}", elt.Name, ee.Message));
            }
            return ret;
        }
        /// <summary>
        /// recherche le bon URI pour un attribut
        /// attention le choix aujourd'hui est de privilegier
        /// l'extension sur la classe a l'extension sur l'attribut
        /// </summary>
        /// <param name="elt"></param>
        /// <param name="atr"></param>
        /// <returns></returns>
        string getPathtURI(EA.Element elt, EA.Attribute atr)
        {
            string ret = "";
            try
            {
                
                string prov = elt.Name;
                string prov1 = atr.Name;
                XMLParser xmlp = new XMLParser(Main.Repo);
                string stereoex = elt.StereotypeEx;
                string atrstereoex = atr.StereotypeEx;
                string liststereos = xmlp.GetXmlValueProfData("ListStereoNamespace");
                string uridebase = profparams.ProfileNamespace.Trim('#') + "/Constraints#";
                string cimuri = RDFLAutilitaires.namespaces["cim"];
                string eligible = "";
                string[] splitns = liststereos.Split('|');
                for (int i = 0; i < splitns.Length; i++)
                {

                    if (splitns[i].Contains(","))
                    {
                        eligible = splitns[i];
                        string[] split = splitns[i].Split(',');
                        for (int j = 0; j < split.Length; j++)
                        {
                            if (!RDFLAutilitaires.StereoExContainsStereo(stereoex, split[j]) && !RDFLAutilitaires.StereoExContainsStereo(atrstereoex, split[j]))
                            {
                                eligible = "";
                                break;
                            }
                        }
                    }
                    else
                    {
                        if (RDFLAutilitaires.StereoExContainsStereo(stereoex, splitns[i]) || RDFLAutilitaires.StereoExContainsStereo(atrstereoex, splitns[i]))
                        {
                            eligible = splitns[i];
                            break;
                        }
                    }
                }
                if (eligible != "")
                {// on a trouve un stereotype qui convient
                    string[] prefuri = profparams.urisbystereo[eligible];
                    ret = prefuri[1]; //+ "#";
                }
                else
                {
                     ret = cimuri;
                }
              }catch(Exception ee)
            {
                throw new Exception(String.Format(" getPathURI elt={0} err={1}",elt.Name, ee.Message));
            }
     
            return ret;
        }

        public string gePathtURI(EA.Package pack)
        {
            string ret = "";
            try { 
            XMLParser xmlp = new XMLParser(Main.Repo);
            
                
             ret = profparams.ProfileNamespace.Trim('#') + "/Constraints#" ;
                
        }catch(Exception ee)
            {
                throw new Exception(String.Format(" getPathURI pack={0} err={1}", pack.Name, ee.Message));
    }
            return ret;
        }
       
        /// <summary>
        /// will add the missing pcompound properties to descendants of previous treated classes
        /// </summary>
        void addPropertiesToDescendants()
        {
            XmlNode shapenode;
            XmlNode originshapenode;
            foreach(EA.Element origin in originElementList)
            {
                originshapenode = shapenodedic[origin.Name];
                List<EA.Element> descendants = new List<EA.Element>();
                RDFLAutilitaires.getAllDescendantsInProfile(origin, descendants);
                foreach (EA.Element descendant in descendants)
                {
                    if (descendant.Abstract == "1") continue;
                    shapenode = shapenodedic[descendant.Name];
                    List<string> compoundnames = compoundsdonedic[originElement];
                    foreach(string compoundname in compoundnames)
                    {
                        XmlElement Element = shacldoc.CreateElement("sh:property", RDFLAutilitaires.namespaces["sh"]);
                        XmlAttribute AttributeXml = shacldoc.CreateAttribute("rdf:resource", RDFLAutilitaires.namespaces["rdf"]);
                        //AttributeXml.Value = "#" + classifier.Name + "." + at.Name + "-datatype";
                        AttributeXml.Value = "#" + compoundname ;
                        Element.Attributes.Append(AttributeXml);
                        XmlNode node = shapenode.AppendChild(Element);
                    }
                }
            }
        }
        //------------------------------------------------------------------
    }
}
