﻿/////////////////////////////////////////////////////////////////////////////////////////
// Author: Alexander Balka
// File: Logger.cs
/////////////////////////////////////////////////////////////////////////////////////////


using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CimSyntaxGen.Utilities
{
    public class Logger
    {
        private string fileName = null;
        private bool on = false;
        public Logger(string fileName, bool on)
        {
            this.fileName = fileName;
            this.on = on;
        }

        public bool On()
        {
            return on;
        }

        public void Log(string msg)
        {
            if (!on) return;
            try
            {
                using (StreamWriter w = System.IO.File.AppendText(fileName))
                {
                    w.Write("|> " + msg + "\n");
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
        }

        public String GetTimestamp()
        {
            DateTime value = DateTime.Now;
            return value.ToString("yyyyMMdd HH:mm:ss.ffff");
        }


        public void LogArray(string[] strings)
        {
            if (!on) return;
            string listStr = strings[0];
            for(int i = 1; i < strings.Length; ++i)
            {
                listStr += ", " + strings[i];
            }
            Log(listStr);
        }

        public void LogArray(long[] arr)
        {
            if (!on) return;
            if (arr.Length <= 0) return;
            string listStr = arr[0].ToString();
            for (int i = 1; i < arr.Length; ++i)
            {
                listStr += ", " + arr[i];
            }
            Log(listStr);
        }

        public void LogListOfArray(string msg, List<long[]> lList)
        {
            if (!on) return;
            Log(msg);
            foreach (long[] item in lList)
            {
                string listStr = item[0].ToString();
                for (int i = 1; i < item.Length; ++i)
                {
                    listStr += ", " + item[i];
                }
                Log("\t" + listStr);
            }
        }


        public static void LogFile(string fileName, string content)
        {
            Stream fileStream = new FileStream(fileName, FileMode.Create, FileAccess.Write, FileShare.None);
            if (fileStream != null)
            {
                byte[] info = new UTF8Encoding(true).GetBytes(content);
                fileStream.Write(info, 0, info.Length);
                fileStream.Flush();
                fileStream.Close();
            }
        }

    }
}
